@@ -10,27 +10,12 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	24 May 1993
- * Last Edited:	4 September 1999
- *
- * Copyright 1999 by the University of Washington
- *
- *  Permission to use, copy, modify, and distribute this software and its
- * documentation for any purpose and without fee is hereby granted, provided
- * that the above copyright notice appears in all copies and that both the
- * above copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the University of Washington not be
- * used in advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  This software is made
- * available "as is", and
- * THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
- * WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
- * NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
- * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
- * (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
+ * Last Edited:	26 April 2001
+ * 
+ * The IMAP toolkit provided in this Distribution is
+ * Copyright 2001 University of Washington.
+ * The full text of our legal notices is contained in the file called
+ * CPYRIGHT, included with this Distribution.
  */
 
 
@@ -65,7 +50,7 @@ DRIVER dummydriver = {
   dummy_create,			/* create mailbox */
   dummy_delete,			/* delete mailbox */
   dummy_rename,			/* rename mailbox */
-  NIL,				/* status of mailbox */
+  mail_status_default,		/* status of mailbox */
   dummy_open,			/* open mailbox */
   dummy_close,			/* close mailbox */
   NIL,				/* fetch message "fast" attributes */
@@ -104,10 +89,18 @@ DRIVER *dummy_valid (char *name)
   char *s,tmp[MAILTMPLEN];
   struct stat sbuf;
 				/* must be valid local mailbox */
-  if (!(name && *name && (*name != '{'))) return NIL;
-				/* no trailing \ */
-  if ((s = strrchr (mailboxfile (tmp,name),'\\')) && !s[1]) *s = '\0';
-  return (!tmp[0] || !stat (tmp,&sbuf)) ? &dummydriver : NIL;
+  if (name && *name && (*name != '{') && (s = mailboxfile (tmp,name))) {
+				/* indeterminate INBOX */
+    if (!*s) return &dummydriver;
+				/* remove trailing \ */
+    if ((s = strrchr (s,'\\')) && !s[1]) *s = '\0';
+    else if (!stat (s,&sbuf)) switch (sbuf.st_mode & S_IFMT) {
+    case S_IFREG:		/* file */
+    case S_IFDIR:		/* future use */
+      return &dummydriver;
+    }
+  }
+  return NIL;
 }
 
 
@@ -119,7 +112,7 @@ DRIVER *dummy_valid (char *name)
 
 void *dummy_parameters (long function,void *value)
 {
-  return value;
+  return NIL;
 }
 
 /* Dummy scan mailboxes
@@ -185,19 +178,18 @@ void dummy_list (MAILSTREAM *stream,char *ref,char *pat)
 void dummy_lsub (MAILSTREAM *stream,char *ref,char *pat)
 {
   void *sdb = NIL;
-  char *s,*t,test[MAILTMPLEN],tmp[MAILTMPLEN];
+  char *s,*t,test[MAILTMPLEN];
   int showuppers = pat[strlen (pat) - 1] == '%';
 				/* get canonical form of name */
   if (dummy_canonicalize (test,ref,pat) && (s = sm_read (&sdb))) do
     if (*s != '{') {
-      if (pmatch_full (ucase (strcpy (tmp,s)),test,'\\')) {
-	if (pmatch (tmp,"INBOX")) mm_lsub (stream,NIL,s,LATT_NOINFERIORS);
+      if (pmatch_full (s,test,'\\')) {
+	if (pmatch (s,"INBOX")) mm_lsub (stream,NIL,s,LATT_NOINFERIORS);
 	else mm_lsub (stream,'\\',s,NIL);
       }
       else while (showuppers && (t = strrchr (s,'\\'))) {
 	*t = '\0';		/* tie off the name */
-	if (pmatch_full (ucase (strcpy (tmp,s)),test,'\\'))
-	  mm_lsub (stream,'\\',s,LATT_NOSELECT);
+	if (pmatch_full (s,test,'\\')) mm_lsub (stream,'\\',s,LATT_NOSELECT);
       }
     }
   while (s = sm_read (&sdb));	/* until no more subscriptions */
@@ -236,63 +228,49 @@ void dummy_list_work (MAILSTREAM *stream,char *dir,char *pat,char *contents,
   struct _finddata_t f;
   struct stat sbuf;
   long fhandle;
-  char tmp[MAILTMPLEN],tmpx[MAILTMPLEN],tmpy[MAILTMPLEN];
-  char *base = (dir && ((dir[0] == '\\') || (dir[1] == ':'))) ?
-    NIL : myhomedir ();
-				/* build name */
-  if (base) sprintf (tmpx,"%s\\",base);
-  else tmpx[0] = '\0';
-  if (dir) strcat (tmpx,dir);
+  char tmp[MAILTMPLEN];
 				/* punt if bogus name */
-  if (!mailboxfile (tmp,tmpx)) return;
+  if (!mailboxdir (tmp,dir,NIL)) return;
 				/* make directory wildcard */
   strcat (tmp,(tmp[strlen (tmp) -1] == '\\') ? "*.*" : "\\*.*");
-
 				/* do nothing if can't open directory */
   if ((fhandle = _findfirst (tmp,&f)) >= 0) {  
 				/* list it if not at top-level */
-    if (!level && dir && pmatch_full (ucase (strcpy (tmpy,dir)),pat,'\\'))
+    if (!level && dir && pmatch_full (dir,pat,'\\'))
       dummy_listed (stream,'\\',dir,LATT_NOSELECT,contents);
 				/* scan directory */
-    if (tmpx[strlen (tmpx) - 1] == '\\') do
-      if ((f.name[0] != '.') ||
-	  (f.name[1] && ((f.name[1] != '.') || f.name[2]))) {
-	if (base) sprintf (tmpx,"%s\\",base);
-	else tmpx[0] = '\0';
-	if (dir) sprintf (tmpx + strlen (tmpx),"%s%s",dir,f.name);
-	else strcat (tmpx,f.name);
-
+    if (!dir || dir[strlen (dir) -1] == '\\') do
+      if (((f.name[0] != '.') ||
+	   (f.name[1] && ((f.name[1] != '.') || f.name[2]))) &&
+	  (strlen (f.name) <= NETMAXMBX)) {
 				/* see if name is useful */
 	if (dir) sprintf (tmp,"%s%s",dir,f.name);
 	else strcpy (tmp,f.name);
 				/* make sure useful and can get info */
-	if ((pmatch_full (ucase (tmp),pat,'\\') ||
-	     pmatch_full (strcat (tmp,"\\"),pat,'\\') || dmatch (tmp,pat,'\\'))
-	    && !stat (mailboxfile (tmp,tmpx),&sbuf)) {
+	if ((pmatch_full (tmp,pat,'\\') ||
+	     pmatch_full (strcat (tmp,"\\"),pat,'\\') ||
+	     dmatch (tmp,pat,'\\')) &&
+	    mailboxdir (tmp,dir,f.name) && tmp[0] && !stat (tmp,&sbuf)) {
 				/* now make name we'd return */
 	  if (dir) sprintf (tmp,"%s%s",dir,f.name);
 	  else strcpy (tmp,f.name);
 				/* only interested in file type */
-	  switch (sbuf.st_mode &= S_IFMT) {
+	  switch (sbuf.st_mode & S_IFMT) {
 	  case S_IFDIR:		/* directory? */
-	    if (pmatch_full (ucase (strcpy (tmpy,tmp)),pat,'\\')) {
-	      dummy_listed (stream,'\\',tmp,LATT_NOSELECT,contents);
+	    if (pmatch_full (tmp,pat,'\\')) {
+	      if (!dummy_listed (stream,'\\',tmp,LATT_NOSELECT,contents))break;
 	      strcat (tmp,"\\");/* set up for dmatch call */
-	      strcat (tmpy,"\\");
 	    }
-	    else {		/* try again with trailing / */
-	      strcat (tmp,"\\");
-	      strcat (tmpy,"\\");
-	      if (pmatch_full (tmpy,pat,'\\'))
-		dummy_listed (stream,'\\',tmp,LATT_NOSELECT,contents);
-	    }
-	    if (dmatch (tmpy,pat,'\\') &&
+				/* try again with trailing \ */
+	    else if (pmatch_full (strcat (tmp,"\\"),pat,'\\') &&
+		     !dummy_listed (stream,'\\',tmp,LATT_NOSELECT,contents))
+	      break;
+	    if (dmatch (tmp,pat,'\\') &&
 		(level < (long) mail_parameters (NIL,GET_LISTMAXLEVEL,NIL)))
 	      dummy_list_work (stream,tmp,pat,contents,level+1);
 	    break;
 	  case S_IFREG:		/* ordinary name */
-	    if (pmatch_full (ucase (strcpy (tmpy,tmp)),pat,'\\') &&
-		!pmatch ("INBOX",tmpy))
+	    if (pmatch_full (tmp,pat,'\\') && !pmatch ("INBOX",tmp))
 	      dummy_listed (stream,'\\',tmp,LATT_NOINFERIORS,contents);
 	    break;
 	  }
@@ -353,8 +331,8 @@ long dummy_listed (MAILSTREAM *stream,char delimiter,char *name,
 long dummy_create (MAILSTREAM *stream,char *mailbox)
 {
   char tmp[MAILTMPLEN];
-  if (strcmp (ucase (strcpy (tmp,mailbox)),"INBOX") && dummy_file(tmp,mailbox))
-    return dummy_create_path (stream,tmp);
+  if (compare_cstring (mailbox,"INBOX") && dummy_file (tmp,mailbox))
+    return dummy_create_path (stream,tmp,NIL);
   sprintf (tmp,"Can't create %s: invalid name",mailbox);
   mm_log (tmp,ERROR);
   return NIL;
@@ -363,11 +341,12 @@ long dummy_create (MAILSTREAM *stream,char *mailbox)
 
 /* Dummy create path
  * Accepts: mail stream
- *	    path name name to create
+ *	    path name to create
+ *	    directory mode
  * Returns: T on success, NIL on failure
  */
 
-long dummy_create_path (MAILSTREAM *stream,char *path)
+long dummy_create_path (MAILSTREAM *stream,char *path,long dirmode)
 {
   struct stat sbuf;
   char c,*s,tmp[MAILTMPLEN];
@@ -385,7 +364,7 @@ long dummy_create_path (MAILSTREAM *stream,char *path)
     *s = '\0';
 				/* name doesn't exist, create it */
     if ((stat (tmp,&sbuf) || ((sbuf.st_mode & S_IFMT) != S_IFDIR)) &&
-	!dummy_create_path (stream,path)) return NIL;
+	!dummy_create_path (stream,path,dirmode)) return NIL;
     *s = c;			/* restore full name */
   }
   if (wantdir) {		/* want to create directory? */
@@ -455,7 +434,7 @@ long dummy_rename (MAILSTREAM *stream,char *old,char *newname)
     s[1] = c;			/* restore character after delimiter */
   }
 				/* rename of non-ex INBOX creates dest */
-  if (!strcmp (ucase (strcpy (tmp,old)),"INBOX") &&
+  if (!compare_cstring (old,"INBOX") &&
       stat (dummy_file (tmp,old),&sbuf)) return dummy_create (NIL,mbx);
   if (rename (dummy_file (tmp,old),mbx)) {
     sprintf (tmp,"Can't rename mailbox %s to %s: %s",old,newname,
@@ -482,7 +461,7 @@ MAILSTREAM *dummy_open (MAILSTREAM *stream)
 				/* can we open the file? */
   if ((fd = open (dummy_file (tmp,stream->mailbox),O_RDONLY,NIL)) < 0) {
 				/* no, error unless INBOX */
-    if (strcmp (ucase (strcpy (tmp,stream->mailbox)),"INBOX"))
+    if (compare_cstring (stream->mailbox,"INBOX"))
       sprintf (err,"%s: %s",strerror (errno),stream->mailbox);
   }
   else {			/* file had better be empty then */
@@ -539,6 +518,8 @@ long dummy_ping (MAILSTREAM *stream)
 				/* swap the streams */
     memcpy (stream,test,sizeof (MAILSTREAM));
     fs_give ((void **) &test);	/* flush test now that copied */
+				/* make sure application knows */
+    mail_exists (stream,stream->recent = stream->nmsgs);
   }
   return T;
 }
@@ -583,22 +564,20 @@ long dummy_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 /* Dummy append message string
  * Accepts: mail stream
  *	    destination mailbox
- *	    optional flags
- *	    optional date
- *	    stringstruct of message to append
+ *	    append callback function
+ *	    data for callback
  * Returns: T on success, NIL on failure
  */
 
-long dummy_append (MAILSTREAM *stream,char *mailbox,char *flags,char *date,
-		   STRING *message)
+long dummy_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
 {
   struct stat sbuf;
   int fd = -1;
   int e;
   char tmp[MAILTMPLEN];
   MAILSTREAM *ts = default_proto (T);
-  if ((strcmp (ucase (strcpy (tmp,mailbox)),"INBOX")) &&
-	   ((fd = open (dummy_file (tmp,mailbox),O_RDONLY,NIL)) < 0)) {
+  if (compare_cstring (mailbox,"INBOX") &&
+      ((fd = open (dummy_file (tmp,mailbox),O_RDONLY,NIL)) < 0)) {
     if ((e = errno) == ENOENT)	/* failed, was it no such file? */
       mm_notify (stream,"[TRYCREATE] Must create mailbox before append",
 		 (long) NIL);
@@ -611,7 +590,7 @@ long dummy_append (MAILSTREAM *stream,char *mailbox,char *flags,char *date,
     close (fd);			/* toss out the fd */
     if (sbuf.st_size) ts = NIL;	/* non-empty file? */
   }
-  if (ts) return (*ts->dtb->append) (stream,mailbox,flags,date,message);
+  if (ts) return (*ts->dtb->append) (stream,mailbox,af,data);
   sprintf (tmp,"Indeterminate mailbox format: %s",mailbox);
   mm_log (tmp,ERROR);
   return NIL;
