@@ -10,30 +10,22 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	22 May 1990
- * Last Edited:	21 September 1999
- *
- * Copyright 1999 by the University of Washington
- *
- *  Permission to use, copy, modify, and distribute this software and its
- * documentation for any purpose and without fee is hereby granted, provided
- * that the above copyright notice appears in all copies and that both the
- * above copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the University of Washington not be
- * used in advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  This software is made
- * available "as is", and
- * THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
- * WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
- * NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
- * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
- * (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
+ * Last Edited:	9 October 2001
+ * 
+ * The IMAP toolkit provided in this Distribution is
+ * Copyright 2001 University of Washington.
+ * The full text of our legal notices is contained in the file called
+ * CPYRIGHT, included with this Distribution.
  */
 
 
+/*				FILE TIME SEMANTICS
+ *
+ * The atime is the last read time of the file.
+ * The mtime is the last flags update time of the file.
+ * The ctime is the last write time of the file.
+ */
+
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
@@ -52,7 +44,8 @@ extern int errno;		/* just in case */
 
 DRIVER tenexdriver = {
   "tenex",			/* driver name */
-  DR_LOCAL|DR_MAIL|DR_NOSTICKY,	/* driver flags */
+  DR_LOCAL|DR_MAIL|DR_NOSTICKY|DR_LOCKING,
+				/* driver flags */
   (DRIVER *) NIL,		/* next driver */
   tenex_valid,			/* mailbox is valid for us */
   tenex_parameters,		/* manipulate parameters */
@@ -225,19 +218,19 @@ long tenex_rename (MAILSTREAM *stream,char *old,char *newname)
   struct stat sbuf;
   if (fd < 0) {			/* open mailbox */
     sprintf (tmp,"Can't open mailbox %.80s: %s",old,strerror (errno));
-    mm_log (tmp,ERROR);
+    MM_LOG (tmp,ERROR);
     return NIL;
   }
 				/* get exclusive parse/append permission */
   if ((ld = lockfd (fd,lock,LOCK_EX)) < 0) {
-    mm_log ("Unable to lock rename mailbox",ERROR);
+    MM_LOG ("Unable to lock rename mailbox",ERROR);
     return NIL;
   }
 				/* lock out other users */
   if (flock (fd,LOCK_EX|LOCK_NB)) {
     close (fd);			/* couldn't lock, give up on it then */
     sprintf (tmp,"Mailbox %.80s is in use by another process",old);
-    mm_log (tmp,ERROR);
+    MM_LOG (tmp,ERROR);
     unlockfd (ld,lock);		/* release exclusive parse/append permission */
     return NIL;
   }
@@ -246,7 +239,7 @@ long tenex_rename (MAILSTREAM *stream,char *old,char *newname)
     if (!((s = tenex_file (tmp,newname)) && *s)) {
       sprintf (tmp,"Can't rename mailbox %.80s to %.80s: invalid name",
 	       old,newname);
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       ret = NIL;		/* set failure */
     }
     if (s = strrchr (s,'/')) {	/* found superior to destination name? */
@@ -254,28 +247,28 @@ long tenex_rename (MAILSTREAM *stream,char *old,char *newname)
       *s = '\0';		/* tie off to get just superior */
 				/* name doesn't exist, create it */
       if ((stat (tmp,&sbuf) || ((sbuf.st_mode & S_IFMT) != S_IFDIR)) &&
-	  !dummy_create (stream,tmp)) ret = NIL;
+	  !dummy_create_path (stream,tmp,get_dir_protection (newname)))
+	ret = NIL;
       else *s = c;		/* restore full name */
     }
 				/* rename the file */
     if (ret && rename (file,tmp)) {
       sprintf (tmp,"Can't rename mailbox %.80s to %.80s: %s",old,newname,
 	       strerror (errno));
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       ret = NIL;		/* set failure */
     }
   }
   else if (unlink (file)) {
     sprintf (tmp,"Can't delete mailbox %.80s: %s",old,strerror (errno));
-    mm_log (tmp,ERROR);
+    MM_LOG (tmp,ERROR);
     ret = NIL;			/* set failure */
   }
   flock (fd,LOCK_UN);		/* release lock on the file */
   close (fd);			/* close the file */
   unlockfd (ld,lock);		/* release exclusive parse/append permission */
 				/* recreate file if renamed INBOX */
-  if (ret && !strcmp (ucase (strcpy (tmp,old)),"INBOX"))
-    dummy_create (NIL,"mail.txt");
+  if (ret && !compare_cstring (old,"INBOX")) dummy_create (NIL,"mail.txt");
   return ret;			/* return success */
 }
 
@@ -314,8 +307,7 @@ long tenex_status (MAILSTREAM *stream,char *mbx,long flags)
 				/* kludge but probably good enough */
     status.uidnext += systream->nmsgs;
   }
-				/* pass status to main program */
-  mm_status (stream,mbx,&status);
+  MM_STATUS(stream,mbx,&status);/* pass status to main program */
   if (tstream) mail_close (tstream);
   if (systream) mail_close (systream);
   return T;			/* success */
@@ -339,11 +331,11 @@ MAILSTREAM *tenex_open (MAILSTREAM *stream)
       (fd = open (tenex_file (tmp,stream->mailbox),O_RDWR,NIL)) < 0) {
     if ((fd = open (tenex_file (tmp,stream->mailbox),O_RDONLY,NIL)) < 0) {
       sprintf (tmp,"Can't open mailbox: %s",strerror (errno));
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       return NIL;
     }
     else if (!stream->rdonly) {	/* got it, but readonly */
-      mm_log ("Can't get write access to mailbox, access is readonly",WARN);
+      MM_LOG ("Can't get write access to mailbox, access is readonly",WARN);
       stream->rdonly = T;
     }
   }
@@ -352,18 +344,19 @@ MAILSTREAM *tenex_open (MAILSTREAM *stream)
   LOCAL->buflen = MAXMESSAGESIZE;
 
 				/* note if an INBOX or not */
-  stream->inbox = !strcmp(ucase (strcpy (LOCAL->buf,stream->mailbox)),"INBOX");
+  stream->inbox = !compare_cstring (stream->mailbox,"INBOX");
+  LOCAL->fd = fd;		/* bind the file */
 				/* flush old name */
   fs_give ((void **) &stream->mailbox);
 				/* save canonical name */
   stream->mailbox = cpystr (tmp);
 				/* get shared parse permission */
   if ((ld = lockfd (fd,tmp,LOCK_SH)) < 0) {
-    mm_log ("Unable to lock open mailbox",ERROR);
+    MM_LOG ("Unable to lock open mailbox",ERROR);
     return NIL;
   }
   (*bn) (BLOCK_FILELOCK,NIL);
-  flock(LOCAL->fd = fd,LOCK_SH);/* bind and lock the file */
+  flock (LOCAL->fd,LOCK_SH);	/* lock the file */
   (*bn) (BLOCK_NONE,NIL);
   unlockfd (ld,tmp);		/* release shared parse permission */
   LOCAL->filesize = 0;		/* initialize parsed file size */
@@ -374,7 +367,7 @@ MAILSTREAM *tenex_open (MAILSTREAM *stream)
 				/* parse mailbox */
   stream->nmsgs = stream->recent = 0;
   if (tenex_ping (stream) && !stream->nmsgs)
-    mm_log ("Mailbox is empty",(long) NIL);
+    MM_LOG ("Mailbox is empty",(long) NIL);
   if (!LOCAL) return NIL;	/* failure if stream died */
   stream->perm_seen = stream->perm_deleted =
     stream->perm_flagged = stream->perm_answered = stream->perm_draft =
@@ -453,13 +446,12 @@ void tenex_fast (MAILSTREAM *stream,char *sequence,long flags)
 
 void tenex_flags (MAILSTREAM *stream,char *sequence,long flags)
 {
-  MESSAGECACHE *elt;
   unsigned long i;
   if (stream && LOCAL &&
       ((flags & FT_UID) ? mail_uid_sequence (stream,sequence) :
        mail_sequence (stream,sequence)))
     for (i = 1; i <= stream->nmsgs; i++)
-      if ((elt = mail_elt (stream,i))->sequence) tenex_elt (stream,i);
+      if (mail_elt (stream,i)->sequence) tenex_elt (stream,i);
 }
 
 /* TENEX mail fetch message header
@@ -520,7 +512,7 @@ long tenex_text (MAILSTREAM *stream,unsigned long msgno,STRING *bs,long flags)
     elt->seen = T;		/* mark message as seen */
 				/* recalculate status */
     tenex_update_status (stream,msgno,T);
-    mm_flags (stream,msgno);
+    MM_FLAGS (stream,msgno);
   }
 				/* find header position */
   i = tenex_hdrpos (stream,msgno,&j);
@@ -556,11 +548,14 @@ long tenex_text (MAILSTREAM *stream,unsigned long msgno,STRING *bs,long flags)
 
 void tenex_flag (MAILSTREAM *stream,char *sequence,char *flag,long flags)
 {
+  time_t tp[2];
   struct stat sbuf;
   if (!stream->rdonly) {	/* make sure the update takes */
     fsync (LOCAL->fd);
     fstat (LOCAL->fd,&sbuf);	/* get current write time */
-    LOCAL->filetime = sbuf.st_mtime;
+    tp[1] = LOCAL->filetime = sbuf.st_mtime;
+    tp[0] = time (0);		/* make sure read comes after all that */
+    utime (stream->mailbox,tp);
   }
 }
 
@@ -602,7 +597,7 @@ long tenex_ping (MAILSTREAM *stream)
 				/* check for changed message status */
     if (LOCAL->mustcheck || LOCAL->shouldcheck) {
       if (LOCAL->shouldcheck)	/* babble when we do this unilaterally */
-	mm_notify (stream,"[CHECK] Checking for flag updates",NIL);
+	MM_NOTIFY (stream,"[CHECK] Checking for flag updates",NIL);
       while (i <= stream->nmsgs) tenex_elt (stream,i++);
       LOCAL->mustcheck = LOCAL->shouldcheck = NIL;
     }
@@ -625,12 +620,6 @@ long tenex_ping (MAILSTREAM *stream)
 	  unlockfd (ld,lock);	/* release shared parse/append permission */
 	}
       }
-      else if ((sbuf.st_ctime > sbuf.st_atime) ||
-	       (sbuf.st_ctime > sbuf.st_mtime)) {
-	time_t tp[2];		/* whack the times if necessary */
-	LOCAL->filetime = tp[0] = tp[1] = time (0);
-	utime (stream->mailbox,tp);
-      }
     }
   }
   return r;			/* return result of the parse */
@@ -645,7 +634,7 @@ void tenex_check (MAILSTREAM *stream)
 {
 				/* mark that a check is desired */
   if (LOCAL) LOCAL->mustcheck = T;
-  if (tenex_ping (stream)) mm_log ("Check completed",(long) NIL);
+  if (tenex_ping (stream)) MM_LOG ("Check completed",(long) NIL);
 }
 
 /* Tenex mail snarf messages from system inbox
@@ -665,7 +654,7 @@ void tenex_snarf (MAILSTREAM *stream)
   if ((time (0) < (LOCAL->lastsnarf + 30)) ||
       (!strcmp (sysinbox (),stream->mailbox)) ||
       ((ld = lockfd (LOCAL->fd,lock,LOCK_EX)) < 0)) return;
-  mm_critical (stream);		/* go critical */
+  MM_CRITICAL (stream);		/* go critical */
 				/* see if anything there */
   if (!stat (sysinbox (),&sbuf) && sbuf.st_size) {
     fstat (LOCAL->fd,&sbuf);	/* yes, get current file size */
@@ -705,7 +694,7 @@ void tenex_snarf (MAILSTREAM *stream)
       }
       else {
 	sprintf (LOCAL->buf,"Can't copy new mail: %s",strerror (errno));
-	mm_log (LOCAL->buf,ERROR);
+	MM_LOG (LOCAL->buf,ERROR);
 	ftruncate (LOCAL->fd,sbuf.st_size);
       }
       fstat (LOCAL->fd,&sbuf);	/* yes, get current file size */
@@ -713,7 +702,7 @@ void tenex_snarf (MAILSTREAM *stream)
     }
     if (sysibx) mail_close (sysibx);
   }
-  mm_nocritical (stream);	/* release critical */
+  MM_NOCRITICAL (stream);	/* release critical */
   unlockfd (ld,lock);		/* release exclusive parse/append permission */
   LOCAL->lastsnarf = time (0);	/* note time of last snarf */
 }
@@ -724,6 +713,7 @@ void tenex_snarf (MAILSTREAM *stream)
 
 void tenex_expunge (MAILSTREAM *stream)
 {
+  time_t tp[2];
   struct stat sbuf;
   off_t pos = 0;
   int ld;
@@ -737,7 +727,7 @@ void tenex_expunge (MAILSTREAM *stream)
 				/* do nothing if stream dead */
   if (!tenex_ping (stream)) return;
   if (stream->rdonly) {		/* won't do on readonly files! */
-    mm_log ("Expunge ignored on readonly mailbox",WARN);
+    MM_LOG ("Expunge ignored on readonly mailbox",WARN);
     return;
   }
   if (LOCAL->filetime && !LOCAL->shouldcheck) {
@@ -755,7 +745,7 @@ void tenex_expunge (MAILSTREAM *stream)
    */
 				/* get exclusive parse/append permission */
   if ((ld = lockfd (LOCAL->fd,lock,LOCK_EX)) < 0) {
-    mm_log ("Unable to lock expunge mailbox",ERROR);
+    MM_LOG ("Unable to lock expunge mailbox",ERROR);
     return;
   }
 				/* get exclusive access */
@@ -763,12 +753,13 @@ void tenex_expunge (MAILSTREAM *stream)
     (*bn) (BLOCK_FILELOCK,NIL);
     flock (LOCAL->fd,LOCK_SH);	/* recover previous lock */
     (*bn) (BLOCK_NONE,NIL);
-    mm_log("Can't expunge because mailbox is in use by another process",ERROR);
+    MM_LOG ("Can't expunge because mailbox is in use by another process",
+	      ERROR);
     unlockfd (ld,lock);		/* release exclusive parse/append permission */
     return;
   }
 
-  mm_critical (stream);		/* go critical */
+  MM_CRITICAL (stream);		/* go critical */
   recent = stream->recent;	/* get recent now that pinged and locked */
   while (i <= stream->nmsgs) {	/* for each message */
     elt = tenex_elt (stream,i);	/* get cache element */
@@ -792,8 +783,8 @@ void tenex_expunge (MAILSTREAM *stream)
 	while (T) {
 	  lseek (LOCAL->fd,pos,L_SET);
 	  if (write (LOCAL->fd,LOCAL->buf,m) > 0) break;
-	  mm_notify (stream,strerror (errno),WARN);
-	  mm_diskerror (stream,errno,T);
+	  MM_NOTIFY (stream,strerror (errno),WARN);
+	  MM_DISKERROR (stream,errno,T);
 	}
 	pos += m;		/* new position */
 	j += m;			/* next chunk, perhaps */
@@ -808,19 +799,21 @@ void tenex_expunge (MAILSTREAM *stream)
     if (pos != (LOCAL->filesize -= delta)) {
       sprintf (LOCAL->buf,"Calculated size mismatch %lu != %lu, delta = %lu",
 	       (unsigned long) pos,(unsigned long) LOCAL->filesize,delta);
-      mm_log (LOCAL->buf,WARN);
+      MM_LOG (LOCAL->buf,WARN);
       LOCAL->filesize = pos;	/* fix it then */
     }
     ftruncate (LOCAL->fd,LOCAL->filesize);
     sprintf (LOCAL->buf,"Expunged %lu messages",n);
 				/* output the news */
-    mm_log (LOCAL->buf,(long) NIL);
+    MM_LOG (LOCAL->buf,(long) NIL);
   }
-  else mm_log ("No messages deleted, so no update needed",(long) NIL);
+  else MM_LOG ("No messages deleted, so no update needed",(long) NIL);
   fsync (LOCAL->fd);		/* force disk update */
   fstat (LOCAL->fd,&sbuf);	/* get new write time */
-  LOCAL->filetime = sbuf.st_mtime;
-  mm_nocritical (stream);	/* release critical */
+  tp[1] = LOCAL->filetime = sbuf.st_mtime;
+  tp[0] = time (0);		/* reset atime to now */
+  utime (stream->mailbox,tp);
+  MM_NOCRITICAL (stream);	/* release critical */
 				/* notify upper level of new mailbox size */
   mail_exists (stream,stream->nmsgs);
   mail_recent (stream,recent);
@@ -852,19 +845,19 @@ long tenex_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 				/* make sure valid mailbox */
   if (!tenex_isvalid (mailbox,LOCAL->buf)) switch (errno) {
   case ENOENT:			/* no such file? */
-    mm_notify (stream,"[TRYCREATE] Must create mailbox before copy",NIL);
+    MM_NOTIFY (stream,"[TRYCREATE] Must create mailbox before copy",NIL);
     return NIL;
   case 0:			/* merely empty file? */
     break;
   case EINVAL:
     if (pc) return (*pc) (stream,sequence,mailbox,options);
     sprintf (LOCAL->buf,"Invalid Tenex-format mailbox name: %.80s",mailbox);
-    mm_log (LOCAL->buf,ERROR);
+    MM_LOG (LOCAL->buf,ERROR);
     return NIL;
   default:
     if (pc) return (*pc) (stream,sequence,mailbox,options);
     sprintf (LOCAL->buf,"Not a Tenex-format mailbox: %.80s",mailbox);
-    mm_log (LOCAL->buf,ERROR);
+    MM_LOG (LOCAL->buf,ERROR);
     return NIL;
   }
   if (!((options & CP_UID) ? mail_uid_sequence (stream,sequence) :
@@ -872,13 +865,13 @@ long tenex_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 				/* got file? */  
   if ((fd=open(tenex_file(file,mailbox),O_RDWR|O_CREAT,S_IREAD|S_IWRITE))<0) {
     sprintf (LOCAL->buf,"Unable to open copy mailbox: %s",strerror (errno));
-    mm_log (LOCAL->buf,ERROR);
+    MM_LOG (LOCAL->buf,ERROR);
     return NIL;
   }
-  mm_critical (stream);		/* go critical */
+  MM_CRITICAL (stream);		/* go critical */
 				/* get exclusive parse/append permission */
   if ((ld = lockfd (fd,lock,LOCK_EX)) < 0) {
-    mm_log ("Unable to lock copy mailbox",ERROR);
+    MM_LOG ("Unable to lock copy mailbox",ERROR);
     return NIL;
   }
   fstat (fd,&sbuf);		/* get current file size */
@@ -899,7 +892,7 @@ long tenex_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 				/* make sure all the updates take */
   if (!(ret && (ret = !fsync (fd)))) {
     sprintf (LOCAL->buf,"Unable to write message: %s",strerror (errno));
-    mm_log (LOCAL->buf,ERROR);
+    MM_LOG (LOCAL->buf,ERROR);
     ftruncate (fd,sbuf.st_size);
   }
   tp[0] = sbuf.st_atime;	/* preserve atime and mtime */
@@ -907,7 +900,7 @@ long tenex_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
   utime (file,tp);		/* set the times */
   close (fd);			/* close the file */
   unlockfd (ld,lock);		/* release exclusive parse/append permission */
-  mm_nocritical (stream);	/* release critical */
+  MM_NOCRITICAL (stream);	/* release critical */
 				/* delete all requested messages */
   if (ret && (options & CP_MOVE)) {
     for (i = 1; i <= stream->nmsgs; i++)
@@ -919,7 +912,9 @@ long tenex_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
     if (!stream->rdonly) {	/* make sure the update takes */
       fsync (LOCAL->fd);
       fstat (LOCAL->fd,&sbuf);	/* get current write time */
-      LOCAL->filetime = sbuf.st_mtime;
+      tp[1] = LOCAL->filetime = sbuf.st_mtime;
+      tp[0] = time (0);		/* make sure atime remains greater */
+      utime (stream->mailbox,tp);
     }
   }
   return ret;
@@ -928,35 +923,25 @@ long tenex_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 /* Tenex mail append message from stringstruct
  * Accepts: MAIL stream
  *	    destination mailbox
- *	    stringstruct of messages to append
+ *	    append callback
+ *	    data for callback
  * Returns: T if append successful, else NIL
  */
 
-long tenex_append (MAILSTREAM *stream,char *mailbox,char *flags,char *date,
-		   STRING *message)
+long tenex_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
 {
   struct stat sbuf;
-  int fd,ld;
-  char c,*s,tmp[MAILTMPLEN],file[MAILTMPLEN],lock[MAILTMPLEN];
+  int fd,ld,c;
+  char *flags,*date,tmp[MAILTMPLEN],file[MAILTMPLEN],lock[MAILTMPLEN];
   time_t tp[2];
+  FILE *df;
   MESSAGECACHE elt;
-  long i = SIZE (message);
-  long size = 0;
+  long f;
+  unsigned long i,j,uf,size;
+  STRING *message;
   long ret = LONGT;
-  unsigned long uf = 0,j;
-  short f = (short) mail_parse_flags (stream ? stream :
-				      user_flags (&tenexproto),flags,&j);
-				/* reverse bits (dontcha wish we had CIRC?) */
-  while (j) uf |= 1 << (29 - find_rightmost_bit (&j));
-  if (date) {			/* want to preserve date? */
-				/* yes, parse date into an elt */
-    if (!mail_parse_date (&elt,date)) {
-      sprintf (tmp,"Bad date in append: %.80s",date);
-      mm_log (tmp,ERROR);
-      return NIL;
-    }
-  }
-				/* N.B.: can't use LOCAL->buf for tmp */
+				/* default stream to prototype */
+  if (!stream) stream = user_flags (&tenexproto);
 				/* make sure valid mailbox */
   if (!tenex_isvalid (mailbox,tmp)) switch (errno) {
   case ENOENT:			/* no such file? */
@@ -967,62 +952,89 @@ long tenex_append (MAILSTREAM *stream,char *mailbox,char *flags,char *date,
 	((mailbox[4] == 'X') || (mailbox[4] == 'x')) && !mailbox[5])
       dummy_create (NIL,"mail.txt");
     else {
-      mm_notify (stream,"[TRYCREATE] Must create mailbox before append",NIL);
+      MM_NOTIFY (stream,"[TRYCREATE] Must create mailbox before append",NIL);
       return NIL;
     }
 				/* falls through */
   case 0:			/* merely empty file? */
     break;
   case EINVAL:
-    sprintf (tmp,"Invalid Tenex-format mailbox name: %.80s",mailbox);
-    mm_log (tmp,ERROR);
+    sprintf (tmp,"Invalid TENEX-format mailbox name: %.80s",mailbox);
+    MM_LOG (tmp,ERROR);
     return NIL;
   default:
-    sprintf (tmp,"Not a Tenex-format mailbox: %.80s",mailbox);
-    mm_log (tmp,ERROR);
+    sprintf (tmp,"Not a TENEX-format mailbox: %.80s",mailbox);
+    MM_LOG (tmp,ERROR);
     return NIL;
   }
-  if ((fd = open (tenex_file (file,mailbox),O_RDWR|O_CREAT,
-		  S_IREAD|S_IWRITE)) < 0) {
-    sprintf (tmp,"Can't open append mailbox: %s",strerror (errno));
-    mm_log (tmp,ERROR);
-    return NIL;
-  }
-				/* get exclusive parse/append permission */
-  if ((ld = lockfd (fd,lock,LOCK_EX)) < 0) {
-    mm_log ("Unable to lock append mailbox",ERROR);
-    return NIL;
-  }
-  j = GETPOS (message);		/* remember current position */
-  while (i--) if ((c = SNX (message)) != '\015') size++;
-  SETPOS (message,j);		/* restore current position */
+				/* get first message */
+  if (!MM_APPEND (af) (stream,data,&flags,&date,&message)) return NIL;
 
-  mm_critical (stream);		/* go critical */
+				/* open destination mailbox */
+  if (((fd = open (tenex_file (file,mailbox),O_WRONLY|O_APPEND|O_CREAT,
+		   S_IREAD|S_IWRITE)) < 0) || !(df = fdopen (fd,"ab"))) {
+    sprintf (tmp,"Can't open append mailbox: %s",strerror (errno));
+    MM_LOG (tmp,ERROR);
+    return NIL;
+  }
+				/* get parse/append permission */
+  if ((ld = lockfd (fd,lock,LOCK_EX)) < 0) {
+    MM_LOG ("Unable to lock append mailbox",ERROR);
+    close (fd);
+    return NIL;
+  }
+  MM_CRITICAL (stream);		/* go critical */
   fstat (fd,&sbuf);		/* get current file size */
-  lseek (fd,sbuf.st_size,L_SET);/* move to end of file */
-				/* write preseved date */
-  if (date) mail_date (tmp,&elt);
-  else internal_date (tmp);	/* get current date in IMAP format */
-				/* add remainder of header */
-  sprintf (tmp+26,",%ld;%010lo%02o\n",size,uf,(unsigned) f);
-  size += (i = strlen (tmp));	/* size of buffer */
-  s = (char *) fs_get (size);	/* get buffer for message */
-  strncpy (s,tmp,i);		/* copy message */
-  while (i < size) if ((c = SNX (message)) != '\015') s[i++] = c;
-				/* write message */
-  if ((write (fd,s,size) < 0) || fsync (fd)) {
+  do {				/* parse flags */ 
+    if (!SIZE (message)) {	/* guard against zero-length */
+      MM_LOG ("Append of zero-length message",ERROR);
+      ret = NIL;
+      break;
+    }
+    f = mail_parse_flags (stream,flags,&i);
+				/* reverse bits (dontcha wish we had CIRC?) */
+    for (uf = 0; i; uf |= 1 << (29 - find_rightmost_bit (&i)));
+    if (date) {			/* parse date if given */
+      if (!mail_parse_date (&elt,date)) {
+	sprintf (tmp,"Bad date in append: %.80s",date);
+	MM_LOG (tmp,ERROR);
+	ret = NIL;		/* mark failure */
+	break;
+      }
+      mail_date (tmp,&elt);	/* write preseved date */
+    }
+    else internal_date (tmp);	/* get current date in IMAP format */
+    i = GETPOS (message);	/* remember current position */
+    for (j = SIZE (message), size = 0; j; --j)
+      if (SNX (message) != '\015') ++size;
+    SETPOS (message,i);		/* restore position */
+				/* write header */
+    if (fprintf (df,"%s,%lu;%010lo%02lo\n",tmp,size,uf,(unsigned long) f) < 0)
+      ret = NIL;
+    else {			/* write message */
+      while (size) if ((c = 0xff & SNX (message)) != '\015') {
+	if (putc (c,df) != EOF) --size;
+	else break;
+      }
+				/* get next message */
+      if (size || !MM_APPEND (af) (stream,data,&flags,&date,&message))
+	ret = NIL;
+    }
+  } while (ret && message);
+				/* if error... */
+  if (!ret || (fflush (df) == EOF)) {
+    ftruncate (fd,sbuf.st_size);/* revert file */
+    close (fd);			/* make sure fclose() doesn't corrupt us */
     sprintf (tmp,"Message append failed: %s",strerror (errno));
-    mm_log (tmp,ERROR);
-    ftruncate (fd,sbuf.st_size);
+    MM_LOG (tmp,ERROR);
     ret = NIL;
   }
   tp[0] = sbuf.st_atime;	/* preserve atime and mtime */
   tp[1] = sbuf.st_mtime;
   utime (file,tp);		/* set the times */
-  close (fd);			/* close the file */
+  fclose (df);			/* close the file */
   unlockfd (ld,lock);		/* release exclusive parse/append permission */
-  mm_nocritical (stream);	/* release critical */
-  fs_give ((void **) &s);	/* flush the buffer */
+  MM_NOCRITICAL (stream);	/* release critical */
   return ret;
 }
 
@@ -1075,11 +1087,13 @@ long tenex_parse (MAILSTREAM *stream)
   long curpos = LOCAL->filesize;
   long nmsgs = stream->nmsgs;
   long recent = stream->recent;
+  short added = NIL;
   short silent = stream->silent;
   fstat (LOCAL->fd,&sbuf);	/* get status */
   if (sbuf.st_size < curpos) {	/* sanity check */
-    sprintf (tmp,"Mailbox shrank from %ld to %ld!",curpos,sbuf.st_size);
-    mm_log (tmp,ERROR);
+    sprintf (tmp,"Mailbox shrank from %lu to %lu!",
+	     (unsigned long) curpos,(unsigned long) sbuf.st_size);
+    MM_LOG (tmp,ERROR);
     tenex_close (stream,NIL);
     return NIL;
   }
@@ -1091,7 +1105,7 @@ long tenex_parse (MAILSTREAM *stream)
       sprintf (tmp,"Unable to read internal header at %lu, size = %lu: %s",
 	       (unsigned long) curpos,(unsigned long) sbuf.st_size,
 	       i ? strerror (errno) : "no data read");
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       tenex_close (stream,NIL);
       return NIL;
     }
@@ -1099,7 +1113,7 @@ long tenex_parse (MAILSTREAM *stream)
     if (!(s = strchr (LOCAL->buf,'\012'))) {
       sprintf (tmp,"Unable to find newline at %lu in %lu bytes, text: %s",
 	       (unsigned long) curpos,i,LOCAL->buf);
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       tenex_close (stream,NIL);
       return NIL;
     }
@@ -1108,12 +1122,13 @@ long tenex_parse (MAILSTREAM *stream)
     if (!((s = strchr (LOCAL->buf,',')) && (t = strchr (s+1,';')))) {
       sprintf (tmp,"Unable to parse internal header at %lu: %s",
 	       (unsigned long) curpos,LOCAL->buf);
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       tenex_close (stream,NIL);
       return NIL;
     }
     *s++ = '\0'; *t++ = '\0';	/* tie off fields */
 
+    added = T;			/* note that a new message was added */
 				/* swell the cache */
     mail_exists (stream,++nmsgs);
 				/* instantiate an elt for this message */
@@ -1136,7 +1151,7 @@ long tenex_parse (MAILSTREAM *stream)
     else {			/* oops */
       sprintf (tmp,"Unable to parse internal header elements at %ld: %s,%s;%s",
 	       curpos,LOCAL->buf,x,t);
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       tenex_close (stream,NIL);
       return NIL;
     }
@@ -1145,7 +1160,7 @@ long tenex_parse (MAILSTREAM *stream)
       sprintf (tmp,"Last message (at %lu) runs past end of file (%lu > %lu)",
 	       elt->private.special.offset,(unsigned long) curpos,
 	       (unsigned long) sbuf.st_size);
-      mm_log (tmp,ERROR);
+      MM_LOG (tmp,ERROR);
       tenex_close (stream,NIL);
       return NIL;
     }
@@ -1174,6 +1189,12 @@ long tenex_parse (MAILSTREAM *stream)
   LOCAL->filesize = sbuf.st_size;
   fstat (LOCAL->fd,&sbuf);	/* get status again to ensure time is right */
   LOCAL->filetime = sbuf.st_mtime;
+  if (added) {			/* make sure atime updated */
+    time_t tp[2];
+    tp[0] = time (0);
+    tp[1] = LOCAL->filetime;
+    utime (stream->mailbox,tp);
+  }
   stream->silent = silent;	/* can pass up events now */
   mail_exists (stream,nmsgs);	/* notify upper level of new mailbox size */
   mail_recent (stream,recent);	/* and of change in recent messages */
@@ -1204,7 +1225,7 @@ MESSAGECACHE *tenex_elt (MAILSTREAM *stream,unsigned long msgno)
   if ((old.seen != elt->seen) || (old.deleted != elt->deleted) ||
       (old.flagged != elt->flagged) || (old.answered != elt->answered) ||
       (old.draft != elt->draft) || (old.user_flags != elt->user_flags))
-    mm_flags (stream,msgno);	/* let top level know */
+    MM_FLAGS (stream,msgno);	/* let top level know */
   return elt;
 }
 
@@ -1247,8 +1268,9 @@ void tenex_read_flags (MAILSTREAM *stream,MESSAGECACHE *elt)
 
 void tenex_update_status (MAILSTREAM *stream,unsigned long msgno,long syncflag)
 {
-  MESSAGECACHE *elt = mail_elt (stream,msgno);
+  time_t tp[2];
   struct stat sbuf;
+  MESSAGECACHE *elt = mail_elt (stream,msgno);
   unsigned long j,k = 0;
 				/* readonly */
   if (stream->rdonly || !elt->valid) tenex_read_flags (stream,elt);
@@ -1269,7 +1291,9 @@ void tenex_update_status (MAILSTREAM *stream,unsigned long msgno,long syncflag)
     if (syncflag) {		/* sync if requested */
       fsync (LOCAL->fd);
       fstat (LOCAL->fd,&sbuf);	/* get new write time */
-      LOCAL->filetime = sbuf.st_mtime;
+      tp[1] = LOCAL->filetime = sbuf.st_mtime;
+      tp[0] = time (0);		/* make sure read is later */
+      utime (stream->mailbox,tp);
     }
   }
 }
