@@ -4,7 +4,7 @@
  *      Declarations for things used internally by the Ratatosk
  *      procedures but not exported outside the module.
  *
- * TkRat software and its included text is Copyright 1996-2000 by
+ * TkRat software and its included text is Copyright 1996-2002 by
  * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <mail.h>
+#include <tcp.h>
 #include <nl.h>
 #include <rfc822.h>
 #include <env.h>
@@ -89,6 +90,10 @@
 
 #include <tcl.h>
 
+#ifndef CONST84
+#   define CONST84
+#endif
+
 /*
  * Sigh, tcl uses different prototypes for its replacement functions for
  * malloc, realloc and free than in the original functions. Also tcl
@@ -101,7 +106,9 @@
 #   undef ckrealloc
 #   define ckalloc(x) Tcl_DbCkalloc(x, __FILE__, __LINE__)
 #   define ckfree(x)  Tcl_DbCkfree((char*)x, __FILE__, __LINE__)
-#   define ckrealloc(x,y) ((x)?Tcl_DbCkrealloc((char*)(x), (y),__FILE__, __LINE__) : Tcl_DbCkalloc((y), __FILE__, __LINE__))
+#   define ckrealloc(x,y) ((x)? \
+                  Tcl_DbCkrealloc((char*)(x), (y),__FILE__, __LINE__) \
+                : Tcl_DbCkalloc((y), __FILE__, __LINE__))
 #else /* TCL_MEM_DEBUG */
 #   if TCL_MAJOR_VERSION == 8 && TCL_MINOR_VERSION >= 2
 #       undef ckalloc
@@ -125,8 +132,8 @@
  */
 
 typedef enum {
-    TO, FROM, CC, MESSAGE_ID, REFERENCE, SUBJECT, DATE, KEYWORDS, RSIZE, STATUS,
-    EX_TIME, EX_TYPE, FILENAME, RATDBETYPE_END
+    TO, FROM, CC, MESSAGE_ID, REFERENCE, SUBJECT, DATE, KEYWORDS, RSIZE,
+    STATUS, EX_TIME, EX_TYPE, FILENAME, RATDBETYPE_END
 } RatDbEType;
 
 typedef struct RatDbEntry {
@@ -145,6 +152,13 @@ typedef enum {
 
 typedef struct RatFolderInfo *RatFolderInfoPtr;
 
+/*
+ * Current data
+ */
+typedef enum {
+    RAT_HOST, RAT_MAILBOX, RAT_PERSONAL, RAT_HELO
+} RatCurrentType;
+
 /*
  * A SMTP channel
  */
@@ -162,14 +176,12 @@ extern char alphabetHEX[17];
 
 /* ratAppInit.c */
 extern Tcl_Interp *timerInterp;
-extern char *currentHost;
-extern char *currentDomain;
-extern char *currentMailboxName;
-extern char *currentPersonalName;
 extern char *dayName[];
 extern char *monthName[];
-extern void RatLog (Tcl_Interp *interp, RatLogLevel level, char *message,
-	RatLogType type);
+extern char *RatGetCurrent(Tcl_Interp *interp, RatCurrentType what,
+			   const char *role);
+extern void RatLog (Tcl_Interp *interp, RatLogLevel level,
+		    CONST84 char *message, RatLogType type);
 extern void RatLogF (Tcl_Interp *interp, RatLogLevel level, char *tag,
 	RatLogType type, ...);
 extern Tcl_Obj *RatMangleNumber(int number);
@@ -178,28 +190,31 @@ extern long RatTclPuts (void *stream_x, char *string);
 extern long RatStringPuts (void *stream_x, char *string);
 extern long RatDelaySoutr (void *stream_x, char *string);
 extern void RatInitDelayBuffer ();
-extern int RatTranslateWrite(Tcl_Channel channel, char *charbuf, int len);
-extern MESSAGE *RatParseMsg (Tcl_Interp *interp, char *message);
-extern int RatIsEmpty (char *string);
+extern int RatTranslateWrite(Tcl_Channel channel, CONST84 char *charbuf,
+			     int len);
+extern MESSAGE *RatParseMsg (Tcl_Interp *interp, unsigned char *message);
+extern int RatIsEmpty (const char *string);
 extern int RatEncodingCompat (Tcl_Interp *interp, char *wanted, char *avail);
-extern char* RatLindex (Tcl_Interp *interp, char *list, int index);
-extern Tcl_CmdProc RatGenId;
+extern char* RatLindex (Tcl_Interp *interp, const char *list, int index);
+extern Tcl_ObjCmdProc RatGenId;
 extern Tcl_Obj *RatFormatDate(Tcl_Interp *interp, int month, int day);
 extern int RatGetTimeZone(unsigned long currentTime);
-extern char *RatStrNCpy(char *dst, const char *src, size_t n);
-extern void Rat_DStringApendNoCRLF(Tcl_DString *ds, const char *s, int length);
-unsigned char *Rat_ReadFile(Tcl_Interp *interp, const char *filename,
+extern void RatDStringApendNoCRLF(Tcl_DString *ds, const char *s, int length);
+extern unsigned char *RatReadFile(Tcl_Interp *interp, const char *filename,
 			    unsigned long *length, int convert_to_crlf);
+extern CONST84 char *RatGetPathOption(Tcl_Interp *interp, char *name);
+
 
 /* ratHold.c */
-extern Tcl_CmdProc RatHold;
-extern int RatHoldInsert(Tcl_Interp *interp, char *dir, char *handler,
-	char *description);
-extern int RatHoldList(Tcl_Interp *interp, char *dir, Tcl_DString *fileListPtr);
-extern int RatHoldExtract(Tcl_Interp *interp, char *prefix,
-	Tcl_DString *usedVariablesPtr, Tcl_DString *filesPtr);
+extern Tcl_ObjCmdProc RatHold;
+extern int RatHoldInsert(Tcl_Interp *interp, const char *dir,
+			 char *handler, const char *description);
+extern int RatHoldList(Tcl_Interp *interp, const char *dir,
+		       Tcl_Obj *fileListPtr);
+extern int RatHoldExtract(Tcl_Interp *interp, const char *prefix,
+			  Tcl_Obj *usedArraysPtr, Tcl_Obj *filesPtr);
 extern void RatHoldInitVars(Tcl_Interp *interp);
-extern void RatHoldUpdateVars(Tcl_Interp *interp, char *dir, int diff);
+extern void RatHoldUpdateVars(Tcl_Interp *interp, const char *dir, int diff);
 
 /* RatSender.c */
 extern void RatSender(Tcl_Interp *interp);
@@ -209,46 +224,55 @@ extern char *RatSendPGPCommand(char *cmd);
 extern int RatFolderInit (Tcl_Interp *interp);
 
 /* ratStdFolder.c */
-extern void AppendToIMAP(Tcl_Interp *interp, char *mailbox, char *user,
-			 char *flags, char *date, char *msg, int length);
+extern void AppendToIMAP(Tcl_Interp *interp, const char *mailbox,
+			 const char *flags, const char *date, const char *msg,
+			 int length);
 extern void ClearStdPasswds(int freethem);
 extern Tcl_ObjCmdProc RatCheckEncodingsCmd;
 
 /* ratCode.c */
-extern char *RatDecodeHeader(Tcl_Interp *interp, char *string, int adr);
-extern Tcl_DString *RatDecode(Tcl_Interp *interp, int cte,
-	char *data, int length, char *charset);
+extern char *RatDecodeHeader(Tcl_Interp *interp, const char *string, int adr);
+extern Tcl_DString *RatDecode(Tcl_Interp *interp, int cte, const char *data,
+			      int length, const char *charset);
 extern char *RatEncodeHeaderLine(Tcl_Interp *interp, Tcl_Obj *line,
 	int nameLength);
 extern void RatEncodeAddresses(Tcl_Interp *interp, ADDRESS *adrPtr);
-extern Tcl_Encoding RatGetEncoding(Tcl_Interp *interp, char *name);
+extern Tcl_Encoding RatGetEncoding(Tcl_Interp *interp, const char *name);
 extern Tcl_Obj *RatCode64(Tcl_Obj *oPtr);
-extern unsigned char *RatUtf8toMutf7(const unsigned char *src);
-extern unsigned char *RatMutf7toUtf8(const unsigned char *src);
+extern char *RatUtf8toMutf7(const char *src);
+extern char *RatMutf7toUtf8(const char *src);
+extern Tcl_DString* RatEncodeQP(const unsigned char *line);
+extern Tcl_ObjCmdProc RatEncodeQPCmd;
+extern unsigned char *RatDecodeQP(unsigned char *line);
+extern Tcl_ObjCmdProc RatDecodeQPCmd;
 
 /* ratAddress.c */
 extern Tcl_HashTable aliasTable;
 extern Tcl_ObjCmdProc RatCreateAddressCmd;
+extern Tcl_ObjCmdProc RatAddress;
 extern void RatInitAddresses (Tcl_Interp *interp, ADDRESS *addressPtr);
-extern int RatAddress (ClientData dummy, Tcl_Interp *interp, int argc,
-	char *argv[]);
 extern Tcl_CmdDeleteProc RatDeleteAddress;
-extern int RatAddressIsMe (Tcl_Interp *interp, ADDRESS *adrPtr, int useUP);
+extern int RatAddressIsMe (Tcl_Interp *interp, ADDRESS *adrPtr, int trustUser);
 extern int RatAddressCompare (ADDRESS *adr1Ptr, ADDRESS* adr2Ptr);
 extern void RatAddressTranslate (Tcl_Interp *interp, ADDRESS *adrPtr);
 extern Tcl_ObjCmdProc RatAliasCmd;
 extern void RatAddressTranslate (Tcl_Interp *interp, ADDRESS *adrPtr);
 extern char *RatAddressMail(ADDRESS *adrPtr);
-extern Tcl_CmdProc RatSplitAddresses;
+extern Tcl_ObjCmdProc RatSplitAddresses;
 extern size_t RatAddressSize(ADDRESS *adrPtr, int all);
+extern void RatGenerateAddresses(Tcl_Interp *interp, const char *role,
+				 char *msgh, ADDRESS **from, ADDRESS **sender);
+extern Tcl_ObjCmdProc RatGenerateAddressesCmd;
+extern CONST84 char *RatFindCharInHeader(CONST84 char *header, char m);
 
 /* ratDbase.c */
-extern int RatDbInsert (Tcl_Interp *interp, char *to, char *from, char *cc,
-	char *msgid, char *ref, char *subject, long date, char *flags,
-	char *keywords, long exDate, char *exType, char *fromline, char *mail,
-	int length);
+extern int RatDbInsert (Tcl_Interp *interp, const char *to, const char *from,
+			const char *cc, const char *msgid, const char *ref,
+			const char *subject, long date, const char *flags,
+			const char *keywords, long exDate, const char *exType,
+			const char *fromline, const char *mail, int length);
 extern int RatDbSetStatus (Tcl_Interp *interp, int index, char *status);
-extern int RatDbSearch (Tcl_Interp *interp, char *exp, int *numFoundPtr,
+extern int RatDbSearch (Tcl_Interp *interp, Tcl_Obj *exp, int *numFoundPtr,
 	int **foundPtrPtr);
 extern RatDbEntry *RatDbGetEntry (int index);
 extern MESSAGE *RatDbGetMessage (Tcl_Interp *interp, int index, char **bufPtr);
@@ -264,19 +288,21 @@ extern void RatDbClose(void);
 extern int RatDbCheck(Tcl_Interp *interp, int fix);
 
 /* ratSMTP.c */
-extern SMTPChannel RatSMTPOpen (Tcl_Interp *interp, char *host, int verbose);
-extern void RatSMTPClose (Tcl_Interp *interp, SMTPChannel channel, int verbose);
+extern SMTPChannel RatSMTPOpen (Tcl_Interp *interp, char *host, int verbose,
+				const char *role);
+extern void RatSMTPClose (Tcl_Interp *interp, SMTPChannel channel,int verbose);
+extern void RatSMTPCloseAll (Tcl_Interp *interp, int verbose);
 extern int RatSMTPSend (Tcl_Interp *interp, SMTPChannel channel,
 	ENVELOPE *envelopePtr, BODY *bodyPtr, int requestDSN, int verbose);
-extern Tcl_CmdProc RatSMTPSupportDSN;
+extern Tcl_ObjCmdProc RatSMTPSupportDSN;
 extern long RatTclPutsSMTP (void *stream_x, char *string);
 extern long RatTclPutsSendmail (void *stream_x, char *string);
 extern size_t RatHeaderSize(ENVELOPE *env,BODY *body);
 
 /* ratDSN.c */
 extern int RatDSNInit (Tcl_Interp *interp);
-extern DSNhandle RatDSNStartMessage (Tcl_Interp *interp, char *id,
-	char *subject);
+extern DSNhandle RatDSNStartMessage (Tcl_Interp *interp, const char *id,
+				     const char *subject);
 extern void RatDSNAddRecipient (Tcl_Interp *interp, DSNhandle handle,
 	char *recipient);
 extern void RatDSNAbort (Tcl_Interp *interp, DSNhandle handle);
@@ -289,21 +315,35 @@ extern int RatMessageGetHeader(Tcl_Interp *interp, char *srcHeader);
 Tcl_Obj *RatWrapMessage(Tcl_Interp *interp, Tcl_Obj *oPtr);
 
 /* ratMailcap.c */
-extern Tcl_CmdProc RatMailcapReload;
+extern Tcl_ObjCmdProc RatMailcapReload;
 
 /* ratCompat.c */
-extern Tcl_CmdProc RatFile;
+#ifndef HAVE_SNPRINTF
+extern size_t snprintf (char *buf, size_t buflen, const char *fmt, ...);
+#endif /* HAVE_SNPRINTF */
+#ifndef HAVE_STRLCPY
+extern char *strlcpy(char *dst, const char *src, size_t n);
+#endif /* HAVE_STRLCPY */
+#ifndef HAVE_STRLCAT
+extern char *strlcat(char *dst, const char *src, size_t n);
+#endif /* HAVE_STRLCAT */
 
 /* ratPwCache.c */
-extern char *RatGetCachedPassword(Tcl_Interp *interp, const char *host,
-	unsigned int port, const char *user, const char *service);
-extern void RatCachePassword(Tcl_Interp *interp, const char *host,
-        unsigned int port, const char *user, const char *passwd,
-	const char *service, int store);
+extern char *RatGetCachedPassword(Tcl_Interp *interp, const char *spec);
+extern void RatCachePassword(Tcl_Interp *interp, const char *spec,
+			     const char *passwd, int store);
 extern void RatPasswdCachePurge(Tcl_Interp *interp, int disk_also);
 extern Tcl_ObjCmdProc RatPasswdCachePurgeCmd;
 
 /* ratPrint.c */
 extern Tcl_ObjCmdProc RatPrettyPrintMsg;
 
+/* ratWatchdog.c */
+extern void RatReleaseWatchdog(const char *tmpdir);
+
+/* ratBusy.c */
+extern void RatSetBusy(Tcl_Interp *interp);
+extern void RatClearBusy(Tcl_Interp *interp);
+extern Tcl_ObjCmdProc RatBusyCmd;
+
 #endif /* _RAT_H */
