@@ -1175,7 +1175,7 @@ RatExpandAlias(Tcl_Interp *interp, Tcl_DString *list, AliasExpand *ae)
     CONST84 char *entry_start, *entry_end, *key_start, *key_end, *c, *copy, *n;
     struct passwd *pwPtr;
     int length;
-    AliasInfo *aliasPtr;
+    AliasInfo *aliasPtr = NULL;
     Tcl_HashEntry *entryPtr;
     Tcl_DString expanded;
     ParseState ps;
