@@ -1040,7 +1040,7 @@ RatSendDeferred(Tcl_Interp *interp)
     int online;
 
     if (NULL == deferredDir) {
-	char *v;
+	CONST84 char *v;
 	
 	if (NULL == (v = RatGetPathOption(interp, "send_cache"))) {
 	    return TCL_ERROR;
