@@ -4,7 +4,7 @@
  *	This file contains basic support for decoding and encoding of
  *	strings coded in various MIME-encodings.
  *
- * TkRat software and its included text is Copyright 1996-2000 by
+ * TkRat software and its included text is Copyright 1996-2002 by
  * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
@@ -13,15 +13,6 @@
 
 #include "rat.h"
 
-/*
- * Used for list of saved encodings
- */
-typedef struct RatEncoding {
-    char *name;
-    Tcl_Encoding enc;
-    struct RatEncoding *nextPtr;
-} RatEncoding;
-
 /*
  * List used when decoding QP
  */
@@ -32,25 +23,29 @@ char alphabetHEX[17] = "0123456789ABCDEF";
  * It consists of 64 chars plus '=' and null
  */
 static char alphabet64[66] =
-	    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
+	   "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
 
 /*
  * List used when decoding modified base64
  * It consists of 64 chars plus '=' and null
  */
 static char modified64[66] =
-	    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+,=";
+	   "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+,=";
 
+#define RFC2047_MAX_LINE_LENGTH 75
+#define RFC2047_MAX_ENCODED_WORD_LENGTH 75
 
 static int FindMimeHdr(Tcl_Interp *interp, unsigned char *hdr,
 	unsigned char **sPtr, unsigned char **ePtr, Tcl_Encoding *encoding,
 	int *code, unsigned char **data, int *length);
 static int RatUtf8to16(const unsigned char *src, unsigned char *dst);
 static int RatUtf16to8(const unsigned char *src, unsigned char *dst);
+static int RatCheckEncoding(Tcl_Interp *interp, char *encoding_name,
+			    const char *string, int length);
+static int CreateEncWord(Tcl_Interp *interp, Tcl_Encoding enc,
+			 const char *charset, unsigned char *raw, int length,
+			 Tcl_DString *dest, int maxUse);
 
-#ifdef MEM_DEBUG
-static RatEncoding *mem_encPtr = NULL;
-#endif /* MEM_DEBUG */
 
 /*
  *----------------------------------------------------------------------
@@ -117,7 +112,7 @@ FindMimeHdr(Tcl_Interp *interp, unsigned char *hdr, unsigned char **sPtr,
  *
  * RatDecodeHeader --
  *
- *      Decodes a header line encoded according to rfc1522.
+ *      Decodes a header line encoded according to rfc2047.
  *
  * Results:
  *	Returns a pointer to a static storage area
@@ -131,11 +126,10 @@ FindMimeHdr(Tcl_Interp *interp, unsigned char *hdr, unsigned char **sPtr,
  */
 
 char*
-RatDecodeHeader(Tcl_Interp *interp, char *data, int adr)
+RatDecodeHeader(Tcl_Interp *interp, const char *data, int adr)
 {
     static Tcl_DString ds, tmp;
     static int initialized = 0;
-    char buf[1024];
     unsigned char *sPtr, *ePtr, *decoded, *text, *cPtr,
 	    *point = (unsigned char*)data;
     int length, code, first = 1;
@@ -179,10 +173,14 @@ RatDecodeHeader(Tcl_Interp *interp, char *data, int adr)
 		    Tcl_DStringAppend(&ds, (char*)point, sPtr-point);
 		}
 	    } else {
-		Tcl_DStringAppend(&ds, (char*)point, sPtr-point);
-		first = 0;
+		for (i=0; i<sPtr-point; i++) {
+		    if ('\n' != point[i]) {
+			Tcl_DStringAppend(&ds, &point[i], 1);
+		    }
+		}
 	    }
 	}
+	first = 0;
 	point = ePtr;
 	if (NULL == encoding) {
 	    Tcl_DStringAppend(&ds, (char*)sPtr, ePtr-sPtr);
@@ -190,32 +188,34 @@ RatDecodeHeader(Tcl_Interp *interp, char *data, int adr)
 	}
 	if (ENCBASE64 == code) {
 	    decoded = rfc822_base64(text, length, &dlen);
-	    memmove(buf, decoded, dlen);
-	    ckfree(decoded);
-	    buf[dlen] = '\0';
 	} else {
-	    for (i=0, cPtr = text; cPtr-text < length; i++, cPtr++) {
+	    decoded = (char*)ckalloc(length+1);
+	    for (dlen=0, cPtr=text; cPtr-text < length; cPtr++) {
 		if ('_' == *cPtr) {
-		    buf[i] = ' ';
+		    decoded[dlen++] = ' ';
 		} else if ('=' == *cPtr) {
-		    buf[i] =
+		    decoded[dlen++] =
 			    ((strchr(alphabetHEX, cPtr[1])-alphabetHEX)<<4) +
 			     (strchr(alphabetHEX, cPtr[2])-alphabetHEX);
 		    cPtr += 2;
 		} else {
-		    buf[i] = *cPtr;
+		    decoded[dlen++] = *cPtr;
 		}
 	    }
-	    buf[i] = '\0';
-	    dlen = i;
+	    decoded[dlen] = '\0';
 	}
-	Tcl_ExternalToUtfDString(encoding, buf, dlen, &tmp);
+	Tcl_ExternalToUtfDString(encoding, decoded, dlen, &tmp);
+	ckfree(decoded);
 	Tcl_DStringAppend(&ds,
-		Tcl_DStringValue(&tmp), Tcl_DStringLength(&tmp));
+			  Tcl_DStringValue(&tmp), Tcl_DStringLength(&tmp));
 	Tcl_DStringFree(&tmp);
     }
     if (*point) {
-	Tcl_DStringAppend(&ds, (char*)point, -1);
+	for (sPtr = point; *sPtr; sPtr++) {
+	    if ('\n' != *sPtr) {
+		Tcl_DStringAppend(&ds, (char*)sPtr, 1);
+	    }
+	}
     }
     if (myPtr) {
 	Tcl_DStringFree(myPtr);
@@ -250,9 +250,11 @@ RatDecodeHeader(Tcl_Interp *interp, char *data, int adr)
  */
 
 Tcl_DString*
-RatDecode(Tcl_Interp *interp, int cte, char *data, int length, char *charset)
+RatDecode(Tcl_Interp *interp, int cte, const char *data, int length,
+	  const char *charset)
 {
-    char *src, *dst, buf[64], lbuf[4];
+    char *dst, buf[64], lbuf[4];
+    const char *src;
     int allocated, dataIndex = 0, index, srcLength, len;
     Tcl_Encoding enc = NULL;
     Tcl_DString *dsPtr = (Tcl_DString*)ckalloc(sizeof(Tcl_DString)),
@@ -276,11 +278,11 @@ RatDecode(Tcl_Interp *interp, int cte, char *data, int length, char *charset)
 		if (0 == index) {
 		    continue;
 		}
-		src[srcLength++] = lbuf[0] << 2 | ((lbuf[1]>>4)&0x3);
+		buf[srcLength++] = lbuf[0] << 2 | ((lbuf[1]>>4)&0x3);
 		if (strchr(alphabet64, '=')-alphabet64 != lbuf[2]) {
-		    src[srcLength++] = lbuf[1] << 4 | ((lbuf[2]>>2)&0xf);
+		    buf[srcLength++] = lbuf[1] << 4 | ((lbuf[2]>>2)&0xf);
 		    if (strchr(alphabet64, '=')-alphabet64 != lbuf[3]) {
-			src[srcLength++] = lbuf[2] << 6 | (lbuf[3]&0x3f);
+			buf[srcLength++] = lbuf[2] << 6 | (lbuf[3]&0x3f);
 		    }
 		}
 	    }
@@ -294,14 +296,14 @@ RatDecode(Tcl_Interp *interp, int cte, char *data, int length, char *charset)
 		    } else if ('\n' == data[dataIndex+1]) {
 			dataIndex += 2;
 		    } else {
-			src[srcLength++] = 16*(strchr(alphabetHEX,
+			buf[srcLength++] = 16*(strchr(alphabetHEX,
 				data[dataIndex+1])-alphabetHEX)
 				+ strchr(alphabetHEX,
 				data[dataIndex+2])-alphabetHEX;
 			dataIndex += 3;
 		    }
 		} else {
-		    src[srcLength++] = data[dataIndex++];
+		    buf[srcLength++] = data[dataIndex++];
 		}
 	    }
 	} else {
@@ -333,18 +335,104 @@ RatDecode(Tcl_Interp *interp, int cte, char *data, int length, char *charset)
     return dsPtr;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * CreateEncWord --
+ *
+ *      Tres to create an encoded word (if needed) by the given string.
+ *      It uses at most length bytes from raw and stores the result in
+ *      dest. The result will be no more than maxUse characters.
+ *
+ * Results:
+ *      Returns non-zero if the encoding was successful.
+ *
+ * Side effects:
+ *	None.
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+static int
+CreateEncWord(Tcl_Interp *interp, Tcl_Encoding enc, const char *charset,
+	      unsigned char *raw, int length, Tcl_DString *dest, int maxUse)
+{
+    unsigned char buf[RFC2047_MAX_ENCODED_WORD_LENGTH+1],
+	buf2[RFC2047_MAX_ENCODED_WORD_LENGTH+1];
+    Tcl_EncodingState state;
+    int i, consumed, wrote, d;
+    
+    /*
+     * Check if we must encode this
+     */
+    for (i=0; i<length && raw[i] < 0x80; i++);
+    if (i == length) {
+	Tcl_DStringAppend(dest, raw, length);
+	return 1;
+    }
+
+    /*
+     * Nope, we must encode this. Adjust the max output size
+     */
+    if (maxUse > sizeof(buf)-1) {
+	maxUse = sizeof(buf)-1;
+    }
+
+    /*
+     * Try to convert to external encoding
+     */
+    if (TCL_OK != Tcl_UtfToExternal(interp, enc, raw, length,
+				    TCL_ENCODING_START|TCL_ENCODING_END,
+				    &state, buf2, sizeof(buf2),
+				    &consumed, &wrote, NULL)
+	|| consumed != length) {
+	return 0;
+    }
+
+    /*
+     * Convert into quoted-printable, check that we have room all the time
+     */
+    snprintf(buf, sizeof(buf), "=?%s?Q?", charset);
+    for (i=0, d=strlen(buf); i<wrote && d < maxUse-2; i++) {
+	if (' ' == buf2[i]) {
+	    buf[d++] = '_';
+	} else if (buf2[i] & 0x80
+	    || !isprint(buf2[i])
+	    || '=' == buf2[i]
+	    || '_' == buf2[i]
+	    || '?' == buf2[i]) {
+	    if (d+3 >= maxUse-2) {
+		return 0;
+	    }
+	    buf[d++] = '=';
+	    buf[d++] = alphabetHEX[buf2[i]>>4];
+	    buf[d++] = alphabetHEX[buf2[i]&0xf];
+	} else {
+	    buf[d++] = buf2[i];
+	}
+    }
+    if (i < wrote) {
+	return 0;
+    }
+    buf[d++] = '?';
+    buf[d++] = '=';
+    Tcl_DStringAppend(dest, buf, d);
+    return 1;
+
+}
+
 /*
  *----------------------------------------------------------------------
  *
  * RatEncodeHeaderLine --
  *
- *	Encodes one header line according to MIME.
+ *	Encodes one header line according to MIME (rfc2047).
  *	The nameLength argument should tell how long the header name is in
  *	characters. This is so that the line folding can do its job properly.
  *
  * Results:
- *	A block of encoded header line. It is the callers responsibility to
- *	free this block later with a call to ckfree().
+ *	A block of encoded header line. THis block of data will be valid
+ *      until the next call to thius function.
  *
  * Side effects:
  *	None.
@@ -356,86 +444,87 @@ RatDecode(Tcl_Interp *interp, int cte, char *data, int length, char *charset)
 char*
 RatEncodeHeaderLine (Tcl_Interp *interp, Tcl_Obj *line, int nameLength)
 {
-    unsigned char *buf, *srcPtr, *dstPtr, *startPtr, *endPtr;
-    int length, dstLen, used, l;
-    Tcl_DString ds;
-    char *charset;
+    static Tcl_DString ds;
+    static int initialized = 0;
+    Tcl_Obj **objv;
+    int i, objc, l, l1, pre = nameLength, maxUse;
+    char *s;
+    const char *charset;
+    Tcl_Encoding enc;
 
     if (NULL == line) {
 	return NULL;
     }
 
-    /* TODO, handle arbitrarly long lines */
-    Tcl_GetStringFromObj(line, &dstLen);
-    dstLen += 1024;
-    dstPtr = buf = (unsigned char*)ckalloc(dstLen);
-
-    /*
-     * TODO, should select charset instead
-     */
-    charset = Tcl_GetVar2(interp, "option", "charset", TCL_GLOBAL_ONLY);
-
-    /*
-     * Do nothing if we are not initialized first
-     */
-    if (NULL == charset) {
-	return NULL;
+    if (!initialized) {
+	Tcl_DStringInit(&ds);
+	initialized = 1;
+    } else {
+	Tcl_DStringSetLength(&ds, 0);
     }
 
     /*
-     * Now do the encoding
+     * Find suitable encoding
      */
-    Tcl_DStringInit(&ds);
-    length = nameLength;
-    used = 0;
-    for (srcPtr = (unsigned char*)Tcl_GetString(line); *srcPtr;) {
-	if (0x80 & *srcPtr) {
-	    startPtr = (unsigned char*)Tcl_GetString(line);
-	    dstPtr = buf;
-	    used = 0;
-	    endPtr = startPtr + strlen((char*)startPtr);
-	    length = nameLength;
-	    sprintf((char*)dstPtr, "=?%s?Q?", charset);
-	    length += strlen((char*)dstPtr);
-	    if (length > 70) {
-		sprintf((char*)dstPtr, "\n =?%s?Q?", charset);
-		length = -1;
-	    }
-	    dstPtr += strlen((char*)dstPtr);
-	    Tcl_UtfToExternalDString(RatGetEncoding(interp, charset),
-		    (char*)startPtr, endPtr-startPtr, &ds);
-	    for (srcPtr = (unsigned char*)Tcl_DStringValue(&ds),
-		    l = Tcl_DStringLength(&ds); l > 0; srcPtr++, l--) {
-		if (length > 70) {
-		    sprintf((char*)dstPtr, "?=\n =?%s?Q?", charset);
-		    length = strlen((char*)dstPtr+3);
-		    dstPtr += strlen((char*)dstPtr);
-		}
-		if (' ' == *srcPtr) {
-		    *dstPtr++ = '_';
-		    length++;
-		} else if (*srcPtr & 0x80 || *srcPtr == '_'
-			|| *srcPtr == '?' || *srcPtr == '=') {
-		    *dstPtr++ = '=';
-		    *dstPtr++ = alphabetHEX[*srcPtr >> 4];
-		    *dstPtr++ = alphabetHEX[*srcPtr & 0x0f];
-		    length += 3;
-		} else {
-		    *dstPtr++ = *srcPtr;
-		    length++;
-		}
-	    }
-	    Tcl_DStringSetLength(&ds, 0);
-	    srcPtr = endPtr;
-	    *dstPtr++ = '?';
-	    *dstPtr++ = '=';
-	} else {
-	    *dstPtr++ = *srcPtr++;
-	    length++;
+    Tcl_ListObjGetElements(interp,
+			   Tcl_GetVar2Ex(interp, "option",
+					 "charset_candidates",
+					 TCL_GLOBAL_ONLY),
+			   &objc, &objv);
+    s = Tcl_GetStringFromObj(line, &l);
+    for (i=0; i<objc; i++) {
+	if (RatCheckEncoding(interp, Tcl_GetString(objv[i]), s, l)) {
+	    break;
 	}
     }
-    *dstPtr = '\0';
-    return (char*)buf;
+    if (i<objc) {
+	charset = Tcl_GetString(objv[i]);
+    } else {
+	charset = Tcl_GetVar2(interp, "option", "charset", TCL_GLOBAL_ONLY);
+    }
+    enc = RatGetEncoding(interp, charset);
+
+    /*
+     * Do while we have characters left to consume
+     *  - Find candidate for line-break
+     *  - Loop while it can NOT be encoded into a word
+     *    - Search backwards for new canidate
+     *    - If no new canididate is found switch to test every character
+     */
+    while (*s) {
+	if (strlen(s)+pre <= RFC2047_MAX_LINE_LENGTH) {
+	    l = strlen(s);
+	} else {
+	    for (l = RFC2047_MAX_LINE_LENGTH-pre; l>0 && !isspace(s[l]); l--);
+	    if (0 == l) {
+		l = RFC2047_MAX_LINE_LENGTH-pre;
+	    }
+	}
+	maxUse = RFC2047_MAX_LINE_LENGTH-pre;
+	while (!CreateEncWord(interp, enc, charset, s, l, &ds, maxUse)) {
+	    for (l1 = l-1; l1 > 0 && !isspace(s[l1]); l1--);
+	    if (0 < l1) {
+		l = l1;
+	    } else {
+		maxUse = 1024;
+		l--;
+	    }
+	}
+	s += l;
+	if (*s) {
+	    Tcl_DStringAppend(&ds, "\r\n", 2);
+	    for (pre=0; isspace(*s) && pre<RFC2047_MAX_LINE_LENGTH; s++,pre++){
+		Tcl_DStringAppend(&ds, s, 1);
+	    }
+	    if (0 == pre) {
+		Tcl_DStringAppend(&ds, " ", 1);
+		pre = 1;
+	    }
+	}
+    }
+    
+    Tcl_FreeEncoding(enc);
+    return Tcl_DStringValue(&ds);
 }
 
 /*
@@ -480,12 +569,14 @@ RatEncodeAddresses(Tcl_Interp *interp, ADDRESS *adrPtr)
 /*
  *----------------------------------------------------------------------
  *
- * RatCheckEncodingsCmd --
+ * RatGetEncoding --
  *
- *	See ../doc/interface for a descriptions of arguments and result.
+ *	Return the tcl-encoding attached to the given name. This name
+ *      may be mapped from a MIME-name into a tcl-name.
  *
  * Results:
- *      See above
+ *      A tcl Tcl_Endoding blob. The given encoding must be freed by the
+ *      caller by calling Tcl_FreeEncoding().
  *
  * Side effects:
  *      None.
@@ -495,45 +586,72 @@ RatEncodeAddresses(Tcl_Interp *interp, ADDRESS *adrPtr)
  */
 
 Tcl_Encoding
-RatGetEncoding(Tcl_Interp *interp, char *name)
+RatGetEncoding(Tcl_Interp *interp, const char *name)
 {
-    static RatEncoding *encPtr = NULL;
-    RatEncoding *ePtr;
-    Tcl_Encoding tenc;
-    char *tclName, lname[256];
+    Tcl_Encoding enc;
+    const char *tclName;
+    char lname[256];
 
     if (NULL == name) {
 	return NULL;
     }
 
-    RatStrNCpy(lname, name, sizeof(lname));
+    strlcpy(lname, name, sizeof(lname));
     lcase(lname);
     tclName = Tcl_GetVar2(interp, "charsetMapping", lname, TCL_GLOBAL_ONLY);
     if (NULL == tclName) {
 	tclName = lname;
     }
 
-    for (ePtr = encPtr; ePtr; ePtr = ePtr->nextPtr) {
-	if (!strcmp(ePtr->name, tclName)) {
-	    return ePtr->enc;
-	}
-    }
-
-    tenc = Tcl_GetEncoding(interp, tclName);
-    if (NULL == tenc) {
+    enc = Tcl_GetEncoding(interp, tclName);
+    if (NULL == enc) {
 	return NULL;
     }
-    ePtr = (RatEncoding*)ckalloc(sizeof(RatEncoding));
-    ePtr->name = cpystr(tclName);
-    ePtr->enc = tenc;
-    ePtr->nextPtr = encPtr;
-    encPtr = ePtr;
-#ifdef MEM_DEBUG
-    mem_encPtr = encPtr;
-#endif /* MEM_DEBUG */
-    return ePtr->enc;
+    return enc;
 }
 
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * RatCheckEncoding --
+ *
+ *	Check if the given encoding can encode the given string
+ *
+ * Results:
+ *      Non-zero if all characters in the give string can be encoded
+ *      successfully
+ *
+ * Side effects:
+ *      None.
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+static int
+RatCheckEncoding(Tcl_Interp *interp, char *encoding_name,
+		 const char *string, int length)
+{
+    Tcl_EncodingState state;
+    Tcl_Encoding enc;
+    char buf[1024];
+    int ret, in;
+
+    if (NULL == (enc = RatGetEncoding(interp, encoding_name))) {
+	return 0;
+    }
+    ret = 0;
+    while (length && TCL_CONVERT_UNKNOWN != ret) {
+	ret = Tcl_UtfToExternal(interp, enc, string, length,
+				TCL_ENCODING_STOPONERROR|TCL_ENCODING_START,
+				&state, buf, sizeof(buf),
+				&in, NULL, NULL);
+	string += in;
+	length -= in;
+    }
+    Tcl_FreeEncoding(enc);
+    return TCL_CONVERT_UNKNOWN != ret;
+}
 
 /*
  *----------------------------------------------------------------------
@@ -556,12 +674,10 @@ int
 RatCheckEncodingsCmd(ClientData dummy, Tcl_Interp *interp, int objc,
 	Tcl_Obj *const objv[])
 {
-    int i, listLength, srcLen, in, ret;
+    int i, listLength, srcLen;
     Tcl_Obj *oPtr, *vPtr;
-    Tcl_EncodingState state;
-    char buf[1024], *src;
-    Tcl_Encoding enc;
-
+    char *src;
+    
     if (3 != objc) {
 	Tcl_AppendResult(interp, "Usage: ", Tcl_GetString(objv[0]), \
 		" variable charsets", (char*) NULL);
@@ -569,22 +685,10 @@ RatCheckEncodingsCmd(ClientData dummy, Tcl_Interp *interp, int objc,
     }
     vPtr = Tcl_GetVar2Ex(interp, Tcl_GetString(objv[1]), NULL, 0);
     Tcl_ListObjLength(interp, objv[2], &listLength);
+    src = Tcl_GetStringFromObj(vPtr, &srcLen);
     for (i=0; i<listLength; i++) {
 	Tcl_ListObjIndex(interp, objv[2], i, &oPtr);
-	if (NULL == (enc = RatGetEncoding(interp, Tcl_GetString(oPtr)))) {
-	    continue;
-	}
-	src = Tcl_GetStringFromObj(vPtr, &srcLen);
-	state = 0;
-	ret = 0;
-	while (srcLen && TCL_CONVERT_UNKNOWN != ret) {
-	    ret = Tcl_UtfToExternal(interp, enc, src, srcLen,
-			TCL_ENCODING_STOPONERROR, &state, buf, sizeof(buf),
-			&in, NULL, NULL);
-	    src += in;
-	    srcLen -= in;
-	}
-	if (TCL_CONVERT_UNKNOWN != ret) {
+	if (RatCheckEncoding(interp, Tcl_GetString(oPtr),  src, srcLen)) {
 	    Tcl_SetObjResult(interp, oPtr);
 	    return TCL_OK;
 	}
@@ -617,15 +721,15 @@ RatCode64(Tcl_Obj *sPtr)
     unsigned char *cPtr, buf[4];
     int l, ll;
 
-    cPtr = Tcl_GetStringFromObj(sPtr, &l);
+    cPtr = (unsigned char*)Tcl_GetStringFromObj(sPtr, &l);
 
     for (ll = 0; l > 0; l -= 3, cPtr += 3) {
 	buf[0] = alphabet64[cPtr[0] >> 2];
-	buf[1] = alphabet64[((cPtr[0] << 4) + (l>1 ? (cPtr[1]>>4) : 0)) & 0x3f];
+	buf[1] = alphabet64[((cPtr[0] << 4) + (l>1 ? (cPtr[1]>>4) : 0))&0x3f];
 	buf[2] = l > 1 ?
 	    alphabet64[((cPtr[1]<<2) + (l>2 ? (cPtr[2]>>6) : 0)) & 0x3f] : '=';
 	buf[3] = l > 2 ? alphabet64[cPtr[2] & 0x3f] : '=';
-	Tcl_AppendToObj(dPtr, buf, 4);
+	Tcl_AppendToObj(dPtr, (char*)buf, 4);
 	if (18 == ++ll || l < 4) {
 	    Tcl_AppendToObj(dPtr, "\n", 1);
 	    ll = 0;
@@ -725,8 +829,7 @@ RatUtf16to8(const unsigned char *src, unsigned char *dst)
  *	Convert the given utf-8 encoded text to modified utf-7
  *
  * Results:
- *      Returns the length of the generated string on success.
- *	On failure a negative number is returned.
+ *      Returns a pointer to a static buffer containing the new text
  *
  * Side effects:
  *      None.
@@ -735,16 +838,16 @@ RatUtf16to8(const unsigned char *src, unsigned char *dst)
  *----------------------------------------------------------------------
  */
 
-unsigned char*
-RatUtf8toMutf7(const unsigned char *src)
+char*
+RatUtf8toMutf7(const char *signed_src)
 {
     static unsigned char *dst = NULL;
     static int dstlen = 0;
-    unsigned char buf[3];
+    unsigned char buf[3], *src = (unsigned char*)signed_src;
     int len = 0, overflow = 0;
 
-    if (dstlen < strlen(src)*3) {
-	dstlen = strlen(src)*3;
+    if (dstlen < strlen((char*)src)*3+1) {
+	dstlen = strlen((char*)src)*3;
 	dst = (unsigned char *)ckrealloc(dst, dstlen);
     }
     while (*src) {
@@ -806,7 +909,7 @@ RatUtf8toMutf7(const unsigned char *src)
 	}
     }
     dst[len] = '\0';
-    return dst;
+    return (char*)dst;
 }
 
 /*
@@ -827,16 +930,16 @@ RatUtf8toMutf7(const unsigned char *src)
  *----------------------------------------------------------------------
  */
 
-unsigned char*
-RatMutf7toUtf8(const unsigned char *src)
+char*
+RatMutf7toUtf8(const char *signed_src)
 {
     static unsigned char *dst = NULL;
     static int dstlen = 0;
-    unsigned char utf16[2], lbuf[4];
+    unsigned char utf16[2], lbuf[4], *src = (unsigned char*)signed_src;
     int i, l, len=0, odd;
 
-    if (dstlen < strlen(src)*3) {
-	dstlen = strlen(src)*3;
+    if (dstlen < strlen((char*)src)*3) {
+	dstlen = strlen((char*)src)*3;
 	dst = (unsigned char *)ckrealloc(dst, dstlen);
     }
     while (*src) {
@@ -892,7 +995,159 @@ RatMutf7toUtf8(const unsigned char *src)
 	}
     }
     dst[len] = '\0';
-    return dst;
+    return (char*)dst;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * RatEncodeQP -
+ *
+ *	Encode the given text to QP
+ *
+ * Results:
+ *      Returns an intialized Tcl_DString pointer. It is up to the caller to
+ *      free this when not needing it anymore.
+ *
+ * Side effects:
+ *      None.
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+Tcl_DString*
+RatEncodeQP(const unsigned char *line)
+{
+    Tcl_DString *ds = (Tcl_DString*)ckalloc(sizeof(*ds));
+    const unsigned char *c;
+    unsigned char buf[4];
+
+    Tcl_DStringInit(ds);
+    for (c=line; *c; c++) {
+	if ('=' == *c || 0x80 <= *c) {
+	    snprintf(buf, sizeof(buf), "=%02X", *c);
+	    Tcl_DStringAppend(ds, buf, 3);
+	} else {
+	    Tcl_DStringAppend(ds, c, 1);
+	}
+    }
+    return ds;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * RatEncodeQPCmd --
+ *
+ *	See ../doc/interface
+ *
+ * Results:
+ *      A standard tcl result
+ *
+ * Side effects:
+ *      None
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+RatEncodeQPCmd(ClientData dummy, Tcl_Interp *interp, int objc,
+	   Tcl_Obj *const objv[])
+{
+    Tcl_Encoding enc;
+    Tcl_DString ext, *encoded;
+    
+    if (objc != 3) {
+ 	Tcl_AppendResult(interp, "Bad usage", TCL_STATIC);
+	return TCL_ERROR;
+    }
+
+    enc = Tcl_GetEncoding(interp, Tcl_GetString(objv[1]));
+    Tcl_UtfToExternalDString(enc, Tcl_GetString(objv[2]), -1, &ext);
+    encoded = RatEncodeQP(Tcl_DStringValue(&ext));
+    Tcl_DStringFree(&ext);
+    Tcl_DStringResult(interp, encoded);
+    Tcl_FreeEncoding(enc);
+    ckfree(encoded);
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * RatDecodeQP -
+ *
+ *	Dencode the given text from QP
+ *
+ * Results:
+ *      Returns a pointer to a string. This string has been allocated with
+ *      ckalloc and it is up to the caller to free it when not needing it.
+ *
+ * Side effects:
+ *      None.
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+
+unsigned char*
+RatDecodeQP(unsigned char *line)
+{
+    unsigned char *s, *d;
+
+    d = s = line;
+    while (*s) {
+	if ('=' == *s && isxdigit(s[1]) && isxdigit(s[2])) {
+	    *d++ = ((strchr(alphabetHEX, s[1])-alphabetHEX)<<4) +
+		(strchr(alphabetHEX, s[2])-alphabetHEX);
+	    s += 3;
+	} else {
+	    *d++ = *s++;
+	}
+    }
+    *d = '\0';
+    return line;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * RatDecodeQPCmd --
+ *
+ *	See ../doc/interface
+ *
+ * Results:
+ *      A standard tcl result
+ *
+ * Side effects:
+ *      None
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+RatDecodeQPCmd(ClientData dummy, Tcl_Interp *interp, int objc,
+	   Tcl_Obj *const objv[])
+{
+    Tcl_Encoding enc;
+    Tcl_DString utf;
+    char *text;
+
+    if (objc != 3) {
+ 	Tcl_AppendResult(interp, "Bad usage", TCL_STATIC);
+	return TCL_ERROR;
+    }
+
+    enc = Tcl_GetEncoding(interp, Tcl_GetString(objv[1]));
+    text = cpystr(Tcl_GetString(objv[2]));
+    RatDecodeQP(text);
+    Tcl_ExternalToUtfDString(enc, text, -1, &utf);
+    ckfree(text);
+    Tcl_DStringResult(interp, &utf);
+    Tcl_FreeEncoding(enc);
+    return TCL_OK;
 }
 
 /*
@@ -926,16 +1181,3 @@ int main()
 
     return 0;
 } */
-
-#ifdef MEM_DEBUG
-void ratCodeCleanup()
-{
-    RatEncoding *e, *n;
-
-    for (e = mem_encPtr; e; e = n) {
-	n = e->nextPtr;
-	ckfree(e->name);
-	ckfree(e);
-    }
-}
-#endif /* MEM_DEBUG */
