@@ -3,7 +3,7 @@
  *
  *	This file contains compatibility functions.
  *
- * TkRat software and its included text is Copyright 1996-2000 by
+ * TkRat software and its included text is Copyright 1996-2002 by
  * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
@@ -34,3 +34,37 @@ snprintf (char *buf, size_t buflen, const char *fmt, ...)
 }
 
 #endif /* HAVE_SNPRINTF */
+
+#ifndef HAVE_STRLCPY
+
+char*
+strlcpy(char *dst, const char *src, size_t n)
+{
+    int i;
+
+    for (i=0; src[i] && i<n-1; i++) {
+	dst[i] = src[i];
+    }
+    dst[i] = '\0';
+    return dst;
+}
+
+#endif /* HAVE_STRLCPY */
+
+#ifndef HAVE_STRLCAT
+
+char*
+strlcat(char *dst, const char *src, size_t n)
+{
+    int i;
+    const char *c;
+
+    for (i=0; dst[i] && i<n-1; i++);
+    for (c = src; *c && i<n-1; i++, c++) {
+	dst[i] = *c;
+    }
+    dst[i] = '\0';
+    return dst;
+}
+
+#endif /* HAVE_STRLCAT */
