@@ -2632,7 +2632,7 @@ RatDbCheck(Tcl_Interp *interp, int fix)
 	    sprintf(buf, "%d", itemPtr->fileSize);
 	    itemPtr->entry.content[RSIZE] =extraPtrPtr[extraNum++]=cpystr(buf);
 	    itemPtr->entry.content[STATUS] = flags ? flags : "";
-	    sprintf(buf, "%ld", time(NULL)+60*60*24*100);
+	    sprintf(buf, "%ld", time(NULL) + 60L*60L*24L*100L);
 	    itemPtr->entry.content[EX_TIME] = extraPtrPtr[extraNum++] =
 		cpystr(buf);
 	    itemPtr->entry.content[EX_TYPE] = "backup";
