@@ -3,7 +3,7 @@
  *
  *	This file handles the expressions.
  *
- * TkRat software and its included text is Copyright 1996-2000 by
+ * TkRat software and its included text is Copyright 1996-2002 by
  * Martin Forssn
  *
  * The full text of my legal notices is contained in the file called
@@ -15,7 +15,7 @@
 /*
  * Private types
  */
-typedef enum {TT_Field, TT_Operator, TT_Boolean, TT_Grouping, TT_Spec}TokenType;
+typedef enum {TT_Field, TT_Operator, TT_Boolean,TT_Grouping,TT_Spec} TokenType;
 typedef enum {T_To, T_From, T_Subject, T_Sender, T_Cc, T_Reply_To, T_Size,
 	      T_Has, T_Is, T_Gt, T_Lt,
 	      T_And, T_Or, T_Not,
@@ -79,7 +79,8 @@ static TokenList   *GetToken(char **sPtr);
 static char 	   *GetString(char **sPtr);
 static void	    FreeExp(Expression *expPtr);
 static Expression  *ParseExpression(char **sPtr, char **errPtr, int inParen);
-static void	    GetExpression(Tcl_DString *dsPtr, Expression *expPtr);
+static void	    GetExpression(Tcl_Interp *interp, Tcl_Obj *ePtr,
+				  Expression *expPtr);
 static int	    RatExpMatchDo(Tcl_Interp *interp, Expression *expPtr,
 		    RatInfoProc *infoProc, ClientData clientData, int index);
 
@@ -117,7 +118,8 @@ GetToken(char **sPtr)
 	return NULL;
     }
     for (i=0; tokenList[i].string; i++) {
-	if (!strncasecmp(cPtr,tokenList[i].string,strlen(tokenList[i].string))){
+	if (!strncasecmp(cPtr, tokenList[i].string,
+			 strlen(tokenList[i].string))) {
 	    cPtr += strlen(tokenList[i].string);
 	    *sPtr = cPtr;
 	    return &tokenList[i];
@@ -212,9 +214,7 @@ FreeExp(Expression *expPtr)
 	FreeExp(expPtr->arg1.expPtr);
 	FreeExp(expPtr->arg2.expPtr);
     } else {
-	if (expPtr->arg2.string) {
-	    ckfree(expPtr->arg2.string);
-	}
+	ckfree(expPtr->arg2.string);
     }
     ckfree(expPtr);
 }
@@ -242,7 +242,7 @@ ParseExpression(char **sPtr, char **errPtr, int inParen)
     TokenList *tokPtr;
     Expression *expPtr = NULL, *exp2Ptr;
     char *newString;
-    int negated;
+    int negated, l;
 
     while (**sPtr) {
 	negated = 0;
@@ -278,10 +278,11 @@ ParseExpression(char **sPtr, char **errPtr, int inParen)
 		    return expPtr;
 		} else if (T_Is == exp2Ptr->op) {
 		    exp2Ptr->op = T_Has;
-		    newString = (char*)ckalloc(strlen(exp2Ptr->arg2.string)+3);
-		    strcpy(newString, "^");
-		    strcat(newString, exp2Ptr->arg2.string);
-		    strcat(newString, "$");
+		    l = strlen(exp2Ptr->arg2.string)+3;
+		    newString = (char*)ckalloc(l);
+		    strlcpy(newString, "^", l);
+		    strlcat(newString, exp2Ptr->arg2.string, l);
+		    strlcat(newString, "$", l);
 		    ckfree(exp2Ptr->arg2.string);
 		    exp2Ptr->arg2.string = newString;
 		}
@@ -358,26 +359,27 @@ ParseExpression(char **sPtr, char **errPtr, int inParen)
  */
 
 int
-RatParseExpCmd(ClientData clientData, Tcl_Interp *interp, int argc,
-	char *argv[])
+RatParseExpCmd(ClientData clientData, Tcl_Interp *interp, int objc,
+	       Tcl_Obj *const objv[])
 {
-    char *error = NULL, *cPtr;
+    char *error = NULL, *cPtr, *exp;
     Expression *expPtr;
     ExpList *elemPtr;
 
-    if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		" expression\"", (char *) NULL);
+    if (objc < 2) {
+	Tcl_AppendResult(interp, "wrong # args: should be \"",
+			 Tcl_GetString(objv[0]), " expression\"",
+			 (char *) NULL);
 	return TCL_ERROR;
     }
 
-    cPtr = argv[1];
+    exp = cPtr = Tcl_GetString(objv[1]);
     expPtr = ParseExpression(&cPtr, &error, 0);
     if (error) {
 	char buf[32];
 
 	FreeExp(expPtr);
-	sprintf(buf, "%d", cPtr-argv[1]);
+	sprintf(buf, "%d", cPtr-exp);
 	Tcl_AppendElement(interp, buf);
 	Tcl_AppendElement(interp, error);
 	return TCL_ERROR;
@@ -411,28 +413,33 @@ RatParseExpCmd(ClientData clientData, Tcl_Interp *interp, int argc,
  */
 
 static void
-GetExpression(Tcl_DString *dsPtr, Expression *expPtr)
+GetExpression(Tcl_Interp *interp, Tcl_Obj *ePtr, Expression *expPtr)
 {
     int opIndex, fIndex;
+    Tcl_Obj *oPtr;
 
     for (opIndex=0; tokenList[opIndex].token != expPtr->op; opIndex++);
 
     if (expPtr->negate) {
-	Tcl_DStringAppendElement(dsPtr, "not");
+	Tcl_ListObjAppendElement(interp, ePtr, Tcl_NewStringObj("not", 3));
     }
     if (tokenList[opIndex].type == TT_Boolean) {
-	Tcl_DStringStartSublist(dsPtr);
-	GetExpression(dsPtr, expPtr->arg1.expPtr);
-	Tcl_DStringEndSublist(dsPtr);
-	Tcl_DStringAppendElement(dsPtr, tokenList[opIndex].string);
-	Tcl_DStringStartSublist(dsPtr);
-	GetExpression(dsPtr, expPtr->arg2.expPtr);
-	Tcl_DStringEndSublist(dsPtr);
+	oPtr = Tcl_NewObj();
+	GetExpression(interp, oPtr, expPtr->arg1.expPtr);
+	Tcl_ListObjAppendElement(interp, ePtr, oPtr);
+	Tcl_ListObjAppendElement(interp, ePtr,
+			      Tcl_NewStringObj(tokenList[opIndex].string, -1));
+	oPtr = Tcl_NewObj();
+	GetExpression(interp, oPtr, expPtr->arg2.expPtr);
+	Tcl_ListObjAppendElement(interp, ePtr, oPtr);
     } else {
 	for (fIndex=0; tokenList[fIndex].info != expPtr->arg1.info; fIndex++);
-	Tcl_DStringAppendElement(dsPtr, tokenList[fIndex].string);
-	Tcl_DStringAppendElement(dsPtr, tokenList[opIndex].string);
-	Tcl_DStringAppendElement(dsPtr, expPtr->arg2.string);
+	Tcl_ListObjAppendElement(interp, ePtr,
+			      Tcl_NewStringObj(tokenList[fIndex].string, -1));
+	Tcl_ListObjAppendElement(interp, ePtr,
+			      Tcl_NewStringObj(tokenList[opIndex].string, -1));
+	Tcl_ListObjAppendElement(interp, ePtr,
+			      Tcl_NewStringObj(expPtr->arg2.string, -1));
     }
 }
 
@@ -454,29 +461,30 @@ GetExpression(Tcl_DString *dsPtr, Expression *expPtr)
  */
 
 int
-RatGetExpCmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
+RatGetExpCmd(ClientData clientData, Tcl_Interp *interp, int objc,
+	     Tcl_Obj *const objv[])
 {
     ExpList *elemPtr;
-    Tcl_DString ds;
+    Tcl_Obj *rPtr;
     int id;
 
-    if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		" id\"", (char *) NULL);
+    if (objc < 2
+	|| TCL_OK != Tcl_GetIntFromObj(interp, objv[1], &id)) {
+	Tcl_AppendResult(interp, "wrong # args: should be \"",
+			 Tcl_GetString(objv[0]), " id\"", (char *) NULL);
 	return TCL_ERROR;
     }
 
-    id = atoi(argv[1]);
     for (elemPtr = expListPtr; elemPtr; elemPtr = elemPtr->next) {
 	if (elemPtr->id == id) {
-	    Tcl_DStringInit(&ds);
-	    GetExpression(&ds, elemPtr->expPtr);
-	    Tcl_DStringResult(interp, &ds);
+	    rPtr = Tcl_NewObj();
+	    GetExpression(interp, rPtr, elemPtr->expPtr);
+	    Tcl_SetObjResult(interp, rPtr);
 	    return TCL_OK;
 	}
     }
-    Tcl_AppendResult(interp, "No expression with id \"", argv[1],
-	    "\"", (char *) NULL);
+    Tcl_AppendResult(interp, "No expression with id \"",
+		     Tcl_GetString(objv[1]), "\"", (char *) NULL);
     return TCL_ERROR;
 }
 
@@ -499,18 +507,20 @@ RatGetExpCmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
  */
 
 int
-RatFreeExpCmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
+RatFreeExpCmd(ClientData clientData, Tcl_Interp *interp, int objc,
+	      Tcl_Obj *const objv[])
 {
     ExpList **elemPtrPtr, *elemPtr;
     int id;
 
-    if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		" id\"", (char *) NULL);
+    if (objc < 2
+	|| TCL_OK != Tcl_GetIntFromObj(interp, objv[1], &id)) {
+	Tcl_AppendResult(interp, "Illegal usage: should be \"",
+			 Tcl_GetString(objv[0]), " id\"", (char *) NULL);
 	return TCL_ERROR;
     }
-    id = atoi(argv[1]);
-    for (elemPtrPtr = &expListPtr; *elemPtrPtr;elemPtrPtr=&(*elemPtrPtr)->next){
+    for (elemPtrPtr = &expListPtr; *elemPtrPtr;
+	 elemPtrPtr=&(*elemPtrPtr)->next){
 	if ((*elemPtrPtr)->id == id) {
 	    elemPtr = *elemPtrPtr;
 	    FreeExp(elemPtr->expPtr);
