@@ -243,6 +243,7 @@ RatOpenFolder(ClientData clientData, Tcl_Interp *interp, int objc,
 	infoPtr = RatStdFolderCreate(interp, objv[1]);
     }
     if (NULL == infoPtr) {
+	Tcl_AppendResult(interp, "Failed to create folder", NULL);
 	return TCL_ERROR;
     }
     Tcl_ListObjGetElements(interp, fobjv[2], &lobjc, &lobjv);
@@ -1953,6 +1954,9 @@ RatGetFolderSpec(Tcl_Interp *interp, Tcl_Obj *def)
     }
 
     Tcl_ListObjGetElements(interp, def, &objc, &objv);
+    if (objc < 4) {
+	return NULL;
+    }
     type = Tcl_GetString(objv[1]);
     if (!strcmp(type, "file")) {
 	file = Tcl_TranslateFileName(interp, Tcl_GetString(objv[3]), &tmpDS);
@@ -1974,6 +1978,9 @@ RatGetFolderSpec(Tcl_Interp *interp, Tcl_Obj *def)
 	Tcl_DStringAppend(&ds, file, -1);
 	ckfree(file);
     } else if (!strcmp(type, "dbase")) {
+	if (objc < 6) {
+	    return NULL;
+	}
 	Tcl_DStringAppend(&ds, Tcl_GetString(objv[3]), -1);
 	Tcl_DStringAppend(&ds, Tcl_GetString(objv[4]), -1);
 	Tcl_DStringAppend(&ds, Tcl_GetString(objv[5]), -1);
@@ -1982,6 +1989,9 @@ RatGetFolderSpec(Tcl_Interp *interp, Tcl_Obj *def)
 	       || !strcmp(type, "dis")) {
 	oPtr = Tcl_GetVar2Ex(interp, "mailServer", Tcl_GetString(objv[3]),
 			     TCL_GLOBAL_ONLY);
+	if (!oPtr) {
+	    return NULL;
+	}
 	Tcl_ListObjGetElements(interp, oPtr, &mobjc, &mobjv);
 	Tcl_DStringAppend(&ds, "{", 1);
 	Tcl_DStringAppend(&ds, Tcl_GetString(mobjv[0]),
