@@ -3,7 +3,7 @@
  *
  *      Declarations of types used in the folder and messages system.
  *
- * TkRat software and its included text is Copyright 1996-2000 by
+ * TkRat software and its included text is Copyright 1996-2002 by
  * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
@@ -21,10 +21,12 @@
  */
 typedef enum {
     RAT_FOLDER_SUBJECT,		/* The subject of the message */
-    RAT_FOLDER_CANONSUBJECT,	/* The canonical subject of the message
-				   This is the subject without any leading Re:*/
-    RAT_FOLDER_NAME,		/* The full name of the sender (From:) if
-				 * available otherwise the mail address */
+    RAT_FOLDER_CANONSUBJECT,	/* The canonical subject of the message. This
+				   is the subject without any leading Re:*/
+    RAT_FOLDER_NAME,		/* The full name of the sender (From:)
+				   or To: if From: is me. Both fall
+				   back to "RAT_FOLDER_MAIL" below
+				   when no full name is present */
     RAT_FOLDER_MAIL_REAL,	/* The mail address of the sender (From:) */
     RAT_FOLDER_MAIL,		/* The mail address of the sender (From:)
 				   or To: if From: is me */
@@ -58,8 +60,8 @@ typedef enum {
 /*
  * These are the possible flags.
  *
- * OBSERVER: if you change this list you MUST update the stdFlagNames array
- * in ratStdFolder.c
+ * OBSERVE: if you change this list you MUST update the flag_name array
+ * in ratFolder.c
  */
 typedef enum {
     RAT_SEEN,			/* Message content has been seen by the user */
@@ -70,11 +72,12 @@ typedef enum {
     RAT_RECENT			/* Message is a not seen but has been in the
 				   folder for some time */
 } RatFlag;
-#define RAT_SEEN_STR		"\\Seen"
-#define RAT_DELETED_STR		"\\Deleted"
-#define RAT_FLAGGED_STR		"\\Flagged"
-#define RAT_ANSWERED_STR	"\\Answered"
-#define RAT_DRAFT_STR		"\\Draft"
+typedef struct {
+    char *imap_name;	/* The name of the flag when used with c-client */
+    char *tkrat_name;   /* The name of the flag when used in tkrat */
+    char unix_char;	/* Character representing flag in unix mailboxes */
+} flag_name_t;
+extern flag_name_t flag_name[];
 
 /*
  * The different types of updates which can be performed
@@ -101,6 +104,9 @@ typedef enum {SORT_NONE, SORT_SUBJECT, SORT_SUBJDATE, SORT_THREADED,
  * 	This procedure should initialize the privatePtr part of
  *	the folder structure.
  *
+ * void finalProc(RatFolderInfoPtr infoPtr, Tcl_Interp *interp)
+ *	Does final initialization when creating the folder
+ *
  * int closeProc(RatFolderInfo *infoPtr, Tcl_Interp *interp, int expunge)
  *
  *	This procedure should close the folder and free all private
@@ -157,6 +163,7 @@ typedef enum {SORT_NONE, SORT_SUBJECT, SORT_SUBJDATE, SORT_THREADED,
 
 typedef void (RatInitProc) (RatFolderInfoPtr infoPtr,
 	Tcl_Interp *interp, int index);
+typedef void (RatFinalProc) (RatFolderInfoPtr infoPtr, Tcl_Interp *interp);
 typedef int (RatCloseProc) (RatFolderInfoPtr infoPtr, Tcl_Interp *interp,
 	int expunge);
 typedef int (RatUpdateProc) (RatFolderInfoPtr infoPtr,
@@ -181,12 +188,13 @@ typedef int (RatSyncProc) (RatFolderInfoPtr infoPtr, Tcl_Interp *interp);
  */
 typedef struct RatFolderInfo {
     char *cmdName;		/* Name of the folder command */
-    char *name;			/* Name of folder (this is a pointer which
+    char *name;			/* Mailbox name (this is a pointer which
 				 * may be ckfree():ed) */
     char *type;			/* Type of folder */
     char *definition;		/* Definition id of folder */
     int refCount;		/* Reference count (<=0 means closing) */
     SortOrder sortOrder;	/* Sort order for folder */
+    Tcl_Obj *role;		/* Role to use */
     int sortOrderChanged;	/* Non null if sort order has changed */
     int reverse;		/* If the sort order should be reversed */
     int number;			/* Number of messages in folder */
@@ -214,6 +222,7 @@ typedef struct RatFolderInfo {
     int watcherInterval;	/* Interval between updates */
     Tcl_TimerToken timerToken;	/* Token for update timer */
     RatInitProc *initProc;
+    RatFinalProc *finalProc;
     RatCloseProc *closeProc;
     RatUpdateProc *updateProc;
     RatInsertProc *insertProc;
@@ -337,8 +346,16 @@ typedef struct {
     char *postString;		/* Any character sthat should be appended. */
 } ListExpression;
 
+/*
+ * Folder management operations
+ */
+typedef enum {
+    RAT_MGMT_CREATE,
+    RAT_MGMT_DELETE
+} RatManagementAction;
+
 /* ratFolder.c (note that this file also exports functions in rat.h) */
-extern RatFolderInfo *RatGetOpenFolder(char *def);
+extern RatFolderInfo *RatGetOpenFolder(Tcl_Interp *interp, Tcl_Obj *defPtr);
 extern Tcl_Obj *RatFolderCanonalizeSubject (const char *s);
 extern Tcl_Obj *RatGetMsgInfo(Tcl_Interp *interp, RatFolderInfoType type,
 	MessageInfo *msgPtr, ENVELOPE *envPtr, BODY *bodyPtr,
@@ -346,50 +363,52 @@ extern Tcl_Obj *RatGetMsgInfo(Tcl_Interp *interp, RatFolderInfoType type,
 extern char* MsgFlags(MESSAGECACHE *eltPtr);
 extern MESSAGECACHE *RatParseFrom(const char *from);
 extern int RatFolderClose(Tcl_Interp *interp, RatFolderInfo *infoPtr,
-	int force);
+			  int force);
 extern int RatFolderInsert(Tcl_Interp *interp, RatFolderInfo *infoPtr,
-	int num, char **msgs);
+			   int num, char **msgs);
+extern int RatUpdateFolder(Tcl_Interp *interp, RatFolderInfo *infoPtr,
+			   RatUpdateType mode);
+extern char *RatGetFolderSpec(Tcl_Interp *interp, Tcl_Obj *def);
+extern Tcl_Obj *RatExtractRef(CONST84 char *text);
 
 /* ratStdFolder.c */
-extern int RatStdFolderInit (Tcl_Interp *interp);
-extern RatFolderInfo *RatStdFolderCreate (Tcl_Interp *interp, int objc,
-	Tcl_Obj *CONST objv[]);
-extern MAILSTREAM* OpenStdFolder(Tcl_Interp *interp, char *name, char *prot,
-	char *user, void *stdPtr);
+extern int RatStdFolderInit(Tcl_Interp *interp);
+extern RatFolderInfo *RatStdFolderCreate(Tcl_Interp *interp, Tcl_Obj *defPtr);
+extern MAILSTREAM* OpenStdFolder(Tcl_Interp *interp, char *spec, void *stdPtr);
 extern void CloseStdFolder(Tcl_Interp *interp, MAILSTREAM *stream);
+extern int RatStdManageFolder(Tcl_Interp *interp, RatManagementAction op,
+			      Tcl_Obj *fptr);
+void RatStdCheckNet(Tcl_Interp *interp);
 
 /* ratDbFolder.c */
 extern int RatDbFolderInit (Tcl_Interp *interp);
-extern RatFolderInfo *RatDbFolderCreate (Tcl_Interp *interp, int objc,
-	Tcl_Obj *CONST objv[]);
+extern RatFolderInfo *RatDbFolderCreate(Tcl_Interp *interp, Tcl_Obj *defPtr);
 extern RatInfoProc Db_InfoProc;
 extern Tcl_Obj* Db_InfoProcInt(Tcl_Interp *interp, RatFolderInfo *infoPtr,
 	RatFolderInfoType type, int rIndex);
 
 /* ratDisFolder.c */
 extern int RatDisFolderInit (Tcl_Interp *interp);
-extern MAILSTREAM* RatDisFolderOpenStream (Tcl_Interp *interp, int objc,
-	Tcl_Obj *CONST objv[], char **mailbox);
-extern RatFolderInfo *RatDisFolderCreate (Tcl_Interp *interp, int objc,
-	Tcl_Obj *CONST objv[]);
-extern char* RatDisFolderDir(Tcl_Interp *interp, const char *spec,
-	const char *user, const char *prot);
-extern const char* RatDisPrepareDir(Tcl_Interp *interp, const char *name,
-	const char *folder_spec, const char *user, const char *prot);
+extern MAILSTREAM* RatDisFolderOpenStream(Tcl_Interp *interp, Tcl_Obj *defPtr);
+extern RatFolderInfo *RatDisFolderCreate(Tcl_Interp *interp, Tcl_Obj *defPtr);
+extern char* RatDisFolderDir(Tcl_Interp *interp, Tcl_Obj *defPtr);
+extern int RatDisOnOffTrans(Tcl_Interp *interp, int newState);
+extern void RatDisManageFolder(Tcl_Interp *interp, RatManagementAction op,
+			       Tcl_Obj *fptr);
 
 /* ratMessage.c */
 extern void RatInitMessages (void);
 extern Tcl_ObjCmdProc RatMessageCmd;
 extern BodyInfo *CreateBodyInfo(MessageInfo *msgPtr);
-extern Tcl_CmdProc RatBodyCmd;
+extern Tcl_ObjCmdProc RatBodyCmd;
 extern int RatMessageDelete (Tcl_Interp *interp, char *msgCmd);
 extern void RatMessageGet (Tcl_Interp *interp, MessageInfo *msgPtr,
 	Tcl_DString *ds, char *flags, size_t flaglen, char *date,
 	size_t datelen);
-extern Tcl_CmdProc RatInsertCmd;
+extern Tcl_ObjCmdProc RatInsertCmd;
 extern int RatInsertMsg (Tcl_Interp *interp, MessageInfo *msgPtr,
 	char *keywords, char *exDate, char *exType);
-extern ListExpression *RatParseList(char *format);
+extern ListExpression *RatParseList(const char *format);
 extern void RatFreeListExpression(ListExpression *exprPtr);
 extern Tcl_Obj *RatDoList(Tcl_Interp *interp, ListExpression *exprPtr,
 	RatInfoProc *infoProc, ClientData clientData, int index);
@@ -402,7 +421,7 @@ extern Tcl_Obj *RatBodyData(Tcl_Interp *interp, BodyInfo *bodyInfoPtr,
 	int encoded, char *charset);
 extern MESSAGECACHE *RatMessageInternalDate(Tcl_Interp *interp,
 	MessageInfo *msgPtr);
-extern void RatPurgeFlags(char *flags);
+extern char *RatPurgeFlags(char *flags, int level);
 
 
 /* ratDbMessage.c */
@@ -418,15 +437,15 @@ extern char* RatFrMessageCreate(Tcl_Interp *interp, char *data, int length,
 
 /* ratStdMessage.c */
 extern void RatStdMessagesInit (MessageProcInfo* procInfo);
-extern int RatStdEasyCopyingOK(MessageInfo *msgPtr, char *prot,
-	char *destination, char *user);
+extern int RatStdEasyCopyingOK(Tcl_Interp *interp, MessageInfo *msgPtr,
+			       Tcl_Obj *defPtr);
 extern int RatStdMessageCopy (Tcl_Interp *interp, MessageInfo *msgPtr,
 	char *destination);
 
 /* ratExp.c */
-extern Tcl_CmdProc RatParseExpCmd;
-extern Tcl_CmdProc RatGetExpCmd;
-extern Tcl_CmdProc RatFreeExpCmd;
+extern Tcl_ObjCmdProc RatParseExpCmd;
+extern Tcl_ObjCmdProc RatGetExpCmd;
+extern Tcl_ObjCmdProc RatFreeExpCmd;
 extern int RatExpMatch(Tcl_Interp *interp, int expId,
 	RatInfoProc *infoProc, ClientData clientData, int index);
 
