@@ -3,7 +3,7 @@
  *
  *	This file contains code which implements free messages.
  *
- * TkRat software and its included text is Copyright 1996-2000 by
+ * TkRat software and its included text is Copyright 1996-2002 by
  * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
@@ -20,7 +20,7 @@ typedef struct FrMessageInfo {
     char *from;
     char *headers;
     char *msgData;
-    char *bodyData;
+    unsigned char *bodyData;
 } FrMessageInfo;
 
 /*
@@ -135,15 +135,15 @@ RatFrMessageCreate(Tcl_Interp *interp, char *data, int length,
 	msgPtr->info[j] = NULL;
     }
     frMsgPtr->msgData = msgData;
-    frMsgPtr->messagePtr = RatParseMsg(interp, msgData);
+    frMsgPtr->messagePtr = RatParseMsg(interp, (unsigned char*)msgData);
     frMsgPtr->bodyData = frMsgPtr->messagePtr->text.text.data +
 			 frMsgPtr->messagePtr->text.offset;
     frMsgPtr->headers = (char*)ckalloc(headerLength+1);
-    RatStrNCpy(frMsgPtr->headers, data, headerLength+1);
+    strlcpy(frMsgPtr->headers, data, headerLength+1);
     if (!strncmp("From ", data, 5) && (cPtr = strchr(data, '\n'))) {
 	fromLength = cPtr-data;
 	frMsgPtr->from = (char*)ckalloc(fromLength+1);
-	RatStrNCpy(frMsgPtr->from, frMsgPtr->headers, fromLength);
+	strlcpy(frMsgPtr->from, frMsgPtr->headers, fromLength);
     } else {
 	frMsgPtr->from = NULL;
     }
@@ -204,14 +204,14 @@ Fr_GetEnvelopeProc(Tcl_Interp *interp, MessageInfo *msgPtr)
     } else {
 	adrPtr = frMsgPtr->messagePtr->env->from;
     }
-    if (!strcmp(currentHost, adrPtr->host)) {
+    if (!strcmp(Tcl_GetHostName(), adrPtr->host)) {
 	snprintf(buf, sizeof(buf), "From %s", adrPtr->mailbox);
     } else {
-	sprintf(buf, "From ");
+	strlcpy(buf, "From ", sizeof(buf));
 	if (RatAddressSize(adrPtr, 0) > sizeof(buf)-32) {
-	    sprintf(buf+5, "ridiculously@long.address");
+	    snprintf(buf+5, sizeof(buf)-5, "ridiculously@long.address");
 	} else {
-	    rfc822_address(buf+5, adrPtr);
+	    rfc822_write_address_full(buf+5, adrPtr, NULL);
 	}
     }
     if (T == mail_parse_date(&elt, frMsgPtr->messagePtr->env->date)) {
@@ -442,7 +442,7 @@ Fr_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
     MESSAGECACHE elt;
     Tcl_Obj *oPtr = NULL;
     char *cPtr;
-    int i;
+    int i, f;
 
     if (msgPtr->info[type]) {
 	return msgPtr->info[type];
@@ -465,7 +465,7 @@ Fr_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
 	case RAT_FOLDER_MSGID:		/* fallthrough */
 	case RAT_FOLDER_REF:		/* fallthrough */
 	case RAT_FOLDER_PARAMETERS:
-	    return RatGetMsgInfo(interp, type, msgPtr,frMsgPtr->messagePtr->env,
+	    return RatGetMsgInfo(interp, type,msgPtr,frMsgPtr->messagePtr->env,
 		    frMsgPtr->messagePtr->body, NULL, 0);
 	case RAT_FOLDER_SIZE:		/* fallthrough */
 	case RAT_FOLDER_SIZE_F:
@@ -538,12 +538,13 @@ Fr_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
 	    do {
 		if (!strncasecmp(cPtr, "status:", 7)) {
 		    for (i=7; cPtr[i] != '\n' && cPtr[i]; i++) {
-			switch (cPtr[i]) {
-			case 'R': strcat(buf, " " RAT_SEEN_STR); break;
-			case 'D': strcat(buf, " " RAT_DELETED_STR); break;
-			case 'F': strcat(buf, " " RAT_FLAGGED_STR); break;
-			case 'A': strcat(buf, " " RAT_ANSWERED_STR); break;
-			case 'T': strcat(buf, " " RAT_DRAFT_STR); break;
+			for (f=0; flag_name[f].imap_name; f++) {
+			    if (flag_name[f].unix_char == cPtr[i]) {
+				strlcat(buf, " ", sizeof(buf));
+				strlcat(buf, flag_name[f].imap_name,
+					sizeof(buf));
+				break;
+			    }
 			}
 		    }
 		    if (*buf) {
