@@ -1,9 +1,10 @@
 /*
  * ratMailcap.c --
  *
- *	This file contains basic support for sending messages via SMTP.
+ *	This file contains support for reading & parsing mailcap files
+ *	Mailcap files are defined in rfc1343
  *
- * TkRat software and its included text is Copyright 1996-2000 by
+ * TkRat software and its included text is Copyright 1996-2002 by
  * Martin Forssn
  *
  * The full text of the legal notices is contained in the file called
@@ -49,7 +50,6 @@ static int tableAllocated = 0;
 static void MailcapReload(Tcl_Interp *interp);
 static char *ExpandString(Tcl_Interp *interp, BodyInfo *bodyInfoPtr, char *s,
 	char **filePtr);
-static void FixQuotes(char *s);
 
 
 /*
@@ -97,7 +97,7 @@ MailcapReload(Tcl_Interp *interp)
 	Tcl_DStringAppend(&ds, ":", 1);
     }
     Tcl_DStringAppend(&ds,
-	    Tcl_GetVar2(interp, "option", "mailcap_path", TCL_GLOBAL_ONLY), -1);
+	    Tcl_GetVar2(interp, "option", "mailcap_path", TCL_GLOBAL_ONLY),-1);
     /*
      * Check all files mentioned in the path
      */
@@ -125,7 +125,7 @@ MailcapReload(Tcl_Interp *interp)
 	if (allocTextBlocks == numTextBlocks) {
 	    allocTextBlocks += 5;
 	    textBlock =
-		    (char**)ckrealloc(textBlock, allocTextBlocks*sizeof(char*));
+		    (char**)ckrealloc(textBlock,allocTextBlocks*sizeof(char*));
 	}
 	data = textBlock[numTextBlocks++] = (char*)ckalloc(sbuf.st_size+1);
 	read(fd, data, sbuf.st_size);
@@ -191,7 +191,7 @@ MailcapReload(Tcl_Interp *interp)
 	    tablePtr[tableSize].type = cPtr;
 	    for (;'/' != *cPtr && ';' != *cPtr && *cPtr; cPtr++);
 	    if (!*cPtr) {
-		RatLogF(interp, RAT_ERROR, "syntax_error", RATLOG_TIME, s,line);
+		RatLogF(interp, RAT_ERROR, "syntax_error", RATLOG_TIME,s,line);
 		continue;
 	    }
 	    if (';' == *cPtr) {
@@ -214,7 +214,7 @@ MailcapReload(Tcl_Interp *interp)
 	    }
 	    for (;isspace((unsigned char)*cPtr) && *cPtr; cPtr++);
 	    if (!*cPtr) {
-		RatLogF(interp, RAT_ERROR, "syntax_error", RATLOG_TIME, s,line);
+		RatLogF(interp, RAT_ERROR, "syntax_error", RATLOG_TIME,s,line);
 		continue;
 	    }
 	    tablePtr[tableSize].view = cPtr;
@@ -272,24 +272,6 @@ MailcapReload(Tcl_Interp *interp)
 		*cPtrPtr = cPtr;
 		for (;*cPtr && (';' != *cPtr || '\\' == *(cPtr-1)); cPtr++);
 	    }
-	    if (tablePtr[tableSize].view) {
-		FixQuotes(tablePtr[tableSize].view);
-	    }
-	    if (tablePtr[tableSize].test) {
-		FixQuotes(tablePtr[tableSize].test);
-	    }
-	    if (tablePtr[tableSize].compose) {
-		FixQuotes(tablePtr[tableSize].compose);
-	    }
-	    if (tablePtr[tableSize].composetyped) {
-		FixQuotes(tablePtr[tableSize].composetyped);
-	    }
-	    if (tablePtr[tableSize].edit) {
-		FixQuotes(tablePtr[tableSize].edit);
-	    }
-	    if (tablePtr[tableSize].print) {
-		FixQuotes(tablePtr[tableSize].print);
-	    }
 	    tableSize++;
 	}
     }
@@ -321,7 +303,7 @@ MailcapReload(Tcl_Interp *interp)
  */
 
 static char*
-ExpandString(Tcl_Interp *interp, BodyInfo *bodyInfoPtr, char *s, char **filePtr)
+ExpandString(Tcl_Interp *interp, BodyInfo *bodyInfoPtr, char *s,char **filePtr)
 {
     static Tcl_DString ds;
     static Tcl_DString file;
@@ -373,7 +355,7 @@ ExpandString(Tcl_Interp *interp, BodyInfo *bodyInfoPtr, char *s, char **filePtr)
 	    continue;
 	}
 	if ('t' == *srcPtr) {
-	    for (cPtr = body_types[bodyInfoPtr->bodyPtr->type]; *cPtr; cPtr++) {
+	    for (cPtr = body_types[bodyInfoPtr->bodyPtr->type]; *cPtr; cPtr++){
 		if (strchr("|<>%*?\"`'", *cPtr)) {
 		    Tcl_DStringAppend(&ds, " ", 1);
 		} else {
@@ -450,10 +432,10 @@ ExpandString(Tcl_Interp *interp, BodyInfo *bodyInfoPtr, char *s, char **filePtr)
 int
 RatMcapFindCmd(Tcl_Interp *interp, BodyInfo *bodyInfoPtr)
 {
-    Tcl_DString ds;
-    char *cmd, *file;
+    char *cmd, *file, *s;
     Tcl_Channel channel;
     int i, perm;
+    Tcl_Obj *rPtr, *oPtr;
 
     /*
      * We start by making sure that the mailcap files have been loaded
@@ -468,7 +450,7 @@ RatMcapFindCmd(Tcl_Interp *interp, BodyInfo *bodyInfoPtr)
      * - If they matched we check eventual test commands
      */
     for (i=0; i<tableSize; i++) {
-	if (strcasecmp(tablePtr[i].type, body_types[bodyInfoPtr->bodyPtr->type])
+	if (strcasecmp(tablePtr[i].type,body_types[bodyInfoPtr->bodyPtr->type])
 		|| ('*' != *tablePtr[i].subtype
 		    && strcasecmp(tablePtr[i].subtype,
 				  bodyInfoPtr->bodyPtr->subtype))) {
@@ -480,8 +462,9 @@ RatMcapFindCmd(Tcl_Interp *interp, BodyInfo *bodyInfoPtr)
 		continue;
 	    }
 	    if (file) {
-		Tcl_GetInt(interp, Tcl_GetVar2(interp, "option", "permissions",
-			TCL_GLOBAL_ONLY), &perm);
+		oPtr = Tcl_GetVar2Ex(interp, "option", "permissions",
+				     TCL_GLOBAL_ONLY);
+		Tcl_GetIntFromObj(interp, oPtr, &perm);
 		channel = Tcl_OpenFileChannel(interp, file, "w", perm);
 		RatBodySave(interp, channel, bodyInfoPtr, 0, 1);
 		Tcl_Close(interp, channel);
@@ -496,16 +479,18 @@ RatMcapFindCmd(Tcl_Interp *interp, BodyInfo *bodyInfoPtr)
 		unlink(file);
 	    }
 	}
-	Tcl_DStringInit(&ds);
-	Tcl_DStringAppendElement(&ds,
-		ExpandString(interp, bodyInfoPtr, tablePtr[i].view, NULL));
-	Tcl_DStringAppendElement(&ds, tablePtr[i].needsterminal?"1":"0");
-	Tcl_DStringAppendElement(&ds, tablePtr[i].copiousoutput?"1":"0");
-	Tcl_DStringAppendElement(&ds,
-		(tablePtr[i].description ? tablePtr[i].description : ""));
-	Tcl_DStringAppendElement(&ds,
-		(tablePtr[i].bitmap ? tablePtr[i].bitmap : ""));
-	Tcl_DStringResult(interp, &ds);
+	rPtr = Tcl_NewObj();
+	s = ExpandString(interp, bodyInfoPtr, tablePtr[i].view, NULL);
+	Tcl_ListObjAppendElement(interp, rPtr, Tcl_NewStringObj(s, -1));
+	Tcl_ListObjAppendElement(interp, rPtr,
+				 Tcl_NewBooleanObj(tablePtr[i].needsterminal));
+	Tcl_ListObjAppendElement(interp, rPtr,
+				 Tcl_NewBooleanObj(tablePtr[i].copiousoutput));
+	Tcl_ListObjAppendElement(interp, rPtr,
+				 Tcl_NewStringObj(tablePtr[i].description,-1));
+	Tcl_ListObjAppendElement(interp, rPtr,
+				 Tcl_NewStringObj(tablePtr[i].bitmap, -1));
+	Tcl_SetObjResult(interp, rPtr);
 	return TCL_OK;
     }
 
@@ -532,61 +517,9 @@ RatMcapFindCmd(Tcl_Interp *interp, BodyInfo *bodyInfoPtr)
  */
 
 int
-RatMailcapReload(ClientData dummy, Tcl_Interp *interp, int argc, char *argv[])
+RatMailcapReload(ClientData dummy, Tcl_Interp *interp, int objc,
+		 Tcl_Obj *const objv[])
 {
     MailcapReload(interp);
     return TCL_OK;
 }
-
-
-/*
- *----------------------------------------------------------------------
- *
- * FixQuotes --
- *
- *      Change pairs of single quotes to braces
- *
- * Results:
- *	None
- *
- * Side effects:
- *	May modify the passed string
- *
- *
- *----------------------------------------------------------------------
- */
-
-static void
-FixQuotes(char *line)
-{
-    char *c, *s, *e;
-
-    c = line;
-    while(*c) {
-	/*
-	 * Find start
-	 */
-	do {
-	    if (NULL == (s = strchr(c, '\''))) {
-		return;
-	    }
-	    c = s+1;
-	} while (s == line || !isspace(*(s-1)));
-
-	/*
-	 * Find end
-	 */
-	do {
-	    if (NULL == (e = strchr(c, '\''))) {
-		return;
-	    }
-	    c = e+1;
-	} while ('\\' != *(e-1) && e[1] && !isspace(e[1]));
-
-	/*
-	 * Convert to braces
-	 */
-	*s = '{';
-	*e = '}';
-    }
-}
