@@ -771,7 +771,7 @@ RatBodyCmd(ClientData clientData, Tcl_Interp *interp, int objc,
 	goto usage;
     }
     if (!strcmp(Tcl_GetString(objv[1]), "children")) {
-	BodyInfo *partInfoPtr;
+	BodyInfo **partInfoPtrPtr, *partInfoPtr;
 
 	if (TYPEMULTIPART != bodyPtr->type) {
 	    return TCL_OK;
@@ -780,11 +780,12 @@ RatBodyCmd(ClientData clientData, Tcl_Interp *interp, int objc,
 	if (!bodyInfoPtr->firstbornPtr) {
 	    (*messageProcInfo[bodyInfoPtr->type].makeChildrenProc)
 		    (interp, bodyInfoPtr);
-	    for (partInfoPtr = bodyInfoPtr->firstbornPtr; partInfoPtr;
-		    partInfoPtr = partInfoPtr->nextPtr) {
-		RatPGPBodyCheck(interp, messageProcInfo, &partInfoPtr);
-		Tcl_CreateObjCommand(interp, partInfoPtr->cmdName, RatBodyCmd,
-			(ClientData) partInfoPtr, NULL);
+	    for (partInfoPtrPtr = &bodyInfoPtr->firstbornPtr; *partInfoPtrPtr;
+		 partInfoPtrPtr = &(*partInfoPtrPtr)->nextPtr) {
+		RatPGPBodyCheck(interp, messageProcInfo, partInfoPtrPtr);
+		Tcl_CreateObjCommand(interp, (*partInfoPtrPtr)->cmdName,
+				     RatBodyCmd, (ClientData)(*partInfoPtrPtr),
+				     NULL);
 	    }
 	}
 	rPtr = Tcl_NewObj();
