@@ -3,7 +3,7 @@
  *
  *      Declarations of functions used in the Std folder and messages
  *
- * TkRat software and its included text is Copyright 1996-2000 by
+ * TkRat software and its included text is Copyright 1996-2002 by
  * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
@@ -19,14 +19,15 @@ extern char *RatSenderPGPPhrase(Tcl_Interp *interp);
 extern Tcl_TimerProc ClearPGPPass;
 extern void RatPGPBodyCheck(Tcl_Interp *interp, MessageProcInfo *procInfo,
 			    BodyInfo **bodyInfoPtrPtr);
-extern Tcl_CmdProc RatPGPCmd;
+extern Tcl_ObjCmdProc RatPGPCmd;
 extern char *RatPGPStrFind(char *haystack, long straws, char *needle,
 			   int linestart);
 
 /* ratPGPprog.c */
 extern BODY *RatPGPEncrypt(Tcl_Interp *interp, ENVELOPE *env, BODY *body,
 			   int sign);
-extern BODY *RatPGPSign(Tcl_Interp *interp, ENVELOPE *env, BODY *body);
+extern BODY *RatPGPSign(Tcl_Interp *interp, ENVELOPE *env, BODY *body,
+			const char *keyid);
 extern void RatPGPChecksig(Tcl_Interp *interp, MessageProcInfo *procInfo,
 			   BodyInfo *bodyInfoPtr);
 extern void RatPGPDecrypt(Tcl_Interp *interp, MessageProcInfo *procInfo,
