@@ -3,7 +3,7 @@
  *
  *	This file contains compatibility functions.
  *
- * TkRat software and its included text is Copyright 1996-2000 by
+ * TkRat software and its included text is Copyright 1996-2002 by
  * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
@@ -77,16 +77,17 @@ RatRunPGP(Tcl_Interp *interp, int nopass, char *command, char *args,
 {
     int toPipe[2], errPipe[2], argc, pid, i, out;
     Tcl_DString cmd;
-    char *pgp_path, *opt_args, **argv, cmdbuf[1024], *tmp;
+    char cmdbuf[1024];
+    CONST84 char *opt_args, *pgp_path, **argv, *tmp;
     struct rlimit rlim;
     static char name[1024];
 
     /*
      * Setup command arrays
      */
-    pgp_path = Tcl_GetVar2(interp, "option", "pgp_path", TCL_GLOBAL_ONLY);
+    pgp_path = RatGetPathOption(interp, "pgp_path");
     opt_args = Tcl_GetVar2(interp, "option", "pgp_args", TCL_GLOBAL_ONLY);
-    if (strlen(pgp_path)) {
+    if (pgp_path && strlen(pgp_path)) {
 	snprintf(cmdbuf, sizeof(cmdbuf), "%s/%s", pgp_path, command);
     } else {
 	snprintf(cmdbuf, sizeof(cmdbuf), "%s", command);
@@ -106,11 +107,9 @@ RatRunPGP(Tcl_Interp *interp, int nopass, char *command, char *args,
     /*
      * Open outfile and create the pgp subprocess.
      */
-    if (NULL == (tmp = Tcl_GetVar2(interp, "option", "tmp",TCL_GLOBAL_ONLY))) {
-	return 0;
-    }
+    tmp = Tcl_GetVar(interp, "rat_tmp", TCL_GLOBAL_ONLY);
     snprintf(name, sizeof(name), "%s/pgptmp.%d", tmp, getpid());
-    if ( 0 > (out = open(name, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR))) {
+    if ( 0 > (out = open(name, O_WRONLY|O_CREAT|O_TRUNC, 0600))) {
 	return 0;
     }
     pipe(toPipe);
@@ -131,7 +130,7 @@ RatRunPGP(Tcl_Interp *interp, int nopass, char *command, char *args,
 	if (!nopass) {
 	    putenv("PGPPASSFD=0");
 	}
-	execvp(cmdbuf, argv);
+	execvp(cmdbuf, (char**)argv);
 	{
 	    char buf[1024];
 	    snprintf(buf, sizeof(buf), "ERROR executing '%s %s': %s\n",
@@ -172,8 +171,8 @@ BODY*
 RatPGPEncrypt(Tcl_Interp *interp, ENVELOPE *env, BODY *body, int sign)
 {
     int toPGP, fromPGP, errPGP, length, retry, pid, result, status, i, j;
-    char *passPhrase = NULL, *hdrPtr, *version, *from;
-    char buf[MAILTMPLEN], *command;
+    char *passPhrase = NULL, *hdrPtr, *from, buf[MAILTMPLEN], *command;
+    CONST84 char *version;
     ADDRESS *adrPtr;
     Tcl_DString cmdDS, encDS, ds;
     BODY *multiPtr;
@@ -214,14 +213,15 @@ RatPGPEncrypt(Tcl_Interp *interp, ENVELOPE *env, BODY *body, int sign)
 	recipSep=" ";
     } else if (!strcmp("6", version)) {
 	command = "pgp";
-	Tcl_DStringAppend(&cmdDS, "+BATCHMODE +VERBOSE=0 -eaf", -1);
+	Tcl_DStringAppend(&cmdDS, "+BATCHMODE +VERBOSE=0 +force -eaf", -1);
 	if (sign) {
 	    Tcl_DStringAppend(&cmdDS, "s", 1);
-	    if (RatAddressSize(env->return_path, 0) < sizeof(buf)) {
+	    if (RatAddressSize(env->from, 0) < sizeof(buf)) {
 		buf[0] = '\0';
-		rfc822_address(buf, env->return_path);
-		Tcl_DStringAppend(&cmdDS, " -u ", 4);
+		rfc822_write_address_full(buf, env->from, NULL);
+		Tcl_DStringAppend(&cmdDS, " -u {", 5);
 		Tcl_DStringAppend(&cmdDS, buf, -1);
+		Tcl_DStringAppend(&cmdDS, "}", 1);
 	    }
 	}
 	recipSep=" ";
@@ -241,7 +241,7 @@ RatPGPEncrypt(Tcl_Interp *interp, ENVELOPE *env, BODY *body, int sign)
     for (adrPtr = env->cc; adrPtr; adrPtr = adrPtr->next) {
 	Tcl_DStringSetLength(&ds, RatAddressSize(adrPtr, 0));
 	Tcl_DStringValue(&ds)[0] = '\0';
-	rfc822_address(Tcl_DStringValue(&ds), adrPtr);
+	rfc822_write_address_full(Tcl_DStringValue(&ds), adrPtr, NULL);
 	Tcl_DStringSetLength(&ds, strlen(Tcl_DStringValue(&ds)));
 	Tcl_DStringAppend(&cmdDS, recipSep, -1);
 	Tcl_DStringAppend(&cmdDS, Tcl_DStringValue(&ds), -1);
@@ -249,7 +249,7 @@ RatPGPEncrypt(Tcl_Interp *interp, ENVELOPE *env, BODY *body, int sign)
     for (adrPtr = env->bcc; adrPtr; adrPtr = adrPtr->next) {
 	Tcl_DStringSetLength(&ds, RatAddressSize(adrPtr, 0));
 	Tcl_DStringValue(&ds)[0] = '\0';
-	rfc822_address(Tcl_DStringValue(&ds), adrPtr);
+	rfc822_write_address_full(Tcl_DStringValue(&ds), adrPtr, NULL);
 	Tcl_DStringSetLength(&ds, strlen(Tcl_DStringValue(&ds)));
 	Tcl_DStringAppend(&cmdDS, recipSep, -1);
 	Tcl_DStringAppend(&cmdDS, Tcl_DStringValue(&ds), -1);
@@ -276,10 +276,10 @@ RatPGPEncrypt(Tcl_Interp *interp, ENVELOPE *env, BODY *body, int sign)
         if (strcmp("6", version) || sign) {
 	    write(toPGP, "\n", 1);
 	}
-	buf[0] = '\0';
 	hdrPtr = buf;
+	buf[0] = '\0';
 	rfc822_write_body_header(&hdrPtr, body);
-	strcat(hdrPtr, "\015\012");
+	strlcat(buf, "\015\012", sizeof(buf));
 	write(toPGP, buf, strlen(buf));
 	RatInitDelayBuffer();
 	rfc822_output_body(body, RatDelaySoutr, (void*)toPGP);
@@ -319,7 +319,7 @@ RatPGPEncrypt(Tcl_Interp *interp, ENVELOPE *env, BODY *body, int sign)
 	    Tcl_DStringAppendElement(&error, "error");
 	    Tcl_DStringStartSublist(&error);
 	    do {
-		if ( 0 < (length = read(errPGP, buf, sizeof(buf)))) {
+		if (0 < (length = length = read(errPGP, buf, sizeof(buf)))) {
 		    Tcl_DStringAppend(&error, buf, length);
 		}
 	    } while (length > 0);
@@ -395,11 +395,11 @@ RatPGPEncrypt(Tcl_Interp *interp, ENVELOPE *env, BODY *body, int sign)
  */
 
 BODY*
-RatPGPSign(Tcl_Interp *interp, ENVELOPE *env, BODY *body)
+RatPGPSign(Tcl_Interp *interp, ENVELOPE *env, BODY *body, const char *keyid)
 {
     int toPGP, fromPGP, errPGP, length, retry, pid, status, result, i, j;
-    char *passPhrase, *hdrPtr, *version, *from;
-    char buf[MAILTMPLEN];
+    char *passPhrase, *hdrPtr, *from, *cmd, buf[MAILTMPLEN];
+    CONST84 char *version;
     Tcl_DString sigDS;
     BODY *multiPtr;
     PARAMETER *parmPtr;
@@ -418,42 +418,45 @@ RatPGPSign(Tcl_Interp *interp, ENVELOPE *env, BODY *body)
 	}
 	rfc822_encode_body_7bit(NIL, body);
 	if (!strcmp("gpg-1", version)) {
-	    strcpy(buf, "--detach-sign --armor --no-secmem-warning "
-		        "--passphrase-fd 0 --batch");
-	    pid = RatRunPGP(interp, 0, "gpg", buf, &toPGP, &from, &errPGP);
+	    cmd = "gpg";
+	    strlcpy(buf, "--detach-sign --armor --no-secmem-warning "
+		    "--passphrase-fd 0 --batch", sizeof(buf));
 	} else if (!strcmp("2", version)) {
-	    pid = RatRunPGP(interp, 0, "pgp", "+BATCHMODE +VERBOSE=0 -satbf",
-			    &toPGP, &from, &errPGP);
+	    cmd = "pgp";
+	    strlcpy(buf, "+BATCHMODE +VERBOSE=0 -satbf", sizeof(buf));
 	} else if (!strcmp("5", version)) {
-	    pid = RatRunPGP(interp, 0, "pgps", "-abf",
-			    &toPGP, &from, &errPGP);
+	    cmd = "pgps";
+	    strlcpy(buf, "-abf", sizeof(buf));
 	} else if (!strcmp("6", version)) {
-	    strcpy(buf, "+BATCHMODE +VERBOSE=0 -satbf");
-	    if (RatAddressSize(env->return_path,0)<sizeof(buf)-strlen(buf)-1) {
-		strcat(buf, " -u ");
-		rfc822_address(buf, env->return_path);
-	    }
-	    pid = RatRunPGP(interp, 0, "pgp", buf, &toPGP, &from, &errPGP);
+	    cmd = "pgp";
+	    strlcpy(buf, "+BATCHMODE +VERBOSE=0 +force -satbf", sizeof(buf));
 	} else {
 	    Tcl_SetResult(interp, "Unkown pgp version", TCL_STATIC);
 	    return NULL;
 	}
+	if (keyid && *keyid) {
+	    strlcat(buf, " -u {", sizeof(buf));
+	    strlcat(buf, keyid, sizeof(buf));
+	    strlcat(buf, "}", sizeof(buf));
+	}
+	pid = RatRunPGP(interp, 0, cmd, buf, &toPGP, &from, &errPGP);
+	
 	write(toPGP, passPhrase, strlen(passPhrase));
 	memset(passPhrase, '\0', strlen(passPhrase));
 	write(toPGP, "\n", 1);
-	buf[0] = '\0';
 	hdrPtr = buf;
+	buf[0] = '\0';
 	rfc822_write_body_header(&hdrPtr, body);
-	strcat(hdrPtr, "\015\012");
+	strlcat(buf, "\015\012", sizeof(buf));
 	write(toPGP, buf, strlen(buf));
 	RatInitDelayBuffer();
 	rfc822_output_body(body, RatDelaySoutr, (void*)toPGP);
 	close(toPGP);
 	/*i = open("/tmp/sigdump", O_CREAT | O_TRUNC | O_WRONLY, 0644);
-	write(i, buf, strlen(buf));
-	RatInitDelayBuffer();
-	rfc822_output_body(body, RatDelaySoutr, (void*)i);
-	close(i);*/
+	  write(i, buf, strlen(buf));
+	  RatInitDelayBuffer();
+	  rfc822_output_body(body, RatDelaySoutr, (void*)i);
+	  close(i);*/
 	do {
 	    result = waitpid(pid, &status, 0);
 	} while(-1 == result && EINTR == errno);
@@ -571,7 +574,7 @@ FindBoundary(char *text, char*boundary)
 
     if (NULL == text) return NULL;
     do {
-	if ('-' == cPtr[0] && '-' == cPtr[1] && !strncmp(cPtr+2, boundary, l)) {
+	if ('-' == cPtr[0] && '-' == cPtr[1] && !strncmp(cPtr+2, boundary, l)){
 	    return cPtr;
 	}
 	cPtr = strchr(cPtr, '\n');
@@ -603,7 +606,8 @@ RatPGPChecksig(Tcl_Interp *interp, MessageProcInfo* procInfo,
 {
     unsigned char *text;
     unsigned long length;
-    char *command, *version;
+    char *command;
+    CONST84 char *version;
 	
     version = Tcl_GetVar2(interp, "option", "pgp_version", TCL_GLOBAL_ONLY);
 	
@@ -612,21 +616,22 @@ RatPGPChecksig(Tcl_Interp *interp, MessageProcInfo* procInfo,
      * The algorithms kind of differ:-)
      */
     if (bodyInfoPtr->secPtr) {
-	char *boundary, *start, *end, *tmp, *from;
+	char *boundary, *start, *end, *from;
 	char buf[2048], textfile[1024], sigfile[1024];
 	int fd, toPGP, fromPGP, errPGP, pid, status, result;
+	CONST84 char *tmp;
 	PARAMETER *parPtr;
 	Tcl_DString *resultDS = (Tcl_DString*)ckalloc(sizeof(Tcl_DString));
 
 	/*
 	 * Generate filenames
 	 */
-	tmp = Tcl_GetVar2(interp, "option", "tmp", TCL_GLOBAL_ONLY);
+	tmp = Tcl_GetVar(interp, "rat_tmp", TCL_GLOBAL_ONLY);
 	RatGenId(NULL, interp, 0, NULL);
 	snprintf(textfile, sizeof(textfile), "%s/rat.%s",
 		tmp, Tcl_GetStringResult(interp));
-	RatStrNCpy(sigfile, textfile, sizeof(sigfile));
-	strcat(sigfile, ".sig");
+	strlcpy(sigfile, textfile, sizeof(sigfile));
+	strlcat(sigfile, ".sig", sizeof(sigfile));
 
 	/*
 	 * Save text and signature in files
@@ -641,7 +646,7 @@ RatPGPChecksig(Tcl_Interp *interp, MessageProcInfo* procInfo,
 		break;
 	    }
 	}
-	if (!boundary || NULL == (start = FindBoundary((char*)text, boundary))){
+	if (!boundary || NULL == (start = FindBoundary((char*)text,boundary))){
 	    bodyInfoPtr->sigStatus = RAT_SIG_BAD;
 	    return;
 	}
@@ -651,12 +656,12 @@ RatPGPChecksig(Tcl_Interp *interp, MessageProcInfo* procInfo,
 	    return;
 	}
 	end -= 2;
-	fd = open(textfile, O_CREAT | O_TRUNC | O_WRONLY, 0666);
+	fd = open(textfile, O_CREAT | O_TRUNC | O_WRONLY, 0600);
 	write(fd, start, end-start);
 	close(fd);
 	text = (unsigned char*)(*procInfo[bodyInfoPtr->type].fetchBodyProc)
 		(bodyInfoPtr->secPtr->firstbornPtr->nextPtr, &length);
-	fd = open(sigfile, O_CREAT | O_TRUNC | O_WRONLY, 0666);
+	fd = open(sigfile, O_CREAT | O_TRUNC | O_WRONLY, 0600);
 	if (text) {
 	    write(fd, text, length);
 	}
@@ -680,7 +685,7 @@ RatPGPChecksig(Tcl_Interp *interp, MessageProcInfo* procInfo,
 		    sigfile, textfile);
 	} else if (!strcmp("6", version)) {
 	    command = "pgp";
-	    snprintf(buf, sizeof(buf), "+batchmode +verbose=0 %s %s",
+	    snprintf(buf, sizeof(buf), "+batchmode +verbose=0 +force %s %s",
 		    sigfile, textfile);
 	} else {
 	    Tcl_SetResult(interp, "Unkown pgp version", TCL_STATIC);
@@ -735,7 +740,7 @@ RatPGPChecksig(Tcl_Interp *interp, MessageProcInfo* procInfo,
 		Tcl_ResetResult(interp);
 		return;
 	    }
-	    end = RatPGPStrFind(start, length-(start-(char*)text),"END PGP ",1);
+	    end = RatPGPStrFind(start,length-(start-(char*)text),"END PGP ",1);
 	    bodyDSPtr=RatPGPRunOld(interp,bodyInfoPtr,(char*)text,start,end+1);
 	    Tcl_DStringFree(bodyDSPtr);
 	    ckfree(bodyDSPtr);
@@ -771,8 +776,9 @@ void
 RatPGPDecrypt(Tcl_Interp *interp, MessageProcInfo *procInfo,
 	      BodyInfo **bodyInfoPtrPtr)
 {
-    int toPGP, fromPGP, errPGP, result, pid, retry, status, i;
-    char *text, *passPhrase, buf[1024], *version, *from;
+    int toPGP, fromPGP, errPGP, result, pid, retry, status;
+    char *text, *passPhrase, buf[1024], *from;
+    CONST84 char *version;
     BodyInfo *origPtr = *bodyInfoPtrPtr, *partInfoPtr;
     MessageInfo *msgPtr;
     unsigned long length;
@@ -807,8 +813,9 @@ RatPGPDecrypt(Tcl_Interp *interp, MessageProcInfo *procInfo,
 	    pid = RatRunPGP(interp, 0, "pgpv", "+batchmode=1 -f",
 		    &toPGP, &from, &errPGP);
 	} else if (!strcmp("6", version)) {
-	    pid = RatRunPGP(interp, 0, "pgp", "+BATCHMODE +VERBOSE=0 -f",
-		    &toPGP, &from, &errPGP);
+	    pid = RatRunPGP(interp, 0,
+			    "pgp", "+BATCHMODE +VERBOSE=0 +force -f",
+			    &toPGP, &from, &errPGP);
 	} else {
 	    Tcl_SetResult(interp, "Unkown pgp version", TCL_STATIC);
 	    break;
@@ -898,7 +905,7 @@ failed:
      */
     for (partInfoPtr = (*bodyInfoPtrPtr)->firstbornPtr; partInfoPtr;
 	    partInfoPtr = partInfoPtr->nextPtr) {
-	Tcl_CreateCommand(interp, partInfoPtr->cmdName, RatBodyCmd,
+	Tcl_CreateObjCommand(interp, partInfoPtr->cmdName, RatBodyCmd,
 		(ClientData) partInfoPtr, NULL);
     }
     RatLog(interp, RAT_PARSE, "", RATLOG_EXPLICIT);
@@ -929,6 +936,7 @@ RatPGPListKeys(Tcl_Interp *interp, char *keyringName)
     RatPGPKeyring *k = NULL;
     Tcl_Obj **list, **l = NULL, *l3[3];
     int i, j, size;
+    CONST84 char *value;
 
     if (keyringName) {
 	switch (keyringName[0]) {
@@ -948,9 +956,11 @@ RatPGPListKeys(Tcl_Interp *interp, char *keyringName)
 	    break;
 	}
     } else {
-	Tcl_TranslateFileName(interp,
-		Tcl_GetVar2(interp, "option", "pgp_keyring", TCL_GLOBAL_ONLY),
-		&ck);
+	if (NULL == (value = RatGetPathOption(interp, "pgp_keyring"))) {
+	    return TCL_ERROR;
+	}
+	Tcl_DStringInit(&ck);
+	Tcl_DStringAppend(&ck, value, -1);
     }
 
     /*
@@ -998,9 +1008,7 @@ RatPGPListKeys(Tcl_Interp *interp, char *keyringName)
 	l3[1] = Tcl_NewListObj(k->keyCount, list);
 	Tcl_SetObjResult(interp, Tcl_NewListObj(2, l3));
 	ckfree(list);
-	if (l) {
-	    ckfree(l);
-	}
+	ckfree(l);
     } else {
 	Tcl_ResetResult(interp);
     }
@@ -1032,9 +1040,10 @@ int
 RatPGPExtractKey(Tcl_Interp *interp, char *id, char *keyringName)
 {
     int toPGP, fromPGP, errPGP, pid, status, length, ret;
-    Tcl_DString cmd, result, ck;
-    char buf[1024];
-    char *cPtr, *command, *version, *from;
+    Tcl_DString cmd, ck;
+    char buf[1024], *cPtr, *command, *from;
+    CONST84 char *value, *version;
+    Tcl_Obj *rPtr;
 
     if (keyringName) {
 	switch (keyringName[0]) {
@@ -1054,13 +1063,15 @@ RatPGPExtractKey(Tcl_Interp *interp, char *id, char *keyringName)
 	    break;
 	}
     } else {
-	Tcl_TranslateFileName(interp,
-		Tcl_GetVar2(interp, "option", "pgp_keyring", TCL_GLOBAL_ONLY),
-		&ck);
+	if (NULL == (value = RatGetPathOption(interp, "pgp_keyring"))) {
+	    return TCL_ERROR;
+	}
+	Tcl_DStringInit(&ck);
+	Tcl_DStringAppend(&ck, value, -1);
     }
 
     Tcl_DStringInit(&cmd);
-    Tcl_DStringInit(&result);
+    rPtr = Tcl_NewObj();
     version = Tcl_GetVar2(interp, "option", "pgp_version", TCL_GLOBAL_ONLY);
     if (!strcmp("gpg-1", version)) {
 	command = "gpg";
@@ -1074,7 +1085,8 @@ RatPGPExtractKey(Tcl_Interp *interp, char *id, char *keyringName)
 	Tcl_DStringAppend(&cmd, "+batchmode=1 -x +PubRing=", -1);
     } else if (!strcmp("6", version)) {
 	command = "pgp";
-	Tcl_DStringAppend(&cmd, "-kxaf +BATCHMODE +VERBOSE=0 +PubRing=", -1);
+	Tcl_DStringAppend(&cmd,
+			  "-kxaf +BATCHMODE +VERBOSE=0 +force +PubRing=", -1);
     } else {
 	Tcl_SetResult(interp, "Unkown pgp version", TCL_STATIC);
 	return TCL_ERROR;
@@ -1102,8 +1114,8 @@ RatPGPExtractKey(Tcl_Interp *interp, char *id, char *keyringName)
      */
     fromPGP = open(from, O_RDONLY);
     do {
-	if ( 0 < (length = read(fromPGP, buf, sizeof(buf)))) {
-	    Tcl_DStringAppend(&result, buf, length);
+	if (0 < (length = read(fromPGP, buf, sizeof(buf)))) {
+	    Tcl_AppendToObj(rPtr, buf, length);
 	}
     } while (length > 0);
     close(fromPGP);
@@ -1114,18 +1126,18 @@ RatPGPExtractKey(Tcl_Interp *interp, char *id, char *keyringName)
      */
     if (pid != ret
 	    || (WEXITSTATUS(status) != 0 && WEXITSTATUS(status) != 1)) {
-	Tcl_DStringSetLength(&result, 0);
+	Tcl_SetStringObj(rPtr, NULL, 0);
 	do {
 	    if (0 < (length = read(errPGP, buf, sizeof(buf)))) {
-		Tcl_DStringAppend(&result, buf, length);
+		Tcl_AppendToObj(rPtr, buf, length);
 	    }
 	} while (length > 0);
 	close(errPGP);
-	Tcl_DStringResult(interp, &result);
+	Tcl_SetObjResult(interp, rPtr);
 	return TCL_ERROR;
     }
     close(errPGP);
-    Tcl_DStringResult(interp, &result);
+    Tcl_SetObjResult(interp, rPtr);
     return TCL_OK;
 }
 
@@ -1189,10 +1201,12 @@ RatPGPRunOld(Tcl_Interp *interp, BodyInfo *bodyInfoPtr, char *text,
 		char *start, char *end)
 {
     Tcl_DString *errDSPtr = (Tcl_DString*)ckalloc(sizeof(Tcl_DString)),
-		*bodyDSPtr = (Tcl_DString*)ckalloc(sizeof(Tcl_DString));
+	*bodyDSPtr = (Tcl_DString*)ckalloc(sizeof(Tcl_DString)),
+	*dsPtr = NULL;
     int needPhrase, toPGP, fromPGP, errPGP, length, preamble, pid, status,
 	result, retry;
-    char *ePtr, *cPtr, *passPhrase, buf[1024], *version, *from;
+    char *ePtr, *cPtr, *passPhrase, buf[1024], *from;
+    CONST84 char *version;
     FILE *fp;
 
     version = Tcl_GetVar2(interp, "option", "pgp_version", TCL_GLOBAL_ONLY);
@@ -1201,21 +1215,18 @@ RatPGPRunOld(Tcl_Interp *interp, BodyInfo *bodyInfoPtr, char *text,
      * Prepare text part
      */
     Tcl_DStringInit(bodyDSPtr);
-    Rat_DStringApendNoCRLF(bodyDSPtr, text, start-text);
+    RatDStringApendNoCRLF(bodyDSPtr, text, start-text);
     preamble = start-text;
 
     /*
      * Setup and run pgp command
      */
-    if (!(ePtr = strchr(end, '\n'))) {
-	ePtr = end + strlen(end);
-    }
     needPhrase = strncmp(start, "-----BEGIN PGP SIGNED", 21);
     do {
 	if (needPhrase) {
 	    passPhrase = RatPGPPhrase(interp);
 	    if (NULL == passPhrase) {
-		Rat_DStringApendNoCRLF(bodyDSPtr, start, ePtr-start);
+		RatDStringApendNoCRLF(bodyDSPtr, start, end-start);
 		ckfree(errDSPtr);
 		bodyInfoPtr->pgpOutput = NULL;
 		return bodyDSPtr;
@@ -1234,25 +1245,47 @@ RatPGPRunOld(Tcl_Interp *interp, BodyInfo *bodyInfoPtr, char *text,
 	    pid = RatRunPGP(interp, 0, "pgpv", "+batchmode=1 -f",
 		    &toPGP, &from, &errPGP);
 	} else if (!strcmp("6", version)) {
-	    pid = RatRunPGP(interp, 0, "pgp", "+BATCHMODE +VERBOSE=0 -f",
-		    &toPGP, &from, &errPGP);
+	    pid = RatRunPGP(interp, 0,
+			    "pgp", "+BATCHMODE +VERBOSE=0 +force -f",
+			    &toPGP, &from, &errPGP);
 	} else {
 	    Tcl_SetResult(interp, "Unkown pgp version", TCL_STATIC);
 	    return NULL;
 	}
 	write(toPGP, passPhrase, strlen(passPhrase));
 	memset(passPhrase, '\0', strlen(passPhrase));
-	if (needPhrase) {
-	    ckfree(passPhrase);
-	}
+	ckfree(passPhrase);
 	write(toPGP, "\n", 1);
-	/* Convert to local newline conventions */
 	fp = fdopen(toPGP, "w");
+	/*
+	 * Undo any encoding ant buggy MTA may have applied. Since we have this
+	 * code here it means that the "BEGIN PGP" stuff must be visible in
+	 * the raw (undecoded) text, so the only encoding we actually can
+	 * handle is quoted-printable. I can live with this restriction and
+	 * it is better than the performance penalty to decode all bodyparts.
+	 */
+	if (ENC7BIT != bodyInfoPtr->bodyPtr->encoding) {
+	
+	    dsPtr = RatDecode(interp, bodyInfoPtr->bodyPtr->encoding,
+				  start, end-start, NULL);
+	    start = Tcl_DStringValue(dsPtr);
+	    ePtr = start + Tcl_DStringLength(dsPtr);
+	} else {
+	    if (!(ePtr = strchr(end, '\n'))) {
+		ePtr = end + strlen(end);
+	    }
+	}
+	
+	/* Convert to local newline conventions */
 	for (cPtr = start; cPtr < ePtr; cPtr++) {
 	    if ('\r' == cPtr[0] && '\n' == cPtr[1]) cPtr++;
 	    fputc(*cPtr, fp);
 	}
 	fclose(fp);
+
+	if (dsPtr) {
+	    Tcl_DStringFree(dsPtr);
+	}
 	do {
 	    result = waitpid(pid, &status, 0);
 	} while(-1 == result && EINTR == errno);
@@ -1290,7 +1323,7 @@ RatPGPRunOld(Tcl_Interp *interp, BodyInfo *bodyInfoPtr, char *text,
 		Tcl_DStringFree(errDSPtr);
 		ckfree(errDSPtr);
 		RatLog(interp, RAT_PARSE, "", RATLOG_EXPLICIT);
-		Rat_DStringApendNoCRLF(bodyDSPtr, start, ePtr-start);
+		RatDStringApendNoCRLF(bodyDSPtr, start, ePtr-start);
 		bodyInfoPtr->pgpOutput = NULL;
 		return bodyDSPtr;
 	    } else {
@@ -1337,7 +1370,8 @@ RatPGPHandleOld(Tcl_Interp *interp, BodyInfo *bodyInfoPtr, char *text,
 		char *start, char *end)
 {
     if (strncmp(start, "-----BEGIN PGP SIGNED", 21)) {
-	char *cPtr, *t;
+	char *cPtr;
+	CONST84 char *t;
 
 	bodyInfoPtr->decodedTextPtr = RatPGPRunOld(interp, bodyInfoPtr, text,
 						start, end);
@@ -1345,7 +1379,7 @@ RatPGPHandleOld(Tcl_Interp *interp, BodyInfo *bodyInfoPtr, char *text,
 	    cPtr = end + strlen(end);
 	}
 	if (*cPtr) {
-	    Rat_DStringApendNoCRLF(bodyInfoPtr->decodedTextPtr, cPtr, -1);
+	    RatDStringApendNoCRLF(bodyInfoPtr->decodedTextPtr, cPtr, -1);
 	}
 	if (bodyInfoPtr->pgpOutput
 		&& 1 < Tcl_DStringLength(bodyInfoPtr->pgpOutput)) {
@@ -1387,13 +1421,13 @@ static int
 RatUpdatePGPKeys(Tcl_Interp *interp, RatPGPKeyring *k)
 {
     int toPGP, errPGP, pid, status, length, ret, preamble, blankiscont;
-    char buf[1024], title[1024], idbuf[1024], *start, *end, *command, *last,
-	    *version, *from;
-    Tcl_DString cmd, tmp, result;
+    char buf[1024], title[1024], idbuf[1024], *command, *from;
+    CONST84 char *version, *start, *end, *last;
+    Tcl_DString cmd, tmp;
     struct stat sbuf;
     Tcl_RegExp exp_id, exp_addr;
     FILE *fp;
-    Tcl_Obj **ids;
+    Tcl_Obj **ids, *rPtr;
     unsigned int idNum, idAlloc;
 
     stat(k->name, &sbuf);
@@ -1428,7 +1462,8 @@ RatUpdatePGPKeys(Tcl_Interp *interp, RatPGPKeyring *k)
 
     } else if (!strcmp("6", version)) {
 	command = "pgp";
-	Tcl_DStringAppend(&cmd, "-kv +BATCHMODE +VERBOSE=0 +PubRing=", -1);
+	Tcl_DStringAppend(&cmd, "-kv +BATCHMODE +VERBOSE=0 +force +PubRing=",
+			  -1);
 	exp_id = Tcl_RegExpCompile(interp, "0x([0-9A-F]{8})");
 	exp_addr = Tcl_RegExpCompile(interp, "<[a-zA-Z.+@-]+>");
 	blankiscont = 1;
@@ -1484,8 +1519,8 @@ RatUpdatePGPKeys(Tcl_Interp *interp, RatPGPKeyring *k)
 		if (Tcl_RegExpExec(interp, exp_id, buf, buf)) {
 		    Tcl_RegExpRange(exp_id, 1, &start, &end);
 		    last = end;
-		    RatStrNCpy(idbuf, "0x", sizeof(idbuf));
-		    RatStrNCpy(idbuf+2, start, end-start+1);
+		    strlcpy(idbuf, "0x", sizeof(idbuf));
+		    strlcpy(idbuf+2, start, end-start+1);
 		    ids[idNum++] = Tcl_NewStringObj(idbuf, 2+end-start);
 		}
 		if (Tcl_RegExpExec(interp, exp_addr, buf, buf)) {
@@ -1502,7 +1537,7 @@ RatUpdatePGPKeys(Tcl_Interp *interp, RatPGPKeyring *k)
 		idNum = 0;
 	    }
 	    if (preamble && buf[0] && buf[0] != '-') {
-		RatStrNCpy(title, buf, sizeof(title));
+		strlcpy(title, buf, sizeof(title));
 	    }
 	}
     }
@@ -1521,14 +1556,14 @@ RatUpdatePGPKeys(Tcl_Interp *interp, RatPGPKeyring *k)
      */
     if (pid != ret
 	    || (WEXITSTATUS(status) != 0 && WEXITSTATUS(status) != 1)) {
-	Tcl_DStringInit(&result);
+	rPtr = Tcl_NewObj();
 	do {
-	    if (0 < (length = read(errPGP, buf, sizeof(buf)))) {
-		Tcl_DStringAppend(&result, buf, length);
+	    if ( 0 < (length = read(errPGP, buf, sizeof(buf)))) {
+		Tcl_AppendToObj(rPtr, buf, length);
 	    }
 	} while (length > 0);
 	close(errPGP);
-	Tcl_DStringResult(interp, &result);
+	Tcl_SetObjResult(interp, rPtr);
 	return TCL_ERROR;
     }
     close(errPGP);
