@@ -4,7 +4,7 @@
  *	This file contains the code for a simple prettyprinter.
  *	Unfortunately it is currently limited to iso8859-1 characters.
  *
- * TkRat software and its included text is Copyright 1996-2000 by
+ * TkRat software and its included text is Copyright 1996-2002 by
  * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
@@ -25,7 +25,10 @@ typedef enum {
  * Misc defines and variables describing the size of the page and
  * other related data
  */
-#define PS_MARGIN	15
+#define PS_LEFT_MARGIN	 50
+#define PS_RIGHT_MARGIN	 25
+#define PS_TOP_MARGIN	 25
+#define PS_BOTTOM_MARGIN 25
 static int ps_xsize, ps_ysize;
 static int portrait;
 static int fontsize;
@@ -266,21 +269,23 @@ static int cob_wx[256] = {
  * Local functions
  */
 static void InitPrintData(Tcl_Interp *interp);
-static float GetStringLength(RatFont font, const unsigned char *string,
+static float GetStringLength(RatFont font, const char *string,
 	int length);
 static void Newpage(Tcl_Interp *interp, Tcl_Channel channel,
 	const char *subjectArg, MESSAGECACHE *elt);
 static void Startpage(Tcl_Interp *interp, Tcl_Channel channel,
 	const char *subject, MESSAGECACHE *elt, int pagenum);
 static void Endpage(Tcl_Channel channel);
-static void PsPrintString(Tcl_Interp *interp, Tcl_Channel channel, RatFont font,
-	float lm, float hm, const char *string, int length);
-static void PrintHeaders(Tcl_Interp *interp, Tcl_Channel channel, char *hs,
-	MessageInfo *msgPtr);
+static void PsPrintString(Tcl_Interp *interp, Tcl_Channel channel,
+			  RatFont font, Tcl_Encoding enc,
+			  float lm, float hm, const char *string, int length);
+static void PrintHeaders(Tcl_Interp *interp, Tcl_Channel channel,
+			 Tcl_Encoding enc, char *hs,
+			 MessageInfo *msgPtr);
 static void PrintBody(Tcl_Interp *interp, Tcl_Channel channel,
-	BodyInfo *bodyInfoPtr);
+		      Tcl_Encoding enc, BodyInfo *bodyInfoPtr);
 static int PrintBodyText(Tcl_Interp *interp, Tcl_Channel channel,
-	BodyInfo *bodyInfoPtr);
+			 Tcl_Encoding enc, BodyInfo *bodyInfoPtr);
 static int PrintBodyImage(Tcl_Interp *interp, Tcl_Channel channel,
 	BodyInfo *bodyInfoPtr);
 
@@ -370,7 +375,7 @@ RatPrettyPrintMsg(ClientData dummy, Tcl_Interp *interp, int objc,
     Newpage(interp, channel, subject, elt);
 
     /* Print headers */
-    PrintHeaders(interp, channel, hs, msgPtr);
+    PrintHeaders(interp, channel, NULL, hs, msgPtr);
 
     /* Print bodyparts */
     Tcl_ListObjGetElements(interp, objv[4], &bc, &bv);
@@ -378,12 +383,12 @@ RatPrettyPrintMsg(ClientData dummy, Tcl_Interp *interp, int objc,
 	yPos -= fontsize*1.1;
 	CHECK_NEWPAGE(interp, channel);
 	Tcl_GetCommandInfo(interp, Tcl_GetString(bv[i]), &cmdInfo);
-	PrintBody(interp, channel, (BodyInfo*)cmdInfo.clientData);
+	PrintBody(interp, channel, NULL, (BodyInfo*)cmdInfo.objClientData);
     }
 
     /* Print postludium */
     Endpage(channel);
-    snprintf(buf, sizeof(buf), "%%%%Trailer\n%%%%Pages: %d\n%%%%EOF\n",pagenum);
+    snprintf(buf, sizeof(buf),"%%%%Trailer\n%%%%Pages: %d\n%%%%EOF\n",pagenum);
     Tcl_WriteChars(channel, buf, -1);
     return TCL_OK;
 }
@@ -408,14 +413,14 @@ static void
 InitPrintData(Tcl_Interp *interp)
 {
     Tcl_Obj *oPtr, *o2Ptr, *o3Ptr;
-    char *s, *f;
+    CONST84 char *s, *f;
     int i;
 
     /*
      * Papersize
      */
     s = Tcl_GetVar2(interp, "option", "print_papersize", TCL_GLOBAL_ONLY);
-    oPtr = Tcl_GetVar2Ex(interp, "option", "print_papersizes", TCL_GLOBAL_ONLY);
+    oPtr = Tcl_GetVar2Ex(interp, "option", "print_papersizes",TCL_GLOBAL_ONLY);
     Tcl_ListObjLength(interp, oPtr, &i);
     for (i--; i >= 0; i--) {
 	Tcl_ListObjIndex(interp, oPtr, i, &o2Ptr);
@@ -429,8 +434,8 @@ InitPrintData(Tcl_Interp *interp)
     Tcl_GetIntFromObj(interp, o2Ptr, &ps_xsize);
     Tcl_ListObjIndex(interp, o3Ptr, 1, &o2Ptr);
     Tcl_GetIntFromObj(interp, o2Ptr, &ps_ysize);
-    ps_xsize -= PS_MARGIN*2;
-    ps_ysize -= PS_MARGIN*2;
+    ps_xsize -= PS_LEFT_MARGIN + PS_RIGHT_MARGIN;
+    ps_ysize -= PS_TOP_MARGIN + PS_BOTTOM_MARGIN;
 
     /*
      * Orientation
@@ -471,7 +476,7 @@ InitPrintData(Tcl_Interp *interp)
      */
     oPtr = Tcl_GetVar2Ex(interp, "option", "print_fontsize", TCL_GLOBAL_ONLY);
     Tcl_GetIntFromObj(interp, oPtr, &fontsize);
-    oPtr = Tcl_GetVar2Ex(interp, "option", "print_resolution", TCL_GLOBAL_ONLY);
+    oPtr = Tcl_GetVar2Ex(interp, "option", "print_resolution",TCL_GLOBAL_ONLY);
     Tcl_GetIntFromObj(interp, oPtr, &resolution);
 }
 
@@ -492,13 +497,13 @@ InitPrintData(Tcl_Interp *interp)
  */
 
 static float
-GetStringLength(RatFont font, const unsigned char *string, int length)
+GetStringLength(RatFont font, const char *string, int length)
 {
     int *wx = tir_wx, i;
     float l, size = fontsize;
 
     if (-1 == length) {
-	length = strlen(string);
+	length = strlen((char*)string);
     }
     switch(font) {
 	case FONT_SMALL:  wx = font_wx;
@@ -511,7 +516,7 @@ GetStringLength(RatFont font, const unsigned char *string, int length)
 			  break;
     }
     for (i=l=0; i<length; i++) {
-	l += wx[string[i]];
+	l += wx[(unsigned char)string[i]];
     }
     return (l*size)/1000;
 }
@@ -570,19 +575,23 @@ Newpage(Tcl_Interp *interp, Tcl_Channel channel, const char *subjectArg,
 
 static void
 Startpage(Tcl_Interp *interp, Tcl_Channel channel, const char *subject,
-	MESSAGECACHE *elt, int pagenum)
+	  MESSAGECACHE *elt, int pagenum)
 {
-    char buf[1024], *s;
+    char buf[1024];
+    const unsigned char *cPtr;
+    CONST84 char *s;
     float x, y, w, h, l;
     int objc;
     Tcl_Obj **objv, *oPtr;
+    Tcl_DString ds;
 
     /*
      * prepare
      */
     snprintf(buf, sizeof(buf), "%%%%Page: %d %d\n"
-		 "save\n"
-		 "%d %d translate\n", pagenum, pagenum, PS_MARGIN, PS_MARGIN);
+	     "save\n"
+	     "%d %d translate\n",
+	     pagenum, pagenum, PS_LEFT_MARGIN, PS_BOTTOM_MARGIN);
     Tcl_WriteChars(channel, buf, -1);
 
     /*
@@ -616,7 +625,7 @@ Startpage(Tcl_Interp *interp, Tcl_Channel channel, const char *subject,
     Tcl_ListObjGetElements(interp, oPtr, &objc, &objv);
     s = Tcl_GetString(objv[elt->month-1]);
     l = GetStringLength(FONT_SMALL, s, -1);
-    snprintf(buf, sizeof(buf), "%.2f %.2f moveto smallfont setfont (%s) show\n",
+    snprintf(buf, sizeof(buf),"%.2f %.2f moveto smallfont setfont (%s) show\n",
 	    x+w/2-l/2, y-fontsize*1.7, s);
     Tcl_WriteChars(channel, buf, -1);
     sprintf(buf, "%d", elt->year+BASEYEAR);
@@ -628,7 +637,7 @@ Startpage(Tcl_Interp *interp, Tcl_Channel channel, const char *subject,
     x += fontsize*3;
     y -= fontsize/2;
     s = Tcl_GetVar2(interp, "t", "received", TCL_GLOBAL_ONLY);
-    snprintf(buf, sizeof(buf), "%.2f %.2f moveto smallfont setfont (%s) show\n",
+    snprintf(buf, sizeof(buf),"%.2f %.2f moveto smallfont setfont (%s) show\n",
 	    x, y, s);
     Tcl_WriteChars(channel, buf, -1);
     sprintf(buf, "%.2f %.2f moveto textfont setfont (%02d:%02d) show\n",
@@ -643,13 +652,26 @@ Startpage(Tcl_Interp *interp, Tcl_Channel channel, const char *subject,
     yPos = ps_ysize-fontsize*4;
     Tcl_WriteChars(channel, buf, -1);
     s = Tcl_GetVar2(interp, "t", "mail_regarding", TCL_GLOBAL_ONLY);
-    snprintf(buf, sizeof(buf), "boldfont setfont %.2f %.2f moveto\n(%s) show\n",
+    snprintf(buf, sizeof(buf),"boldfont setfont %.2f %.2f moveto\n(%s) show\n",
 	    x, y, s);
     Tcl_WriteChars(channel, buf, -1);
     x += GetStringLength(FONT_BOLD, s, -1);
-    snprintf(buf, sizeof(buf), "textfont setfont %.2f %.2f moveto\n(%s) show\n",
-	    x+2, y, subject);
+    snprintf(buf, sizeof(buf),"textfont setfont %.2f %.2f moveto\n(", x+2, y);
     Tcl_WriteChars(channel, buf, -1);
+    Tcl_UtfToExternalDString(NULL, subject, -1, &ds);
+    for (cPtr = Tcl_DStringValue(&ds); *cPtr; cPtr++) {
+	if ('(' == *cPtr || ')' == *cPtr || '\\' == *cPtr) {
+	    Tcl_WriteChars(channel, "\\", 1);
+	}
+	if (*cPtr >= 32 && *cPtr < 127) {
+	    Tcl_WriteChars(channel, cPtr, 1);
+	} else {
+	    snprintf(buf, sizeof(buf), "\\%o", *cPtr);
+	    Tcl_WriteChars(channel, buf, -1);
+	}
+    }
+    Tcl_DStringFree(&ds);
+    Tcl_WriteChars(channel, ") show\n", -1);
 
     /*
      * Page number
@@ -657,13 +679,13 @@ Startpage(Tcl_Interp *interp, Tcl_Channel channel, const char *subject,
     s = Tcl_GetVar2(interp, "t", "page", TCL_GLOBAL_ONLY);
     x = ps_xsize - PAGENUMWIDTH - GetStringLength(FONT_SMALL, s, -1);
     y = ps_ysize-fontsize;
-    snprintf(buf, sizeof(buf), "%.2f %.2f moveto smallfont setfont (%s) show\n",
+    snprintf(buf, sizeof(buf),"%.2f %.2f moveto smallfont setfont (%s) show\n",
 	    x, y, s);
     Tcl_WriteChars(channel, buf, -1);
     sprintf(buf, "%d", pagenum);
     x = ps_xsize-PAGENUMWIDTH/2 - GetStringLength(FONT_BIG, buf, -1)/2;
     y = ps_ysize-2*fontsize;
-    sprintf(buf, "bigfont setfont %.2f %.2f moveto (%d) show\n", x, y, pagenum);
+    sprintf(buf, "bigfont setfont %.2f %.2f moveto (%d) show\n", x, y,pagenum);
     Tcl_WriteChars(channel, buf, -1);
 
 }
@@ -711,18 +733,14 @@ Endpage(Tcl_Channel channel)
 
 static void
 PsPrintString(Tcl_Interp *interp, Tcl_Channel channel, RatFont font,
-	float lm, float hm, const char *string, int length)
+	      Tcl_Encoding enc, float lm, float hm,
+	      const char *string, int length)
 {
-    static Tcl_Encoding enc = NULL;
     char buf[1024], *fn = "";
     const unsigned char *cPtr;
     float ll, l;
     Tcl_DString ds;
 
-    if (!enc) {
-	enc = Tcl_GetEncoding(interp, "iso8859-1");
-    }
-
     switch (font) {
 	case FONT_SMALL:  fn = "smallfont"; break;
 	case FONT_NORMAL: fn = "textfont"; break;
@@ -734,7 +752,7 @@ PsPrintString(Tcl_Interp *interp, Tcl_Channel channel, RatFont font,
 
     snprintf(buf, sizeof(buf), "%s setfont %.2f %d moveto\n(", fn, lm, yPos);
     Tcl_WriteChars(channel, buf, -1);
-    for (cPtr = Tcl_DStringValue(&ds), ll = lm; *cPtr; cPtr++) {
+    for (cPtr = Tcl_DStringValue(&ds), ll=lm; *cPtr; cPtr++) {
 	ll += (l = GetStringLength(font, cPtr, 1));
 	if (ll > ps_xsize-hm) {
 	    Tcl_WriteChars(channel, ") show\n", -1);
@@ -749,7 +767,12 @@ PsPrintString(Tcl_Interp *interp, Tcl_Channel channel, RatFont font,
 	if ('(' == *cPtr || ')' == *cPtr || '\\' == *cPtr) {
 	    Tcl_WriteChars(channel, "\\", 1);
 	}
-	Tcl_WriteChars(channel, cPtr, 1);
+	if (*cPtr >= 32 && *cPtr < 127) {
+	    Tcl_WriteChars(channel, cPtr, 1);
+	} else {
+	    snprintf(buf, sizeof(buf), "\\%o", *cPtr);
+	    Tcl_WriteChars(channel, buf, -1);
+	}
     }
     Tcl_WriteChars(channel, ") show\n", -1);
     Tcl_DStringFree(&ds);
@@ -772,8 +795,8 @@ PsPrintString(Tcl_Interp *interp, Tcl_Channel channel, RatFont font,
  */
 
 static void
-PrintHeaders(Tcl_Interp *interp, Tcl_Channel channel, char *hs,
-	MessageInfo *msgPtr)
+PrintHeaders(Tcl_Interp *interp, Tcl_Channel channel,
+	     Tcl_Encoding enc, char *hs, MessageInfo *msgPtr)
 {
     Tcl_Obj **fhv, **phv, *oPtr, *o2Ptr;
     char buf[1024], *s;
@@ -831,11 +854,11 @@ PrintHeaders(Tcl_Interp *interp, Tcl_Channel channel, char *hs,
 	CHECK_NEWPAGE(interp, channel);
 	Tcl_ListObjIndex(interp, phv[i], 0, &oPtr);
 	snprintf(buf, sizeof(buf), "%s:", Tcl_GetString(oPtr));
-	PsPrintString(interp, channel, FONT_BOLD, HINDENT + maxX - lengths[i],
-		0, buf, -1);
+	PsPrintString(interp, channel, FONT_BOLD, enc,
+		      HINDENT + maxX - lengths[i], 0, buf, -1);
 	Tcl_ListObjIndex(interp, phv[i], 1, &oPtr);
-	PsPrintString(interp, channel, FONT_NORMAL, HINDENT + maxX + 10, 0,
-		Tcl_GetString(oPtr), -1);
+	PsPrintString(interp, channel, FONT_NORMAL, enc, HINDENT + maxX + 10,
+		      0, Tcl_GetString(oPtr), -1);
 	yPos -= fontsize*1.1;
     }
 
@@ -865,11 +888,13 @@ PrintHeaders(Tcl_Interp *interp, Tcl_Channel channel, char *hs,
  */
 
 static void
-PrintBody(Tcl_Interp *interp, Tcl_Channel channel, BodyInfo *bodyInfoPtr)
+PrintBody(Tcl_Interp *interp, Tcl_Channel channel, Tcl_Encoding enc,
+	  BodyInfo *bodyInfoPtr)
 {
     Tcl_Obj *oPtr = RatBodyType(bodyInfoPtr), **objv;
     int objc;
-    char *m, buf[1024], buf2[42];
+    char buf[1024], buf2[42];
+    CONST84 char *m;
 
     Tcl_ListObjGetElements(interp, oPtr, &objc, &objv);
 
@@ -877,7 +902,7 @@ PrintBody(Tcl_Interp *interp, Tcl_Channel channel, BodyInfo *bodyInfoPtr)
      * Try to print it
      */
     if (!strcasecmp("TEXT", Tcl_GetString(objv[0]))) {
-	if (TCL_OK == PrintBodyText(interp, channel, bodyInfoPtr)) {
+	if (TCL_OK == PrintBodyText(interp, channel, enc, bodyInfoPtr)) {
 	    return;
 	}
     } else if (!strcasecmp("IMAGE", Tcl_GetString(objv[0]))) {
@@ -894,7 +919,7 @@ PrintBody(Tcl_Interp *interp, Tcl_Channel channel, BodyInfo *bodyInfoPtr)
 	    Tcl_GetString(objv[1]));
     snprintf(buf, sizeof(buf), m, buf2);
     CHECK_NEWPAGE(interp, channel);
-    PsPrintString(interp, channel, FONT_BOLD, SPACE, SPACE, buf, -1);
+    PsPrintString(interp, channel, FONT_BOLD, enc, SPACE, SPACE, buf, -1);
     yPos -= fontsize*1.1;
 
 }
@@ -916,7 +941,8 @@ PrintBody(Tcl_Interp *interp, Tcl_Channel channel, BodyInfo *bodyInfoPtr)
  */
 
 static int
-PrintBodyText(Tcl_Interp *interp, Tcl_Channel channel, BodyInfo *bodyInfoPtr)
+PrintBodyText(Tcl_Interp *interp, Tcl_Channel channel, Tcl_Encoding enc,
+	      BodyInfo *bodyInfoPtr)
 {
     Tcl_Obj *oPtr;
     char *cPtr, *nPtr;
@@ -929,7 +955,7 @@ PrintBodyText(Tcl_Interp *interp, Tcl_Channel channel, BodyInfo *bodyInfoPtr)
 	    nPtr = cPtr + strlen(cPtr)-1;
 	}
 	CHECK_NEWPAGE(interp, channel);
-	PsPrintString(interp, channel, FONT_NORMAL, SPACE, SPACE, cPtr,
+	PsPrintString(interp, channel, FONT_NORMAL, enc, SPACE, SPACE, cPtr,
 		nPtr-cPtr);
 	yPos -= fontsize*1.1;
 	cPtr = nPtr+1;
