@@ -3,7 +3,7 @@
  *
  *	This file contains password caching routines
  *
- * TkRat software and its included text is Copyright 1996-2000 by
+ * TkRat software and its included text is Copyright 1996-2002 by
  * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
@@ -20,11 +20,8 @@
  */
 typedef struct CachedPasswd {
     int onDisk;
-    char *host;
-    unsigned int port;
-    char *user;
-    char *service;
-    char *password;
+    char *spec;
+    char *passwd;
     struct CachedPasswd *next;
     Tcl_TimerToken token;
 } CachedPasswd;
@@ -35,11 +32,51 @@ static char *filename = NULL;
 /*
  * Local functions
  */
+static char *Canonify(const char *spec);
 static void ReadDisk(Tcl_Interp *interp);
 static void WriteDisk(Tcl_Interp *interp);
 static void TouchEntry(Tcl_Interp *interp, CachedPasswd *cp);
 static Tcl_TimerProc ErasePasswd;
 
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Canonify --
+ *
+ *      Convert foler specification to canonical form
+ *
+ * Results:
+ *	A pointer to a static buffer containing the canonic form.
+ *	This buffer will be rewritten by the next call.
+ *
+ * Side effects:
+ *	None.
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+
+static char*
+Canonify(const char *spec)
+{
+    static char *cSpec = NULL;
+    static int size = 0;
+    char *c;
+
+    if (strlen(spec)+1 > size) {
+	size = strlen(spec)+64;
+	cSpec = (char*)realloc(cSpec, size);
+    }
+    strlcpy(cSpec, spec, size);
+    if (NULL != (c = strstr(cSpec, "/debug"))) {
+	memmove(c, c+6, strlen(c+6)+1);
+    }
+    if (NULL != (c = strchr(cSpec, '}'))) {
+	c[1] = '\0';
+    }
+    return cSpec;
+}
 
 /*
  *----------------------------------------------------------------------
@@ -61,39 +98,45 @@ static Tcl_TimerProc ErasePasswd;
 void
 ReadDisk(Tcl_Interp *interp)
 {
-    char *name, **argv;
+    CONST84 char **argv, *spec = NULL, *passwd = NULL;
+    const char *name;
     CachedPasswd *cp;
     char buf[1024];
     int argc;
     FILE *fp;
-    Tcl_DString exp;
 
-    name = Tcl_GetVar2(interp, "option", "pwcache_file", TCL_GLOBAL_ONLY);
-    if (NULL == name) {
+    if (NULL == (name = RatGetPathOption(interp, "pwcache_file"))) {
 	return;
     }
-    Tcl_TranslateFileName(interp, name, &exp);
-    filename = cpystr(Tcl_DStringValue(&exp));
-    Tcl_DStringFree(&exp);
+    filename = cpystr(name);
     initialized = 1;
     if (NULL == (fp = fopen(filename, "r"))) {
 	return;
     }
     while (fgets(buf, sizeof(buf), fp), !feof(fp)) {
-	if (TCL_OK != Tcl_SplitList(interp, buf, &argc, &argv) || 5 != argc) {
+	if (TCL_OK != Tcl_SplitList(interp, buf, &argc, &argv)
+	    || (argc != 2 && argc != 5)) {
 	    continue;
+	}	
+	if (2 == argc) {
+	    /* {spec passwd} */
+	    spec = argv[0];
+	    passwd = argv[1];
+	} else if (5 == argc) {
+	    /* {host port user service passwd} */
+	    snprintf(buf, sizeof(buf), "{%s:%s/user=%s%s}",
+		     argv[0], argv[1], argv[2],
+		     (strcmp("imap", argv[3]) ? "/pop3" : ""));
+	    spec = buf;
+	    passwd = argv[4];
 	}
-	cp = (CachedPasswd*)ckalloc(sizeof(CachedPasswd)+strlen(buf));
+	cp = (CachedPasswd*)ckalloc(sizeof(CachedPasswd)
+				    +strlen(spec)+1+strlen(passwd)+1);
 	cp->onDisk = 1;
-	cp->host = (char*)cp + sizeof(CachedPasswd);
-	strcpy(cp->host, argv[0]);
-	cp->port = atoi(argv[1]);
-	cp->user = cp->host + strlen(cp->host)+1;
-	strcpy(cp->user, argv[2]);
-	cp->service = cp->user + strlen(cp->user)+1;
-	strcpy(cp->service, argv[3]);
-	cp->password = cp->service + strlen(cp->service)+1;
-	strcpy(cp->password, argv[4]);
+	cp->spec = (char*)cp + sizeof(CachedPasswd);
+	strcpy(cp->spec, spec);
+	cp->passwd = cp->spec + strlen(cp->spec)+1;
+	strcpy(cp->passwd, passwd);
 	cp->next = cache;
 	cache = cp;
 	ckfree(argv);
@@ -125,7 +168,7 @@ void
 WriteDisk(Tcl_Interp *interp)
 {
     CachedPasswd *cp;
-    char c, buf[64];
+    char c;
     FILE *fp;
     struct stat sbuf;
     int i, fd;
@@ -147,12 +190,8 @@ WriteDisk(Tcl_Interp *interp)
     Tcl_DStringInit(&ds);
     for (cp = cache; cp; cp = cp->next) {
 	if (cp->onDisk) {
-	    Tcl_DStringAppendElement(&ds, cp->host);
-	    sprintf(buf, "%d", cp->port);
-	    Tcl_DStringAppendElement(&ds, buf);
-	    Tcl_DStringAppendElement(&ds, cp->user);
-	    Tcl_DStringAppendElement(&ds, cp->service);
-	    Tcl_DStringAppendElement(&ds, cp->password);
+	    Tcl_DStringAppendElement(&ds, cp->spec);
+	    Tcl_DStringAppendElement(&ds, cp->passwd);
 	    fprintf(fp, "%s\n", Tcl_DStringValue(&ds));
 	    Tcl_DStringSetLength(&ds, 0);
 	}
@@ -185,18 +224,18 @@ void
 TouchEntry(Tcl_Interp *interp, CachedPasswd *cp)
 {
     int timeout;
+    Tcl_Obj *oPtr;
 
     if (cp->onDisk) {
 	return;
     }
-    if (cp->token) {
-	Tcl_DeleteTimerHandler(cp->token);
-    }
-    Tcl_GetInt(interp, Tcl_GetVar2(interp, "option", "cache_passwd_timeout",
-	       TCL_GLOBAL_ONLY), &timeout);
+    Tcl_DeleteTimerHandler(cp->token);
+    oPtr = Tcl_GetVar2Ex(interp, "option", "cache_passwd_timeout",
+			 TCL_GLOBAL_ONLY);
+    Tcl_GetIntFromObj(interp, oPtr, &timeout);
     if (timeout) {
 	cp->token =
-		Tcl_CreateTimerHandler(timeout*1000,ErasePasswd,(ClientData)cp);
+	    Tcl_CreateTimerHandler(timeout*1000, ErasePasswd, (ClientData)cp);
     }
 }
 
@@ -223,7 +262,8 @@ ErasePasswd(ClientData clientData)
 {
     CachedPasswd **tcp, *cp = (CachedPasswd*)clientData;
 
-    memset(cp->password, 0, strlen(cp->password));
+    Tcl_DeleteTimerHandler(cp->token);
+    memset(cp->passwd, 0, strlen(cp->passwd));
     for (tcp = &cache; *tcp != cp; tcp = &(*tcp)->next);
     *tcp = cp->next;
     ckfree(cp);
@@ -249,28 +289,18 @@ ErasePasswd(ClientData clientData)
  */
 
 char*
-RatGetCachedPassword(Tcl_Interp *interp, const char *host, unsigned int port,
-	const char *user, const char *service)
+RatGetCachedPassword(Tcl_Interp *interp, const char *spec)
 {
     CachedPasswd *cp;
+    char *cSpec = Canonify(spec);
 
     if (0 == initialized) {
 	ReadDisk(interp);
     }
-    if (port == 0) {
-	if (!strcmp(service, "pop3")) {
-	    port = 110;
-	} else if (!strcmp(service, "imap")) {
-	    port = 143;
-	}
-    }
     for (cp = cache; cp; cp = cp->next) {
-	if (!strcmp(cp->host, host)
-		&& cp->port == port
-		&& !strcmp(cp->user, user)
-		&& !strcmp(cp->service, service)) {
+	if (!strcmp(cp->spec, cSpec)) {
 	    TouchEntry(interp, cp);
-	    return cp->password;
+	    return cp->passwd;
 	}
     }
     return NULL;
@@ -295,26 +325,22 @@ RatGetCachedPassword(Tcl_Interp *interp, const char *host, unsigned int port,
  */
 
 void
-RatCachePassword(Tcl_Interp *interp, const char *host, unsigned int port,
-	const char *user, const char *service, const char *passwd, int store)
+RatCachePassword(Tcl_Interp *interp, const char *spec, const char *passwd,
+		 int store)
 {
     CachedPasswd *cp;
+    char *cSpec = Canonify(spec);
 
     if (0 == initialized) {
 	ReadDisk(interp);
     }
-    cp = (CachedPasswd*)ckalloc(sizeof(CachedPasswd)+strlen(host)+strlen(user)+
-	    strlen(service)+strlen(passwd)+4);
+    cp = (CachedPasswd*)ckalloc(sizeof(CachedPasswd) +
+				strlen(cSpec) + 1 + strlen(passwd) + 1);
     cp->onDisk = store;
-    cp->host = (char*)cp + sizeof(CachedPasswd);
-    strcpy(cp->host, host);
-    cp->port = port;
-    cp->user = cp->host+strlen(host)+1;
-    strcpy(cp->user, user);
-    cp->service = cp->user+strlen(user)+1;
-    strcpy(cp->service, service);
-    cp->password = cp->service+strlen(service)+1;
-    strcpy(cp->password, passwd);
+    cp->spec = (char*)cp + sizeof(CachedPasswd);
+    strcpy(cp->spec, cSpec);
+    cp->passwd = cp->spec+strlen(cSpec)+1;
+    strcpy(cp->passwd, passwd);
     cp->next = cache;
     cp->token = NULL;
     cache = cp;
@@ -356,10 +382,8 @@ RatPasswdCachePurge(Tcl_Interp *interp, int disk_also)
     }
     for (cp = cache; cp; cp = cpn) {
 	cpn = cp->next;
-	memset(cp->password, 0, strlen(cp->password));
-	if (cp->token) {
-	    Tcl_DeleteTimerHandler(cp->token);
-	}
+	memset(cp->passwd, 0, strlen(cp->passwd));
+	Tcl_DeleteTimerHandler(cp->token);
 	ckfree(cp);
     }
     cache = NULL;
