@@ -3,7 +3,7 @@
  *
  *	This file contains basic support for sending messages via SMTP.
  *
- * TkRat software and its included text is Copyright 1996-2000 by
+ * TkRat software and its included text is Copyright 1996-2002 by
  * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
@@ -22,10 +22,21 @@ typedef struct SMTPChannelPriv {
     unsigned int dsn : 1;	/* True if the peer supports DSN */
 } SMTPChannelPriv;
 
+/*
+ * Linked list of cached channels
+ */
+typedef struct ChannelCache {
+    SMTPChannelPriv *chPtr;
+    char *host;
+    int port;
+    struct ChannelCache *next;
+} ChannelCache;
+static ChannelCache *channelCache = NULL;
+ 
 /*
  * Local functions
  */
-static char *RatTimedGets(Tcl_Interp *interp, Tcl_Channel channel, int timeout);
+static char *RatTimedGets(Tcl_Interp *interp, Tcl_Channel channel,int timeout);
 static int RatSendCommand(Tcl_Interp *interp, Tcl_Channel channel, char *cmd);
 static int RatSendRcpt(Tcl_Interp *interp, Tcl_Channel channel,
 	ADDRESS *adrPtr, DSNhandle handle, int verbose);
@@ -97,14 +108,15 @@ int
 RatSendCommand(Tcl_Interp *interp, Tcl_Channel channel, char *cmd)
 {
     int result, timeout;
+    Tcl_Obj *oPtr;
     char *reply;
 
     Tcl_Write(channel, cmd, -1);
     if ('\n' != cmd[strlen(cmd)-1]) {
 	Tcl_Write(channel, "\r\n", -1);
     }
-    Tcl_GetInt(interp, Tcl_GetVar2(interp, "option", "smtp_timeout",
-	    TCL_GLOBAL_ONLY), &timeout);
+    oPtr = Tcl_GetVar2Ex(interp, "option", "smtp_timeout", TCL_GLOBAL_ONLY);
+    Tcl_GetIntFromObj(interp, oPtr, &timeout);
     do {
 	reply = RatTimedGets(interp, channel, timeout);
 	if (reply) {
@@ -163,7 +175,7 @@ RatSendRcpt(Tcl_Interp *interp, Tcl_Channel channel, ADDRESS *adrPtr,
 	    snprintf(buf+strlen(buf), sizeof(buf)-strlen(buf),
 		    " ORCPT=rfc822;");
 	    for (i = strlen(buf),cPtr = (unsigned char*)adr; *cPtr; cPtr++) {
-		if (*cPtr < 33 || *cPtr > 126 || *cPtr == '+' || *cPtr == '=') {
+		if (*cPtr < 33 || *cPtr > 126 || *cPtr == '+' || *cPtr == '='){
 		    buf[i++] = '+';
 		    buf[i++] = alphabetHEX[*cPtr>>4];
 		    buf[i++] = alphabetHEX[*cPtr&0xf];
@@ -201,13 +213,15 @@ RatSendRcpt(Tcl_Interp *interp, Tcl_Channel channel, ADDRESS *adrPtr,
  */
 
 SMTPChannel
-RatSMTPOpen (Tcl_Interp *interp, char *host, int verbose)
+RatSMTPOpen (Tcl_Interp *interp, char *host, int verbose, const char *role)
 {
     SMTPChannelPriv *chPtr;
-    char *reply, buf[1024], *cPtr;
+    char *reply, buf[1024], *cPtr, *ch;
     int port, timeout;
+    ChannelCache *cachePtr;
+    Tcl_Obj *oPtr;
 
-    RatStrNCpy(buf, host, sizeof(buf));
+    strlcpy(buf, host, sizeof(buf));
     if ((cPtr = strchr(buf, ':'))) {
 	*cPtr++ = '\0';
 	port = atoi(cPtr);
@@ -215,6 +229,16 @@ RatSMTPOpen (Tcl_Interp *interp, char *host, int verbose)
 	port = 25;	/* The default SMTP port */
     }
 
+    for (cachePtr = channelCache; cachePtr; cachePtr = cachePtr->next) {
+	if (!strcmp(cachePtr->host, buf) && cachePtr->port == port) {
+	    if (TCL_OK == RatSendCommand(interp, cachePtr->chPtr->channel,
+					 "RSET")) {
+		return cachePtr->chPtr;
+	    }
+	    break;
+	}
+    }
+
     if (verbose > 1) {
 	RatLogF(interp, RAT_PARSE, "opening_connection", RATLOG_EXPLICIT);
     }
@@ -227,7 +251,7 @@ RatSMTPOpen (Tcl_Interp *interp, char *host, int verbose)
      * releases of tcl7.5/tk4.1 and reconfigure tkrat (this MUST be done).
      * After rerunning configure you may build tkrat.
      */
-    if (NULL == (chPtr->channel = Tcl_OpenTcpClient(interp,port,buf,NULL,0,0))){
+    if (NULL == (chPtr->channel =Tcl_OpenTcpClient(interp,port,buf,NULL,0,0))){
 	ckfree(chPtr);
 	return NULL;
     }
@@ -240,8 +264,8 @@ RatSMTPOpen (Tcl_Interp *interp, char *host, int verbose)
     if (verbose > 1) {
 	RatLogF(interp, RAT_PARSE, "wait_greeting", RATLOG_EXPLICIT);
     }
-    Tcl_GetInt(interp, Tcl_GetVar2(interp, "option", "smtp_timeout",
-	    TCL_GLOBAL_ONLY), &timeout);
+    oPtr = Tcl_GetVar2Ex(interp, "option", "smtp_timeout", TCL_GLOBAL_ONLY);
+    Tcl_GetIntFromObj(interp, oPtr, &timeout);
     do {
 	reply = RatTimedGets(interp, chPtr->channel, timeout);
 	if (!reply || '2' != reply[0]) {
@@ -257,11 +281,12 @@ RatSMTPOpen (Tcl_Interp *interp, char *host, int verbose)
     if (verbose > 1) {
 	RatLogF(interp, RAT_PARSE, "get_capabilities", RATLOG_EXPLICIT);
     }
-    snprintf(buf, sizeof(buf), "EHLO %s\r\n", currentHost);
+    ch = RatGetCurrent(interp, RAT_HOST, role);
+    snprintf(buf, sizeof(buf), "EHLO %s\r\n", ch);
     Tcl_Write(chPtr->channel, buf, -1);
     reply = RatTimedGets(interp, chPtr->channel, timeout);
     if (!reply || '2' != reply[0]) {
-	snprintf(buf, sizeof(buf), "HELO %s.%s\r\n", currentHost, currentHost);
+	snprintf(buf, sizeof(buf), "HELO %s\r\n", ch);
 	Tcl_Write(chPtr->channel, buf, -1);
 	reply = RatTimedGets(interp, chPtr->channel, timeout);
     }
@@ -285,6 +310,15 @@ RatSMTPOpen (Tcl_Interp *interp, char *host, int verbose)
     if (verbose > 1) {
 	RatLog(interp, RAT_PARSE, "", RATLOG_EXPLICIT);
     }
+
+    cachePtr = (ChannelCache*)ckalloc(sizeof(ChannelCache)+strlen(host)+1);
+    cachePtr->chPtr = chPtr;
+    cachePtr->host = (char*)cachePtr + sizeof(*cachePtr);
+    strlcpy(cachePtr->host, host, strlen(host));
+    cachePtr->port = port;
+    cachePtr->next = channelCache;
+    channelCache = cachePtr;
+    
     return (SMTPChannel)chPtr;
 }
 
@@ -309,7 +343,11 @@ void
 RatSMTPClose (Tcl_Interp *interp, SMTPChannel channel, int verbose)
 {
     SMTPChannelPriv *chPtr = (SMTPChannelPriv*)channel;
+    ChannelCache **c1Ptr, *c2Ptr;
 
+    /*
+     * Close connection
+     */
     if (verbose > 1) {
 	RatLogF(interp, RAT_PARSE, "closing", RATLOG_EXPLICIT);
     }
@@ -319,6 +357,42 @@ RatSMTPClose (Tcl_Interp *interp, SMTPChannel channel, int verbose)
 	RatLog(interp, RAT_PARSE, "", RATLOG_EXPLICIT);
     }
     ckfree(chPtr);
+
+    /*
+     * Clear cache
+     */
+    for (c1Ptr = &channelCache; *c1Ptr && (*c1Ptr)->chPtr != chPtr;
+	 c1Ptr = &(*c1Ptr)->next);
+    if (*c1Ptr) {
+	c2Ptr = (*c1Ptr)->next;
+	ckfree(*c1Ptr);
+	*c1Ptr = c2Ptr;
+    }
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * RatSMTPClose --
+ *
+ *      Close an SMTP channel.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	THe chanel is closed.
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+RatSMTPCloseAll (Tcl_Interp *interp, int verbose)
+{
+    while (channelCache) {
+	RatSMTPClose(interp, channelCache->chPtr, verbose);
+    }
 }
 
 /*
@@ -353,10 +427,10 @@ RatSMTPSend (Tcl_Interp *interp, SMTPChannel channel, ENVELOPE *envPtr,
      */
     if (!(envPtr->to || envPtr->cc || envPtr->bcc)) {
 	Tcl_SetResult(interp, "No recipients specified", TCL_STATIC);
-	return TCL_ERROR;
+	goto abort;
     }
     if (TCL_OK != RatSendCommand(interp, chPtr->channel, "RSET")) {
-	return TCL_ERROR;
+	goto abort;
     }
 
     /*
@@ -377,31 +451,31 @@ RatSMTPSend (Tcl_Interp *interp, SMTPChannel channel, ENVELOPE *envPtr,
 	    RatLogF(interp, RAT_PARSE, "send_from", RATLOG_EXPLICIT);
 	}
     }
-    if (RatAddressSize(envPtr->return_path, 0) > sizeof(buf)-128) {
+    if (RatAddressSize(envPtr->from, 0) > sizeof(buf)-128) {
 	RatLogF(interp, RAT_WARN, "ridiculously_long", RATLOG_TIME);
-	return TCL_ERROR;
+	goto abort;
     }
     snprintf(buf, sizeof(buf), "MAIL FROM:<");
-    rfc822_address(buf, envPtr->return_path);
-    strcat(buf, ">");
+    rfc822_address(buf, envPtr->from);
+    strlcat(buf, ">", sizeof(buf));
     if (chPtr->mime) {
-	strcat(buf, " BODY=8BITMIME");
+	strlcat(buf, " BODY=8BITMIME", sizeof(buf));
     }
     if (doDSN) {
 	RatGenId(NULL, interp, 0, NULL);
 	handle = RatDSNStartMessage(interp, Tcl_GetStringResult(interp),
-		envPtr->subject);
-	strcat(buf, " ENVID=");
-	strcat(buf, Tcl_GetStringResult(interp));
+				    envPtr->subject);
+	strlcat(buf, " ENVID=", sizeof(buf));
+	strlcat(buf, Tcl_GetStringResult(interp), sizeof(buf));
     }
     if (TCL_OK != RatSendCommand(interp, chPtr->channel, buf)) {
-	return TCL_ERROR;
+	goto abort;
     }
-    failures += RatSendRcpt(interp, chPtr->channel, envPtr->to, handle,verbose);
-    failures += RatSendRcpt(interp, chPtr->channel, envPtr->cc, handle,verbose);
-    failures += RatSendRcpt(interp, chPtr->channel, envPtr->bcc,handle,verbose);
+    failures += RatSendRcpt(interp, chPtr->channel,envPtr->to,handle,verbose);
+    failures += RatSendRcpt(interp, chPtr->channel,envPtr->cc,handle,verbose);
+    failures += RatSendRcpt(interp, chPtr->channel,envPtr->bcc,handle,verbose);
     if (failures) {
-	goto error;
+	goto abort;
     }
 
     /*
@@ -411,7 +485,7 @@ RatSMTPSend (Tcl_Interp *interp, SMTPChannel channel, ENVELOPE *envPtr,
 	RatLogF(interp, RAT_PARSE, "send_data", RATLOG_EXPLICIT);
     }
     if (TCL_OK != RatSendCommand(interp, chPtr->channel, "DATA")) {
-	goto error;
+	goto abort;
     }
     header = (char*)ckalloc(RatHeaderSize(envPtr, bodyPtr));
     rfc822_output(header, envPtr, bodyPtr, RatTclPutsSMTP, chPtr->channel,
@@ -421,7 +495,7 @@ RatSMTPSend (Tcl_Interp *interp, SMTPChannel channel, ENVELOPE *envPtr,
 	RatLogF(interp, RAT_PARSE, "wait_ack", RATLOG_EXPLICIT);
     }
     if (TCL_OK != RatSendCommand(interp, chPtr->channel, ".")) {
-	goto error;
+	goto abort;
     }
     if (handle) {
 	RatDSNFinish(interp, handle);
@@ -429,9 +503,8 @@ RatSMTPSend (Tcl_Interp *interp, SMTPChannel channel, ENVELOPE *envPtr,
 
     return TCL_OK;
 
-error:
+ abort:
     RatDSNAbort(interp, handle);
-    RatSendCommand(interp, chPtr->channel, "RSET");
     return TCL_ERROR;
 }
 
@@ -454,21 +527,25 @@ error:
  */
 
 int
-RatSMTPSupportDSN(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
+RatSMTPSupportDSN(ClientData dummy, Tcl_Interp *interp, int objc,
+		  Tcl_Obj *const objv[])
 {
     SMTPChannelPriv *chPrivPtr;
     SMTPChannel channel;
     int verbose, result;
 
-    if (argc != 2) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		" hostname\"", (char *) NULL);
+    if (objc != 2) {
+	Tcl_AppendResult(interp, "wrong # args: should be \"",
+			 Tcl_GetString(objv[0]), " hostname\"",
+			 (char *) NULL);
 	return TCL_ERROR;
     }
 
-    Tcl_GetInt(interp, Tcl_GetVar2(interp, "option", "smtp_verbose",
-	    TCL_GLOBAL_ONLY), &verbose);
-    channel = RatSMTPOpen(interp, argv[1], verbose);
+    Tcl_GetIntFromObj(interp,
+		      Tcl_GetVar2Ex(interp, "option", "smtp_verbose",
+				    TCL_GLOBAL_ONLY),
+		      &verbose);
+    channel = RatSMTPOpen(interp, Tcl_GetString(objv[1]), verbose, "");
     if (channel) {
 	chPrivPtr = (SMTPChannelPriv*)channel;
 	result = chPrivPtr->dsn;
