@@ -413,6 +413,7 @@ RatStdFolderCreate(Tcl_Interp *interp, Tcl_Obj *defPtr)
     MAILSTREAM *stream = NULL;
     char buf[32];
     Tcl_Obj *oPtr;
+    char *spec;
     int i;
 
     /*
@@ -428,8 +429,8 @@ RatStdFolderCreate(Tcl_Interp *interp, Tcl_Obj *defPtr)
     stdPtr->handlers.exists = Std_HandleExists;
     stdPtr->handlers.expunged = Std_HandleExpunged;
 
-    stream = OpenStdFolder(interp, RatGetFolderSpec(interp,defPtr), stdPtr);
-    if (!stream) {
+    if (NULL == (spec = RatGetFolderSpec(interp, defPtr))
+	|| NULL == (stream = OpenStdFolder(interp, spec, stdPtr))) {
 	ckfree(stdPtr);
 	return NULL;
     }
