@@ -3,7 +3,7 @@
  *
  *      Declarations of functions used in the Std folder and messages
  *
- * TkRat software and its included text is Copyright 1996-2000 by
+ * TkRat software and its included text is Copyright 1996-2002 by
  * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
@@ -27,9 +27,22 @@ typedef enum {
     RAT_DIS
 } RatStdFolderType;
 
-MAILSTREAM *Std_StreamOpen(Tcl_Interp *interp, char *name, char *user,
-	long options, int *errorFlagPtr);
+/*
+ * Here we handle the events which may come from the mail-server
+ * via parts of the mm interface.
+ */
+typedef void (HandleExists)(void *state, unsigned long nmsgs);
+typedef void (HandleExpunged)(void *state, unsigned long index);
+typedef struct {
+    void *state;
+    HandleExists *exists;
+    HandleExpunged *expunged;
+} FolderHandlers;
+ 
+MAILSTREAM *Std_StreamOpen(Tcl_Interp *interp, char *name, long options,
+			   int *errorFlagPtr, FolderHandlers *handlers);
 void Std_StreamClose(Tcl_Interp *interp, MAILSTREAM *stream);
+void Std_StreamCloseAllCached(Tcl_Interp *interp);
 RatCreateProc Std_CreateProc;
 RatGetHeadersProc Std_GetHeadersProc;
 RatGetEnvelopeProc Std_GetEnvelopeProc;
@@ -58,6 +71,11 @@ extern int searchResultNum;
  */
 extern MAILSTATUS stdStatus;
 
+/*
+ * Controls if we should ignore loging calls or not
+ */
+extern int logIgnore;
+
 /*
  * This is the private part of a std folder info structure.
  */
@@ -67,19 +85,16 @@ typedef struct StdFolderInfo {
     int referenceCount;		/* Number of entities referencing this entry */
     int exists;			/* Number of messages which actually exists
 				   in this folder */
-    int isNotYet;		/* True if the mailbox file doesn't exist yet */
-    char *origName;		/* The original name of this folder */
+    int error;                  /* Error status */
     RatStdFolderType type;	/* The exact type of this folder */
-    char *host;			/* The host it resides on (if networked) */
-    char *user;			/* The username (if networked) */
+    FolderHandlers handlers;	/* The event handlers */
 } StdFolderInfo;
 
 /* ratStdMessage.c */
 extern void RatStdMsgStructInit(RatFolderInfoPtr infoPtr, Tcl_Interp *interp,
-	int index, MAILSTREAM *stream, RatStdFolderType type, char *host,
-	char *user);
+				int index, MAILSTREAM *stream,
+				RatStdFolderType type);
 extern char *RatStdMessageCreate (Tcl_Interp *interp, RatFolderInfoPtr infoPtr,
-	MAILSTREAM *stream, int msgNo, RatStdFolderType type, char *host,
-	char *user);
+				  MAILSTREAM *stream, int msgNo);
 
 #endif /* _RATSTDFOLDER */
