@@ -3,7 +3,7 @@
  *
  *	This file contains code which implements standard c-client messages.
  *
- * TkRat software and its included text is Copyright 1996-2000 by
+ * TkRat software and its included text is Copyright 1996-2002 by
  * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
@@ -21,8 +21,7 @@ typedef struct StdMessageInfo {
     ENVELOPE *envPtr;
     BODY *bodyPtr;
     RatStdFolderType type;
-    char *host;
-    char *user;
+    char *spec;
 } StdMessageInfo;
 
 /*
@@ -97,20 +96,19 @@ RatStdMessagesInit(MessageProcInfo *messageProcInfoPtr)
 
 char*
 RatStdMessageCreate(Tcl_Interp *interp, RatFolderInfoPtr folderInfoPtr,
-	MAILSTREAM *stream, int msgNo, RatStdFolderType type,
-	char *host, char *user)
+		    MAILSTREAM *stream, int msgNo)
 {
     MessageInfo *msgPtr = (MessageInfo*)folderInfoPtr->privatePtr[msgNo];
     StdMessageInfo *stdMsgPtr = (StdMessageInfo*)msgPtr->clientData;
 
-    msgPtr->clientData = (ClientData)stdMsgPtr;
     stdMsgPtr->envPtr =
-	    mail_fetchstructure_full(stream, msgNo+1, &stdMsgPtr->bodyPtr, NIL);
+	mail_fetchstructure_full(stream, msgNo+1, &stdMsgPtr->bodyPtr, NIL);
     stdMsgPtr->eltPtr = mail_elt(stream, msgNo+1);
     stdMsgPtr->eltPtr->lockcount++;
+    stdMsgPtr->spec = cpystr(stream->mailbox);
     sprintf(msgPtr->name, "RatStdMsg%d", numStdMessages++);
     Tcl_CreateObjCommand(interp, msgPtr->name, RatMessageCmd,
-	    (ClientData)msgPtr, NULL);
+			 (ClientData)msgPtr, NULL);
     return msgPtr->name;
 }
 
@@ -128,34 +126,34 @@ RatStdMessageCreate(Tcl_Interp *interp, RatFolderInfoPtr folderInfoPtr,
  * Side effects:
  *	None.
  *
- *
  *----------------------------------------------------------------------
  */
 
 int
-RatStdEasyCopyingOK(MessageInfo *msgPtr, char *prot, char *destination,
-	char *user)
+RatStdEasyCopyingOK(Tcl_Interp *interp, MessageInfo *msgPtr, Tcl_Obj *defPtr)
 {
     StdMessageInfo *stdMsgPtr = (StdMessageInfo*)msgPtr->clientData;
-    int nameLen;
+    Tcl_Obj **objv;
+    int objc;
+
+    Tcl_ListObjGetElements(interp, defPtr, &objc, &objv);
 
     switch (stdMsgPtr->type) {
-	case RAT_MBX:
+    case RAT_DIS:
+	return 0;
+    case RAT_MBX:
+	return 0;
+    case RAT_UNIX:
+	return !strcasecmp(Tcl_GetString(objv[1]), "file");
+    case RAT_MH:
+	return !strcasecmp(Tcl_GetString(objv[1]), "mh");
+    case RAT_POP:
+	return 0;
+    case RAT_IMAP:
+	if (strcasecmp(Tcl_GetString(objv[1]), "imap")) {
 	    return 0;
-	case RAT_UNIX:
-	    return !strcasecmp(prot, "file");
-	case RAT_MH:
-	    return !strcasecmp(prot, "mh");
-	case RAT_POP:
-	    return 0;
-	case RAT_IMAP:
-	    if(strcasecmp(prot,"imap")) {
-		return 0;
-	    }
-	    for (nameLen=0; '}' != destination[nameLen+1]
-		    && '/' != destination[nameLen+1]; nameLen++);
-	    return (!strncasecmp(stdMsgPtr->host, &destination[1], nameLen) &&
-		    !strcasecmp(stdMsgPtr->user, user));
+	}
+	return !strcmp(stdMsgPtr->spec, RatGetFolderSpec(interp, defPtr));
     }
     return 0;
 }
@@ -189,14 +187,17 @@ RatStdMessageCopy (Tcl_Interp *interp, MessageInfo *msgPtr, char *destination)
 
     sprintf(seq, "%d", msgPtr->msgNo+1);
     if (flagged) {
-	mail_clearflag(stdMsgPtr->stream, seq, RAT_FLAGGED_STR);
+	mail_clearflag(stdMsgPtr->stream, seq,
+		       flag_name[RAT_FLAGGED].imap_name);
     }
     if (deleted) {
-	mail_clearflag(stdMsgPtr->stream, seq, RAT_DELETED_STR);
+	mail_clearflag(stdMsgPtr->stream, seq,
+		       flag_name[RAT_DELETED].imap_name);
     }
     switch (stdMsgPtr->type) {
 	case RAT_UNIX:	/* fallthrough */
 	case RAT_MBX:	/* fallthrough */
+	case RAT_DIS:	/* fallthrough */
 	case RAT_MH:	/* fallthrough */
 	case RAT_POP:	/* fallthrough */
 	    if (T == mail_copy_full(stdMsgPtr->stream, seq, destination, 0)) {
@@ -211,10 +212,10 @@ RatStdMessageCopy (Tcl_Interp *interp, MessageInfo *msgPtr, char *destination)
 	    break;
     }
     if (flagged) {
-	mail_setflag(stdMsgPtr->stream, seq, RAT_FLAGGED_STR);
+	mail_setflag(stdMsgPtr->stream, seq, flag_name[RAT_FLAGGED].imap_name);
     }
     if (deleted) {
-	mail_setflag(stdMsgPtr->stream, seq, RAT_DELETED_STR);
+	mail_setflag(stdMsgPtr->stream, seq, flag_name[RAT_DELETED].imap_name);
     }
     return r;
 }
@@ -292,14 +293,11 @@ Std_GetEnvelopeProc(Tcl_Interp *interp, MessageInfo *msgPtr)
     } else {
 	adrPtr = stdMsgPtr->envPtr->from;
     }
-    if (adrPtr && adrPtr->host && !strcmp(currentHost, adrPtr->host)
-	    && strlen(adrPtr->mailbox) < sizeof(buf)-6) {
-	snprintf(buf, sizeof(buf), "From %s", adrPtr->mailbox);
-    } else if (adrPtr && RatAddressSize(adrPtr, 0) < sizeof(buf)-6) {
-	sprintf(buf, "From ");
+    if (adrPtr && RatAddressSize(adrPtr, 0) < sizeof(buf)-6) {
+	strlcpy(buf, "From ", sizeof(buf));
 	rfc822_address(buf+5, adrPtr);
     } else {
-	sprintf(buf, "From unkown");
+	strlcpy(buf, "From unkown", sizeof(buf));
     }
     tm.tm_sec = stdMsgPtr->eltPtr->seconds;
     tm.tm_min = stdMsgPtr->eltPtr->minutes;
@@ -314,7 +312,7 @@ Std_GetEnvelopeProc(Tcl_Interp *interp, MessageInfo *msgPtr)
     tmPtr = gmtime(&date);
     sprintf(buf + strlen(buf), " %s %s %2d %02d:%02d GMT %04d\n",
 	    dayName[tmPtr->tm_wday], monthName[tmPtr->tm_mon],
-	    tmPtr->tm_mday, tmPtr->tm_hour, tmPtr->tm_min, tmPtr->tm_year+1900);
+	    tmPtr->tm_mday, tmPtr->tm_hour, tmPtr->tm_min,tmPtr->tm_year+1900);
     return buf;
 }
 
@@ -399,6 +397,7 @@ Std_MsgDeleteProc(MessageInfo *msgPtr)
 
     infoPtr->privatePtr[msgPtr->msgNo] = NULL;
     stdMsgPtr->eltPtr->lockcount--;
+    ckfree(stdMsgPtr->spec);
     ckfree(stdMsgPtr);
 }
 
@@ -422,6 +421,7 @@ Std_MakeChildrenProc(Tcl_Interp *interp, BodyInfo *bodyInfoPtr)
     StdBodyInfo *partStdInfoPtr;
     int index = 1;
     PART *partPtr;
+    int size;
 
     if (!bodyInfoPtr->firstbornPtr) {
 	partInfoPtrPtr = &bodyInfoPtr->firstbornPtr;
@@ -435,11 +435,10 @@ Std_MakeChildrenProc(Tcl_Interp *interp, BodyInfo *bodyInfoPtr)
 	    partInfoPtr->msgPtr = bodyInfoPtr->msgPtr;
 	    partInfoPtr->clientData = (ClientData)partStdInfoPtr;
 	    if (stdBodyInfoPtr->section) {
-		partStdInfoPtr->section = (char*)ckalloc(
-			strlen(stdBodyInfoPtr->section)+8);
-		snprintf(partStdInfoPtr->section,
-			sizeof(partStdInfoPtr->section), "%s.%d",
-			stdBodyInfoPtr->section, index++);
+		size = strlen(stdBodyInfoPtr->section) + 8;
+		partStdInfoPtr->section = (char*)ckalloc(size);
+		snprintf(partStdInfoPtr->section, size, "%s.%d",
+			 stdBodyInfoPtr->section, index++);
 	    } else {
 		partStdInfoPtr->section = (char*)ckalloc(8);
 		sprintf(partStdInfoPtr->section, "%d", index++);
@@ -462,14 +461,14 @@ Std_MakeChildrenProc(Tcl_Interp *interp, BodyInfo *bodyInfoPtr)
 char*
 Std_FetchBodyProc(BodyInfo *bodyInfoPtr, unsigned long *lengthPtr)
 {
-    StdMessageInfo *stdMsgPtr =(StdMessageInfo*)bodyInfoPtr->msgPtr->clientData;
+    StdMessageInfo *stdMsgPtr=(StdMessageInfo*)bodyInfoPtr->msgPtr->clientData;
 
     if (bodyInfoPtr->decodedTextPtr) {
 	*lengthPtr = Tcl_DStringLength(bodyInfoPtr->decodedTextPtr);
 	return Tcl_DStringValue(bodyInfoPtr->decodedTextPtr);
     }
     return mail_fetchbody_full(stdMsgPtr->stream, bodyInfoPtr->msgPtr->msgNo+1,
-	    ((StdBodyInfo*)(bodyInfoPtr->clientData))->section, lengthPtr, NIL);
+	    ((StdBodyInfo*)(bodyInfoPtr->clientData))->section, lengthPtr,NIL);
 }
 
 
@@ -487,9 +486,7 @@ void
 Std_BodyDeleteProc(BodyInfo *bodyInfoPtr)
 {
     StdBodyInfo *partStdInfoPtr = (StdBodyInfo*)bodyInfoPtr->clientData;
-    if (partStdInfoPtr->section) {
-	ckfree(partStdInfoPtr->section);
-    }
+    ckfree(partStdInfoPtr->section);
     ckfree(bodyInfoPtr->clientData);
 }
 
@@ -594,7 +591,7 @@ Std_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
 			    stdMsgPtr->stream, msgPtr->msgNo+1,
 			    &stdMsgPtr->bodyPtr, NIL);
 		}
-		oPtr =Tcl_NewStringObj(body_types[stdMsgPtr->bodyPtr->type],-1);
+		oPtr=Tcl_NewStringObj(body_types[stdMsgPtr->bodyPtr->type],-1);
 		Tcl_AppendStringsToObj(oPtr, "/",
 				       stdMsgPtr->bodyPtr->subtype, NULL);
 	    }
@@ -672,7 +669,7 @@ Std_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
  */
 void
 RatStdMsgStructInit(RatFolderInfoPtr infoPtr, Tcl_Interp *interp, int index,
-	MAILSTREAM *stream, RatStdFolderType type, char *host, char *user)
+	MAILSTREAM *stream, RatStdFolderType type)
 {
     StdMessageInfo *stdMsgPtr;
     int i, start, end;
@@ -694,8 +691,7 @@ RatStdMsgStructInit(RatFolderInfoPtr infoPtr, Tcl_Interp *interp, int index,
 	stdMsgPtr->envPtr = mail_fetch_structure(stream, i+1, NIL, NIL);
 	stdMsgPtr->bodyPtr = NULL;
 	stdMsgPtr->type = type;
-	stdMsgPtr->host = host;
-	stdMsgPtr->user = user;
+        stdMsgPtr->spec = NULL;
 	((MessageInfo*)infoPtr->privatePtr[i])->clientData =
 		(ClientData)stdMsgPtr;
     }
@@ -703,8 +699,6 @@ RatStdMsgStructInit(RatFolderInfoPtr infoPtr, Tcl_Interp *interp, int index,
 #ifdef MEM_DEBUG
 void ratStdMessageCleanup()
 {
-    if (mem_header) {
-	ckfree(mem_header);
-    }
+    ckfree(mem_header);
 }
 #endif /* MEM_DEBUG */
