@@ -3,7 +3,7 @@
 # See README for information about what this program does.
 #
 #
-#  TkRat software and its included text is Copyright 1996-2000 by
+#  TkRat software and its included text is Copyright 1996-2002 by
 #  Martin Forssn
 #
 #  The full text of the legal notice is contained in the file called
@@ -129,10 +129,10 @@ foreach f [glob *.text] {
 	puts $fh "proc init_${varName}_${l} {} {"
 	puts $fh "global $varName"
 	foreach n $labels {
-	    if [info exists text($l,$n)] {
+	    if {[info exists text($l,$n)]} {
 		set rl $l
 	    } else {
-		if $warn {
+		if {$warn} {
 		    puts "Text $n not found in [lindex $lang 1] substituting English"
 		}
 		set rl en
