@@ -5,7 +5,7 @@
 # Usage: tclsh merge.tcl file_to_merge_into lang file_to_merge_from
 #
 #
-#  TkRat software and its included text is Copyright 1996-2000 by
+#  TkRat software and its included text is Copyright 1996-2002 by
 #  Martin Forssn
 #
 #  The full text of the legal notice is contained in the file called
@@ -41,7 +41,7 @@ if {-1 == [lsearch -exact $langs $sourcelang]} {
     puts "Unkown language $sourcelang, perhaps you need to modify defs.tcl"
     exit 1
 }
-if ![file readable $sourcefile] {
+if {![file readable $sourcefile]} {
     puts "Can't open $sourcefile for reading"
     exit 1
 }
@@ -82,7 +82,7 @@ source $basefile
 #
 # Write output
 #
-if [catch {open $basefile w} fh] {
+if {[catch {open $basefile w} fh]} {
     puts "Failed to open $basefile for writing: $fh"
     exit 1
 }
@@ -93,7 +93,7 @@ puts $fh ""
 foreach lab $labels {
     puts $fh "label $lab"
     foreach lang $langs {
-	if ![info exists t($lang,$lab)] {
+	if {![info exists t($lang,$lab)]} {
 	    puts "$lang: missing $lab"
 	} else {
 	    puts $fh [list $lang $t($lang,$lab)]
