@@ -3,7 +3,7 @@
 # This file contains code for handling message selection expressions
 #
 #
-#  TkRat software and its included text is Copyright 1996-2000 by
+#  TkRat software and its included text is Copyright 1996-2002 by
 #  Martin Forssn
 #
 #  The full text of my legal notices is contained in the file called
@@ -81,7 +81,7 @@ proc ExpCreate {handler {addproc {}}} {
 
     set hd(mode) {}
     set hd(action) create
-    if [string compare advanced $option(expression_mode)] {
+    if {[string compare advanced $option(expression_mode)]} {
 	ExpModeBasic $id
     } else {
 	ExpModeAdv $id
@@ -138,7 +138,7 @@ proc ExpModeBasic {handler} {
     global t b
 
     # Check that this is really neccessary
-    if ![string compare $hd(mode) basic] {
+    if {![string compare $hd(mode) basic]} {
 	return
     }
 
@@ -183,7 +183,7 @@ proc ExpModeAdv {handler} {
     global t b
 
     # Check that this is really neccessary
-    if ![string compare $hd(mode) advanced] {
+    if {![string compare $hd(mode) advanced]} {
 	return
     }
 
@@ -281,7 +281,7 @@ proc ExpModeAdv {handler} {
 proc ExpClear {handler} {
     upvar #0 $handler hd
 
-    if ![string compare basic $hd(mode)] {
+    if {![string compare basic $hd(mode)]} {
 	foreach f {subject from reply-to sender to cc} {
 	    set hd($f) ""
 	}
@@ -303,32 +303,32 @@ proc ExpDone {handler action} {
     upvar #0 $hd(handler) fHd
     global t b option expList expExp
 
-    if ![info exist expList] {
+    if {![info exist expList]} {
 	set expList {}
     }
     if {1 == $action} {
 	# Build expression
-	if ![string compare basic $hd(mode)] {
+	if {![string compare basic $hd(mode)]} {
 	    set exp ""
 	    foreach f {subject from reply-to sender to cc} {
-		if [string length $hd($f)] {
-		    if [string length $exp] {
+		if {[string length $hd($f)]} {
+		    if {[string length $exp]} {
 			set exp "$exp $hd(op) "
 		    }
-		    if [string compare reply-to $f] {
+		    if {[string compare reply-to $f]} {
 			set exp "${exp}$f has [list $hd($f)]"
 		    } else {
 			set exp "${exp}reply-to has [list $hd($f)]"
 		    }
 		}
 	    }
-	    if [catch {RatParseExp $exp} expId] {
+	    if {[catch {RatParseExp $exp} expId]} {
 		Popup $t(syntax_error_exp) $hd(w)
 		return
 	    }
 	} else {
 	    set exp [string trim [$hd(text) get 1.0 end]]
-	    if [catch {RatParseExp $exp} expId] {
+	    if {[catch {RatParseExp $exp} expId]} {
 	        set i [lindex $expId 0]
 	        $hd(text) tag add error "1.$i wordstart" "1.$i wordend"
 		Popup "$t(error_underlined): [lindex $expId 1]" $hd(w)
@@ -336,8 +336,8 @@ proc ExpDone {handler action} {
 	   }
 	}
 	if {"create" == $hd(action)} {
-	    if $hd(doSave) {
-		if [string length $hd(saveAs)] {
+	    if {$hd(doSave)} {
+		if {[string length $hd(saveAs)]} {
 		    if {-1 != [lsearch -exact $expList $hd(saveAs)]} {
 			Popup $t(name_occupied) $hd(w)
 			return
@@ -355,7 +355,7 @@ proc ExpDone {handler action} {
 	    }
 	    # Apply expression
 	    set ids [$fHd(folder_handler) match $expId]
-	    if [string length $ids] {
+	    if {[string length $ids]} {
 		SetFlag $hd(handler) flagged 1 $ids
 	    }
 
@@ -381,7 +381,7 @@ proc ExpDone {handler action} {
     catch {focus $hd(oldfocus)}
     destroy $hd(w)
     foreach bn [array names b $hd(w).*] {unset b($bn)}
-    if [string compare $hd(mode) $option(expression_mode)] {
+    if {[string compare $hd(mode) $option(expression_mode)]} {
 	set option(expression_mode) $hd(mode)
 	SaveOptions
     }
@@ -414,7 +414,7 @@ proc ExpWrite {} {
 proc ExpRead {} {
     global option expList expExp
 
-    if [file readable $option(ratatosk_dir)/expressions] {
+    if {[file readable $option(ratatosk_dir)/expressions]} {
 	source $option(ratatosk_dir)/expressions
     }
 }
@@ -433,7 +433,7 @@ proc ExpBuildMenu {m handler} {
 
     $m delete 0 end
 
-    if ![info exists expList] {
+    if {![info exists expList]} {
 	return
     }
     foreach i $expList {
@@ -456,7 +456,7 @@ proc ExpMenuApply {id handler} {
 
     set expId [RatParseExp $expExp($id)]
     set ids [$hd(folder_handler) match $expId]
-    if [string length $ids] {
+    if {[string length $ids]} {
 	SetFlag $handler flagged 1 $ids
     }
     RatFreeExp $expId
