@@ -1,5 +1,5 @@
 #
-#  TkRat software and its included text is Copyright 1996-2000 by
+#  TkRat software and its included text is Copyright 1996-2002 by
 #  Martin Forssn
 #
 #  The full text of the legal notices is contained in the file called
@@ -8,12 +8,13 @@
 proc TkRatStart {} {
     global t tkrat_version tkrat_version_date idCnt inbox expAfter logAfter \
 	   statusBacklog currentColor ratLogBottom ratLogTop vFolderDef \
-	   option currentLanguage_t tk_patchLevel vFolderInbox folderWindowList\
-	   openFolders folderChanged propNormFont propLightFont fixedNormFont \
-	   fixedBoldFont watcherFont ISO_Left_Tab tk_version folderUnseen
+	   option currentLanguage_t tk_patchLevel vFolderInbox \
+	   folderWindowList openFolders folderChanged propNormFont \
+	   propLightFont fixedNormFont fixedBoldFont watcherFont \
+	   ISO_Left_Tab tk_version rat_tmp tklead folderUnseen
 
     # Base package requirements
-    package require ratatosk 2.0
+    package require ratatosk 2.1
 
     # Function to let client know we have started
     proc RatPing {} {
@@ -21,8 +22,8 @@ proc TkRatStart {} {
     }
 
     # Initialize variables
-    set tkrat_version 2.0.3
-    set tkrat_version_date 20010903
+    set tkrat_version 2.1.4
+    set tkrat_version_date 20020809
     set idCnt 0
     set inbox ""
     set expAfter {}
@@ -34,7 +35,7 @@ proc TkRatStart {} {
 
     # Bindings
     bind Entry <Return> {focus [tk_focusNext %W]}
-    if [catch {bind all <ISO_Left_Tab> {focus [tk_focusPrev %W]}}] {
+    if {[catch {bind all <ISO_Left_Tab> {focus [tk_focusPrev %W]}}]} {
 	set ISO_Left_Tab Shift-Tab
     } else {
 	set ISO_Left_Tab ISO_Left_Tab
@@ -51,6 +52,11 @@ proc TkRatStart {} {
 	tk useinputmethods $option(useinputmethods)
     }
 
+    # Reinitialize language (if needed)
+    if {[string compare $option(language) $currentLanguage_t]} {
+	InitMessages $option(language) t
+    }
+
     if {[info exists option(last_version_date)] 
 	    && "$option(last_version_date)" != $tkrat_version_date} {
 	NewVersionUpdate
@@ -62,7 +68,7 @@ proc TkRatStart {} {
     set fixedNormFont [RatCreateFont $option(fixed_norm)]
     set fixedBoldFont [RatCreateFont $option(fixed_bold)]
     set watcherFont [RatCreateFont $option(watcher_font)]
-    if $option(override_fonts) {
+    if {$option(override_fonts)} {
 	set pri interactive
     } else {
 	set pri widgetDefault
@@ -72,11 +78,14 @@ proc TkRatStart {} {
     option add *TkRat*Text.font $fixedNormFont $pri
     option add *TkRat*Listbox.font $fixedNormFont $pri
     option add *TkRat*RatList*Listbox.font $propNormFont $pri
+    option add *TkRat*RatTree*font $propLightFont $pri
 
     option add *Menu.tearOff $option(tearoff) widgetDefault
 
+    bind Menubutton <Up> {event generate %W <space>}
+    bind Menubutton <Down> {event generate %W <space>}
+
     # Extra package requirements
-    package require rat_tabbed 1.0
     package require rat_list 1.0
     package require rat_fbox 1.1
     package require rat_balloon 1.0
@@ -85,37 +94,35 @@ proc TkRatStart {} {
     package require blt_busy 1.0
     package require rat_ed 1.0
     package require rat_ispell 1.0
+    package require rat_tree 1.0
+    package require rat_enriched 1.0
 
     # Change the color
-    if $option(override_color) {
+    if {$option(override_color)} {
 	option add *TkRat*foreground black interactive
 	option add *TkRat*background gray85 interactive
 	eval "SetColor $option(color_set)"
     }
 
-    # Reinitialize language (if needed)
-    if [string compare $option(language) $currentLanguage_t] {
-	InitMessages $option(language) t
-    }
-
     # Make sure our config directory exists
-    if ![file isdirectory $option(ratatosk_dir)] {
+    if {![file isdirectory $option(ratatosk_dir)]} {
 	set but [RatDialog "" $t(need_tkrat_dir_title) \
 	       "$t(need_tkrat_dir1) \"$option(ratatosk_dir)\". \
 	       $t(need_tkrat_dir2)" {} 0 $t(create) $t(dont_create) $t(abort)]
 	switch $but {
-	0 {
-	       catch "exec mkdir [RatTildeSubst $option(ratatosk_dir)]" result
-	       if [string length $result] {
-		   Popup [concat "$t(failed_create) \"$option(ratatosk_dir)\":"\
-				 "$result.\n$t(do_without_dir)"]
-	       }
-	  }
-	1 {
-	       Popup $t(do_without_dir)
-	       set option(send_cache) $option(tmp)/send.$env(USER)
-	  }
-	2 {exit 0}
+	    0 {
+		catch "exec mkdir [RatTildeSubst $option(ratatosk_dir)]" result
+		if {[string length $result]} {
+		    Popup [concat \
+			    "$t(failed_create) \"$option(ratatosk_dir)\":"\
+			    "$result.\n$t(do_without_dir)"]
+		}
+	    }
+	    1 {
+		Popup $t(do_without_dir)
+		set option(send_cache) $rat_tmp/send.$env(USER)
+	    }
+	    2 {exit 0}
 	}
     }
     if {![string length $option(last_version)]
@@ -137,7 +144,7 @@ proc TkRatStart {} {
     }
     ReadUserproc
     ReadPos
-    if [file readable $option(ratatosk_dir)/expressions] {
+    if {[file readable $option(ratatosk_dir)/expressions]} {
 	ExpRead
     }
 
@@ -149,21 +156,29 @@ proc TkRatStart {} {
     set openFolders {}
     trace variable folderWindowList wu RatTraceFolder
 
-    # Redo bindings for entry and text to make the selection work more
-    # intuitive
-    bind Entry <1> { 
-	tkEntryButton1 %W %x
-    }
-    bind Text <1> {
-	tkTextButton1 %W %x %y
+    # Setup online status
+    switch $option(start_online_mode) {
+	online { set option(online) 1 }
+	offline { set option(online) 0 }
+	default {}
     }
 
-    if { 0 <= [expr [RatDaysSinceExpire]-$option(expire_interval)]} {
+    # Redo bindings for entry and text to make the selection work more
+    # intuitive
+    if {$tk_version >= 8.4} {
+	set tklead "tk::"
+    } else {
+	set tklead "tk"
+    }
+    bind Entry <1> "${tklead}EntryButton1 %W %x"
+    bind Text <1> "${tklead}TextButton1 %W %x %y"
+
+    if { 0 <= [expr {[RatDaysSinceExpire]-$option(expire_interval)}]} {
 	catch {Expire} err
     } else {
-	set expAfter [after [expr \
-		($option(expire_interval)-[RatDaysSinceExpire])*24*60*60*1000] \
-		Expire]
+	set expAfter \
+		[after [expr {($option(expire_interval)- \
+		[RatDaysSinceExpire])*24*60*60*1000}] Expire]
     }
 
     # Check for deferred mail
@@ -188,8 +203,8 @@ proc TkRatStart {} {
 proc RatCreateFont {s} {
     if {"components" == [lindex $s 0]} {
 	set res [list [lindex $s 1] -[lindex $s 2] [lindex $s 3] [lindex $s 4]]
-	if [lindex $s 5] { lappend res underline }
-	if [lindex $s 6] { lappend res overstrike }
+	if {[lindex $s 5]} { lappend res underline }
+	if {[lindex $s 6]} { lappend res overstrike }
 	return $res
     } else {
 	return [lindex $s 1]
@@ -213,34 +228,34 @@ proc RatLog {level message {mode time}} {
     }
     set ratLog($ratLogTop) [format "%-8s %s" $n $message]
     incr ratLogTop
-    if {$ratLogTop > [expr $ratLogBottom+$option(num_messages)]} {
-	for {} {$ratLogTop > [expr $ratLogBottom+$option(num_messages)]} \
+    if {$ratLogTop > [expr {$ratLogBottom+$option(num_messages)}]} {
+	for {} {$ratLogTop > [expr {$ratLogBottom+$option(num_messages)}]} \
 		{incr ratLogBottom} {
 	    unset ratLog($ratLogBottom)
 	}
     }
     if { 3 < $level} {
 	# Fatal
-	if ![string compare nowait $mode] {
+	if {![string compare nowait $mode]} {
 	    after 1 Popup [list $message]
 	} else {
 	    Popup $message
 	}
     } else {
-	if $level {
-	    if ![string compare explicit $mode] {
-		if [string length $logAfter] {
+	if {$level} {
+	    if {![string compare explicit $mode]} {
+		if {[string length $logAfter]} {
 		    after cancel $logAfter
 		    set statusBacklog {}
 		}
 		set statusText $message
 		set statusId $ratLogTop
 	    } else {
-		if [string length $logAfter] {
+		if {[string length $logAfter]} {
 		    lappend statusBacklog $message
 		} else {
 		    set statusText $message
-		    set logAfter [after [expr $option(log_timeout)*1000] \
+		    set logAfter [after [expr {$option(log_timeout)*1000}] \
 					RatLogAfter]
 		    set statusId ""
 		}
@@ -260,10 +275,10 @@ proc RatLog {level message {mode time}} {
 proc RatLogAfter {} {
     global statusText logAfter statusBacklog option statusId
 
-    if [llength $statusBacklog] {
+    if {[llength $statusBacklog]} {
 	set statusText [lindex $statusBacklog 0]
 	set statusBacklog [lrange $statusBacklog 1 end]
-	set logAfter [after [expr $option(log_timeout)*1000] RatLogAfter]
+	set logAfter [after [expr {$option(log_timeout)*1000}] RatLogAfter]
     } else {
 	set statusText ""
 	set logAfter {}
@@ -282,7 +297,7 @@ proc RatLogAfter {} {
 proc RatClearLog {id} {
     global statusText statusId
 
-    if ![string compare $id $statusId] {
+    if {![string compare $id $statusId]} {
 	set statusId ""
 	set statusText ""
     }
@@ -343,7 +358,7 @@ proc RatBind {w keylist function {menu {}}} {
     global option
 
     foreach k $option($keylist) {
-	if [info exists a] {
+	if {[info exists a]} {
 	    if {[string length $k] < [string length $a]} {
 		set a $k
 	    }
@@ -356,7 +371,7 @@ proc RatBind {w keylist function {menu {}}} {
 	bind $w $k $function
     }
     if {[string length $menu]} {
-	if [info exists a] {
+	if {[info exists a]} {
 	    regsub Key- [string trim $a <>] {} key
 	    if {0 != [regexp Shift- $key]} {
 		set l [split $key -]
@@ -400,7 +415,7 @@ proc MailSteal {handler ask} {
     set fh [open $option(ms_netscape_pref_file) r]
     while {0 == [eof $fh]} {
 	gets $fh line
-	if ![string compare MAIL_DIR: [lindex $line 0]] {
+	if {![string compare MAIL_DIR: [lindex $line 0]]} {
 	    set dir [RatTildeSubst [lindex $line 1]]
 	    break
 	}
@@ -416,14 +431,14 @@ proc MailSteal {handler ask} {
 				0 $t(steal_back) $t(nothing)] } {
 	}
 	if {0 == $ask} {
-	    set f [RatOpenFolder std {} $dir/Inbox]
+	    set f [RatOpenFolder [list Netscape file {} $dir/Inbox]]
 	    set max [lindex [$f info] 1]
 	    for {set i 0} {$i < $max} {incr i} {
 		$inbox insert [$f get $i]
 		$f setFlag $i deleted 1
 	    }
 	    $f close
-	    Sync $handler update
+	    RatBusy {Sync $handler update}
 	}
 	set option(ms_netscape_mtime) [file mtime $dir/Inbox]
 	SaveOptions
@@ -494,19 +509,19 @@ proc SetIcon {w icon} {
     global env
 
     switch $icon {
-    normal {
-	    if [file readable $env(LIBDIR)/tkrat.xbm] {
+	normal {
+	    if {[file readable $env(LIBDIR)/tkrat.xbm]} {
 		wm iconbitmap $w @$env(LIBDIR)/tkrat.xbm
 		wm iconmask $w @$env(LIBDIR)/tkratmask.xbm
 	    }
 	}
-    small {
-	    if [file readable $env(LIBDIR)/tkrat_small.xbm] {
+	small {
+	    if {[file readable $env(LIBDIR)/tkrat_small.xbm]} {
 		wm iconbitmap $w @$env(LIBDIR)/tkrat_small.xbm
 		wm iconmask $w @$env(LIBDIR)/tkrat_smallmask.xbm
 	    }
 	}
-    none {
+	none {
 	    wm iconbitmap $w ""
 	    wm iconmask $w ""
 	}
@@ -528,9 +543,9 @@ proc FixMenu {m} {
 	global t
 
 	# Calculate breakpoint. We assue all entries are of uniform height
-	set i [expr ([$m index last]*$height)/[$m yposition last]-1]
+	set i [expr {([$m index last]*$height)/[$m yposition last]-1}]
 	$m insert $i cascade -label $t(more) -menu $m.m
-	if ![winfo exists $m.m] {
+	if {![winfo exists $m.m]} {
 	    menu $m.m -postcommand "FixMenu $m.m"
 	} else {
 	    $m.m delete 1 end
@@ -568,7 +583,7 @@ proc AliasRead {} {
     global option aliasBook
 
     set as $option(addrbooks)
-    if $option(use_system_aliases) {
+    if {$option(use_system_aliases)} {
 	lappend as $option(system_aliases)
     }
     foreach a $as {
@@ -576,31 +591,40 @@ proc AliasRead {} {
 	set aliasBook(changed,$book) 0
 	switch [lindex $a 1] {
 	    tkrat {
-		    set f [lindex $a 2]
-		    if [file readable $f] {
-			RatAlias read $f
-		    }
-		    set dir [file dirname $f]
-		    if {([file isfile $f] && [file writable $f])
-			    || (![file exists $f] && [file isdirectory $dir]
-				&& [file writable $dir])} {
-			set aliasBook(writable,$book) 1
-		    } else {
-			set aliasBook(writable,$book) 0
-		    }
+		set f [lindex $a 2]
+		if {[file readable $f]} {
+		    catch {RatAlias read $f}
 		}
+		set dir [file dirname $f]
+		if {([file isfile $f] && [file writable $f])
+		|| (![file exists $f] && [file isdirectory $dir]
+		&& [file writable $dir])} {
+		    set aliasBook(writable,$book) 1
+		} else {
+		    set aliasBook(writable,$book) 0
+		}
+	    }
 	    mail {
-		    ReadMailAliases [lindex $a 2] $book
-		    set aliasBook(writable,$book) 0
+		set f [lindex $a 2]
+		if {[file readable $f]} {
+		    ReadMailAliases $f $book
 		}
+		set aliasBook(writable,$book) 0
+	    }
 	    elm  {
-		    ReadElmAliases [lindex $a 2] $book
-		    set aliasBook(writable,$book) 0
+		set f [lindex $a 2]
+		if {[file readable $f]} {
+		    ReadElmAliases $f $book
 		}
+		set aliasBook(writable,$book) 0
+	    }
 	    pine {
-		    ReadPineAliases [lindex $a 2] $book
-		    set aliasBook(writable,$book) 0
+		set f [lindex $a 2]
+		if {[file readable $f]} {
+		    ReadPineAliases $f $book
 		}
+		set aliasBook(writable,$book) 0
+	    }
 	}
     }
 }
@@ -625,7 +649,7 @@ proc FindAccelerators {var ids} {
 	set tot [string length $t($id)]
 	set sub [string length [string trimleft $t($id) $used]]
 	if {$sub > 0} {
-	    set result($id) [expr $tot - $sub]
+	    set result($id) [expr {$tot - $sub}]
 	    set used ${used}[string index $t($id) $result($id)]
 	} else {
 	    set result($id) -1
@@ -668,7 +692,7 @@ proc RatExec {cmds} {
 		    global folderWindowList idCnt
 
 		    if {"open" == [lindex $cmd 0]
-			    && [llength [array names folderWindowList]]} {
+			    && [array size folderWindowList]} {
 			set handler [lindex [array names folderWindowList] 0]
 		    } else {
 			global idCnt option vFolderDef vFolderInbox
@@ -684,12 +708,12 @@ proc RatExec {cmds} {
 			if {"" == $arg} {
 			    set arg [lindex $vFolderDef($vFolderInbox) 0]
 			}
-			set handler [FolderWindowInit $w $arg]
 			if {$option(iconic)} {
 			    wm iconify $w
 			} else {
 			    wm deiconify $w
 			}
+			set handler [FolderWindowInit $w $arg]
 		    }
 		    return [RatExecOpen $handler $arg]
 		}
@@ -707,12 +731,12 @@ proc RatExec {cmds} {
 		    if {"" == $arg} {
 			set arg [lindex $vFolderDef($vFolderInbox) 0]
 		    }
-		    set handler [FolderWindowInit $w $arg]
 		    if {$option(iconic)} {
 			wm iconify $w
 		    } else {
 			wm deiconify $w
 		    }
+		    set handler [FolderWindowInit $w $arg]
 		    FolderWindowClear $handler
 		}
 	    compose {
@@ -750,7 +774,7 @@ proc RatExecOpen {handler name} {
 		break
 	    }
 	}
-	if ![info exists spec] {
+	if {![info exists spec]} {
 	    error "No such folder '$name'"
 	}
 	set check 0
@@ -776,7 +800,7 @@ proc RatExecNetsync {what} {
     global option
 
     set old $option(network_sync)
-    if [llength $what] {
+    if {[llength $what]} {
 	set ns_send 0
 	set ns_fetch 0
 	set ns_cmd 0
@@ -790,7 +814,7 @@ proc RatExecNetsync {what} {
 	}
 	set option(network_sync) [list $ns_send $ns_fetch $ns_cmd]
     }
-    NetworkSync
+    RatBusy {NetworkSync}
     set option(network_sync) $old
 }
 
@@ -806,11 +830,11 @@ proc RatExecNetsync {what} {
 
 proc Browse {w var mode} {
     global env t
-    upvar #0 $var file
+    upvar #0 $var filein
 
-    if {"" != $file} {
-	set dir [file dirname $file]
-	set file [file tail $file]
+    if {"" != $filein} {
+	set dir [file dirname $filein]
+	set file [file tail $filein]
     } else {
 	set dir $env(HOME)
 	set file ""
@@ -819,35 +843,30 @@ proc Browse {w var mode} {
     set r [rat_fbox::run -parent $w -initialdir $dir -initialfile $file \
 	    -title $t(select_file) -ok $t(ok) -mode $mode]
     if {"" != $r} {
-	set file $r
+	uplevel #0 [list set $var $r]
     }
 }
 
-# Busy --
+
+# IsExecutable --
 #
-# Run a command and mark tkrat as busy in the meantime
+# Checks if a given command matches any executable file (by searching path
+# if needed).
 #
 # Arguments:
-# cmd	- Command to run
+# cmd	- Command to check
 
-proc Busy {cmd} {
-    global errorInfo
+proc IsExecutable {cmd} {
+    global env
 
-    set children [winfo children .]
-    foreach c $children {
-	blt_busy hold $c
-    }
-    update idletasks
-
-    set error [catch {uplevel eval [list $cmd]} result]
-    set ei $errorInfo
-
-    foreach c $children {
-	blt_busy release $c
+    if {[regexp / $cmd]} {
+	return [file executable $cmd]
     }
 
-    if {$error} {
-	error $result $ei
+    foreach p [split $env(PATH) :] {
+	if {[file executable $p/$cmd]} {
+	    return 1
+	}
     }
-    return $result
+    return 0
 }
