@@ -1,5 +1,5 @@
 #
-#  TkRat software and its included text is Copyright 1996-2000 by
+#  TkRat software and its included text is Copyright 1996-2002 by
 #  Martin Forssn
 #
 #  The full text of the legal notices is contained in the file called
@@ -26,10 +26,10 @@ proc Place {w id} {
 	default {
 		wm withdraw $w
 		update idletasks
-		set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
-			- [winfo vrootx [winfo parent $w]]]
-		set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
-			- [winfo vrooty [winfo parent $w]]]
+		set x [expr {[winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
+			- [winfo vrootx [winfo parent $w]]}]
+		set y [expr {[winfo screenheight $w]/2-[winfo reqheight $w]/2 \
+			- [winfo vrooty [winfo parent $w]]}]
 		set ratPlaceO($id) +$x+$y
 	    }
 	}
@@ -56,7 +56,7 @@ proc RecordPos {w id} {
     } else {
 	return
     }
-    if ![winfo exists $w] {
+    if {![winfo exists $w]} {
 	return
     }
 
@@ -83,14 +83,14 @@ proc RecordPos {w id} {
 	incr y -$height
     }
     set geom "[lindex $geom 0]${x}[lindex $geom 2]${y}"
-    if [info exists ratPlaceO($id)] {
-	if [string compare $ratPlaceO($id) $geom] {
+    if {[info exists ratPlaceO($id)]} {
+	if {[string compare $ratPlaceO($id) $geom]} {
 	    set ratPlace($id) $geom
 	    set ratPlaceModified 1
 	}
 	unset ratPlaceO($id)
-    } elseif [info exists ratPlace($id)] {
-	if [string compare $ratPlace($id) $geom] {
+    } elseif {[info exists ratPlace($id)]} {
+	if {[string compare $ratPlace($id) $geom]} {
 	    set ratPlace($id) $geom
 	    set ratPlaceModified 1
 	}
@@ -112,7 +112,7 @@ proc Size {w id} {
     global ratSize ratSizeO ratSizeP ratSizeB option
 
     if {[info exists ratSize($id)] && $option(keep_pos)
-	    && 0 != [lindex $ratSize($id) 0] && 0 != [lindex $ratSize($id) 1]} {
+    && 0 != [lindex $ratSize($id) 0] && 0 != [lindex $ratSize($id) 1]} {
 	set width [lindex $ratSize($id) 0]
 	set height [lindex $ratSize($id) 1]
     } else {
@@ -122,7 +122,7 @@ proc Size {w id} {
 	compose		{set width  80; set height  24}
 	source		{set width  80; set height  30}
 	seeLog		{set width  80; set height  20}
-	vFolderDef	{set width 450; set height 600}
+	vFolderDef	{set width 600; set height 540}
 	gFolderL 	{set width  80; set height  10}
 	expList		{set width  30; set height  10}
 	giveCmd		{set width  80; set height   4}
@@ -140,9 +140,11 @@ proc Size {w id} {
 	bookList	{set width  20; set height  10}
 	subjlist	{set width  20; set height   9}
 	helptext	{set width  80; set height  40}
-	showGH		{set width  60; set height  10}
+	showGH		{set width  80; set height  10}
 	editorList	{set width  30; set height  10}
 	bugText		{set width  80; set height  20}
+	prefTree	{set width 120; set height 200}
+	prefPane        {set width 400; set height 250}
 	}
 	set ratSizeO($id) [list $width $height]
     }
@@ -150,11 +152,11 @@ proc Size {w id} {
     # Remembering information of this window:
     #  ratSizeB	-    the size of the total borders in one axis (in pixels)
     #  ratSizeP -    the size of each character
-    set bd [expr 2*([$w cget -borderwidth] \
-		    +[$w cget -highlightthickness])]
+    set bd [expr {2*([$w cget -borderwidth] \
+	    +[$w cget -highlightthickness])}]
     set ratSizeB($id) $bd
-    set ratSizeP($id) [list [expr ([winfo reqwidth $w]-$bd)/$width] \
-		            [expr ([winfo reqheight $w]-$bd)/$height]]
+    set ratSizeP($id) [list [expr {([winfo reqwidth $w]-$bd)/$width}] \
+	    [expr {([winfo reqheight $w]-$bd)/$height}]]
     if {0 == [lindex $ratSizeP($id) 0] || 0 == [lindex $ratSizeP($id) 1]} {
 	set ratSizeP($id) [list 1 1]
     }
@@ -177,26 +179,28 @@ proc RecordSize {w id} {
 	return
     }
 
-    if ![winfo exists $w] {
+    if {![winfo exists $w]} {
 	return
     }
 
     set val [list \
-	    [expr ([winfo width $w]-$ratSizeB($id))/[lindex $ratSizeP($id) 0]] \
-    	    [expr ([winfo height $w]-$ratSizeB($id))/[lindex $ratSizeP($id) 1]]]
+	    [expr {([winfo width $w]-$ratSizeB($id))/ \
+	    [lindex $ratSizeP($id) 0]}] \
+    	    [expr {([winfo height $w]-$ratSizeB($id))/ \
+	    [lindex $ratSizeP($id) 1]}]]
 
     # Ignore unmapped windows
     if { 0 >= [lindex $val 0] || 0 >= [lindex $val 1]} {
 	return
     }
-    if [info exists ratSizeO($id)] {
-	if [string compare $ratSizeO($id) $val] {
+    if {[info exists ratSizeO($id)]} {
+	if {[string compare $ratSizeO($id) $val]} {
 	    set ratSize($id) $val
 	    set ratPlaceModified 1
 	}
 	unset ratSizeO($id)
-    } elseif [info exists ratSize($id)] {
-	if [string compare $ratSize($id) $val] {
+    } elseif {[info exists ratSize($id)]} {
+	if {[string compare $ratSize($id) $val]} {
 	    set ratSize($id) $val
 	    set ratPlaceModified 1
 	}
@@ -216,14 +220,14 @@ proc RecordSize {w id} {
 proc GetPane {id} {
     global ratPane
 
-    if [info exists ratPane($id)] {
-	return $ratPane($id)
-    } else {
+    if {![info exists ratPane($id)]} {
 	switch $id {
-	folderPane {set p 0.35}
+	    folderPane {set p 0.35}
+	    vFolderPane {set p 0.35}
 	}
 	set ratPane($id) $p
-    }
+    } 
+    return $ratPane($id)   
 }
 
 # RecordPane --
@@ -259,7 +263,7 @@ proc SavePos {} {
 
     if { 0 != $ratPlaceModified} {
 	# Just return on errors
-	if [catch {open $option(placement) w} f] {
+	if {[catch {open $option(placement) w} f]} {
 	    return
 	}
 	foreach p [array names ratPlace] {
@@ -285,7 +289,7 @@ proc SavePos {} {
 proc ReadPos {} {
     global option ratPlace ratSize ratPane ratPlaceModified
 
-    if [file readable $option(placement)] {
+    if {[file readable $option(placement)]} {
 	source $option(placement)
     }
     set ratPlaceModified 0
@@ -307,15 +311,19 @@ proc ModalGrab {w {newFocus {}}} {
 	set unfocus "catch {focus -force $oldFocus}"
     }
 
-    set oldGrab [grab current $w]
-    if {$oldGrab != ""} {
-	set grabStatus [grab status $oldGrab]
+
+    set ungrab ""
+    catch {
+	set oldGrab [grab current $w]
+	if {$oldGrab != ""} {
+	    set grabStatus [grab status $oldGrab]
+	}
+	tkwait visibility $w
+	if {"" != $newFocus} {
+	    focus -force $newFocus
+	}
+	grab $w
     }
-    tkwait visibility $w
-    if {"" != $newFocus} {
-	focus $newFocus
-    }
-    grab $w
 
     if {$oldGrab != ""} {
 	if {$grabStatus == "global"} {
@@ -323,8 +331,6 @@ proc ModalGrab {w {newFocus {}}} {
 	} else {
 	    set ungrab "grab $oldGrab"
 	}
-    } else {
-	set ungrab ""
     }
 
     bind $w <Destroy> \
