@@ -1,3 +1,4 @@
+
 /*
  * bltWindow.c --
  *
@@ -32,6 +33,8 @@
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
 
+#include "blt.h"
+
 struct TkIdStack;
 struct TkErrorHandler;
 struct TkSelectionInfo;
@@ -48,6 +51,16 @@ typedef struct TkGrabEvent TkGrabEvent;
 typedef struct TkColormap TkColormap;
 typedef struct TkStressedCmap TkStressedCmap;
 
+/*
+ * This defines whether we should try to use XIM over-the-spot style
+ * input.  Allow users to override it.  It is a much more elegant use
+ * of XIM, but uses a bit more memory.
+ */
+
+#ifndef TK_XIM_SPOT
+#   define TK_XIM_SPOT	1
+#endif
+
 /*
  * One of the following structures is maintained for each display
  * containing a window managed by Tk.  In part, the structure is 
@@ -336,6 +349,9 @@ typedef struct TkDisplay {
     Atom applicationAtom;	/* Atom for TK_APPLICATION. */
     Atom windowAtom;		/* Atom for TK_WINDOW. */
     Atom clipboardAtom;		/* Atom for CLIPBOARD. */
+#if TK_MINOR_VERSION >= 4
+    Atom utf8Atom;
+#endif /* 8.4 or later */
 
     Tk_Window clipWindow;	/* Window used for clipboard ownership and to
 				 * retrieve selections between processes. NULL
@@ -387,11 +403,13 @@ typedef struct TkDisplay {
      * Information used by tkUnixWm.c and tkWinWm.c only:
      */
 
+#if TK_MINOR_VERSION < 4
     int wmTracing;              /* Used to enable or disable tracing in 
 				 * this module.  If tracing is enabled, 
 				 * then information is printed on
 				 * standard output about interesting 
 				 * interactions with the window manager. */
+#endif /* TK_MINOR_VERSION < 4 */
     struct TkWmInfo *firstWmPtr;  /* Points to first top-level window. */
     struct TkWmInfo *foregroundWmPtr;    
                                 /* Points to the foreground window. */
@@ -420,7 +438,12 @@ typedef struct TkDisplay {
      * Miscellaneous information:
      */
 #ifdef TK_USE_INPUT_METHODS     
-    XIM inputMethod;            /* Input method for this display */  
+    XIM inputMethod;            /* Input method for this display */
+#if TK_MINOR_VERSION >= 4
+#if TK_XIM_SPOT
+    XFontSet inputXfs;		/* XFontSet cached for over-the-spot XIM. */
+#endif    
+#else /* < 8.4 */
 #ifdef I18N_IMPROVE              
     Tcl_Encoding imEncoding;    /* Tcl encoding when the first Tcl
                                  * interp was created.
@@ -434,6 +457,7 @@ typedef struct TkDisplay {
     XIMCallback destroyCallback;
 #endif /* XNDestroyCallback */
 #endif /* I18N_IMPROVE */
+#endif /* < 8.4 */
 #endif /* TK_USE_INPUT_METHODS */
     Tcl_HashTable winTable;	/* Maps from X window ids to TkWindow ptrs. */
 
@@ -451,6 +475,9 @@ typedef struct TkDisplay {
     int warpY;
     int useInputMethods;        /* Whether to use input methods */
 #endif /* 8.3 or later */
+#if TK_MINOR_VERSION >= 4
+    long deletionEpoch;         /* Incremented by window deletions */
+#endif /* 8.4 or later */
 } TkDisplay;
 
 struct TkWindow {
@@ -464,7 +491,7 @@ struct TkWindow {
     struct TkWindow *lastChildPtr;
     struct TkWindow *parentPtr;
     struct TkWindow *nextPtr;
-    TkMainInfo *infoPtr;
+    TkMainInfo *mainPtr;
     char *pathName;
     Tk_Uid nameUid;
     Tk_Uid classUid;
@@ -484,7 +511,7 @@ struct TkWindow {
     Tk_GeomMgr *geomMgrPtr;
     ClientData geomData;
     int reqWidth, reqHeight;
-    int internalBorderWidth;
+    int internalBorderLeft;
     TkWinInfo *wmInfoPtr;
     TkClassProcs *classProcsPtr;
     ClientData instanceData;
@@ -508,25 +535,6 @@ Blt_GetParent(display, window)
     return None;
 }
 
-static Window
-GetWindowId(tkwin)
-    Tk_Window tkwin;
-{
-    Window window;
-
-    Tk_MakeWindowExist(tkwin);
-    window = Tk_WindowId(tkwin);
-    if (Tk_IsTopLevel(tkwin)) {
-	Window parent;
-
-	parent = Blt_GetParent(Tk_Display(tkwin), window);
-	if (parent != None) {
-	    window = parent;
-	}
-    }
-    return window;
-}
-
 /*
  *----------------------------------------------------------------------
  *
